@@ -1,3 +1,4 @@
+# 2 "asmcomp/amd64/arch.ml"
 (**************************************************************************)
 (*                                                                        *)
 (*                                 OCaml                                  *)
@@ -50,8 +51,6 @@ type specific_operation =
 and float_operation =
     Ifloatadd | Ifloatsub | Ifloatmul | Ifloatdiv
 
-let spacetime_node_hole_pointer_is_live_before _specific_op = false
-
 (* Sizes, endianness *)
 
 let big_endian = false
@@ -78,13 +77,6 @@ let offset_addressing addr delta =
   | Iscaled(scale, n) -> Iscaled(scale, n + delta)
   | Iindexed2scaled(scale, n) -> Iindexed2scaled(scale, n + delta)
 
-let num_args_addressing = function
-    Ibased _ -> 0
-  | Iindexed _ -> 1
-  | Iindexed2 _ -> 2
-  | Iscaled _ -> 1
-  | Iindexed2scaled _ -> 2
-
 (* Printing operations and addressing modes *)
 
 let print_addressing printreg addr ppf arg =
@@ -136,7 +128,38 @@ let print_specific_operation printreg op ppf arg =
   | Izextend32 ->
       fprintf ppf "zextend32 %a" printreg arg.(0)
 
+(* Are we using the Windows 64-bit ABI? *)
+
 let win64 =
   match Config.system with
   | "win64" | "mingw64" | "cygwin" -> true
   | _                   -> false
+
+(* Specific operations that are pure *)
+
+let operation_is_pure = function
+  | Ilea _ | Ibswap _ | Isqrtf | Isextend32 | Izextend32 -> true
+  | Ifloatarithmem _ | Ifloatsqrtf _ -> true
+  | _ -> false
+
+(* Specific operations that can raise *)
+
+let operation_can_raise _ = false
+
+open X86_ast
+
+(* Certain float conditions aren't represented directly in the opcode for
+   float comparison, so we have to swap the arguments. The swap information
+   is also needed downstream because one of the arguments is clobbered. *)
+let float_cond_and_need_swap cond =
+  match (cond : Lambda.float_comparison) with
+  | CFeq  -> EQf,  false
+  | CFneq -> NEQf, false
+  | CFlt  -> LTf,  false
+  | CFnlt -> NLTf, false
+  | CFgt  -> LTf,  true
+  | CFngt -> NLTf, true
+  | CFle  -> LEf,  false
+  | CFnle -> NLEf, false
+  | CFge  -> LEf,  true
+  | CFnge -> NLEf, true
