@@ -1,3 +1,4 @@
+# 2 "asmcomp/arm64/CSE.ml"
 (**************************************************************************)
 (*                                                                        *)
 (*                                 OCaml                                  *)
@@ -31,7 +32,7 @@ method! class_of_operation op =
 
 method! is_cheap_operation op =
   match op with
-  | Iconst_int n -> n <= 65535n && n >= 0n
+  | Iconst_int n -> n <= 0x7FFF_FFFFn && n >= -0x8000_0000n
   | _ -> false
 
 end
