@@ -15,6 +15,11 @@
 
 (* Common functions for emitting assembly code *)
 
+type error =
+  | Stack_frame_too_large of int
+
+exception Error of error
+
 let output_channel = ref stdout
 
 let emit_string s = output_string !output_channel s
@@ -30,14 +35,17 @@ let emit_printf fmt =
 
 let emit_int32 n = emit_printf "0x%lx" n
 
-let emit_symbol esc s =
+let emit_symbol s =
   for i = 0 to String.length s - 1 do
     let c = s.[i] in
     match c with
       'A'..'Z' | 'a'..'z' | '0'..'9' | '_' ->
         output_char !output_channel c
     | _ ->
-        Printf.fprintf !output_channel "%c%02x" esc (Char.code c)
+      if c = Compilenv.symbol_separator then
+        output_char !output_channel c
+      else
+        Printf.fprintf !output_channel "$%02x" (Char.code c)
   done
 
 let emit_string_literal s =
@@ -69,7 +77,7 @@ let emit_string_directive directive s =
   end else begin
     let i = ref 0 in
     while !i < l do
-      let n = min (l - !i) 80 in
+      let n = Int.min (l - !i) 80 in
       emit_string directive;
       emit_string_literal (String.sub s !i n);
       emit_char '\n';
@@ -148,13 +156,13 @@ let emit_frames a =
       lbl
   in
   let defnames = Hashtbl.create 7 in
-  let label_defname filename defname =
+  let label_defname filename defname loc =
     try
-      snd (Hashtbl.find defnames (filename, defname))
+      snd (Hashtbl.find defnames (filename, defname, loc))
     with Not_found ->
       let file_lbl = label_filename filename in
       let def_lbl = Cmm.new_label () in
-      Hashtbl.add defnames (filename, defname) (file_lbl, def_lbl);
+      Hashtbl.add defnames (filename, defname, loc) (file_lbl, def_lbl);
       def_lbl
   in
   let module Label_table =
@@ -178,6 +186,12 @@ let emit_frames a =
       Label_table.add debuginfos key lbl;
       lbl
   in
+  let efa_16_checked n =
+    assert (n >= 0);
+    if n < 0x1_0000
+    then a.efa_16 n
+    else raise (Error(Stack_frame_too_large n))
+  in
   let emit_frame fd =
     assert (fd.fd_frame_size land 3 = 0);
     let flags =
@@ -185,15 +199,15 @@ let emit_frames a =
       | Dbg_other d | Dbg_raise d ->
         if Debuginfo.is_none d then 0 else 1
       | Dbg_alloc dbgs ->
-        if !Clflags.debug && not Config.spacetime &&
+        if !Clflags.debug &&
            List.exists (fun d ->
              not (Debuginfo.is_none d.Debuginfo.alloc_dbg)) dbgs
         then 3 else 2
     in
     a.efa_code_label fd.fd_lbl;
-    a.efa_16 (fd.fd_frame_size + flags);
-    a.efa_16 (List.length fd.fd_live_offset);
-    List.iter a.efa_16 fd.fd_live_offset;
+    efa_16_checked (fd.fd_frame_size + flags);
+    efa_16_checked (List.length fd.fd_live_offset);
+    List.iter efa_16_checked fd.fd_live_offset;
     begin match fd.fd_debuginfo with
     | _ when flags = 0 ->
       ()
@@ -227,24 +241,55 @@ let emit_frames a =
     a.efa_def_label lbl;
     a.efa_string name
   in
-  let emit_defname (_filename, defname) (file_lbl, lbl) =
+  let emit_defname (_filename, defname, loc) (file_lbl, lbl) =
+    let emit_loc (start_chr, end_chr, end_offset) =
+      a.efa_16 start_chr;
+      a.efa_16 end_chr;
+      a.efa_32 (Int32.of_int end_offset)
+    in
     (* These must be 32-bit aligned, both because they contain a
        32-bit value, and because emit_debuginfo assumes the low 2 bits
        of their addresses are 0. *)
     a.efa_align 4;
     a.efa_def_label lbl;
     a.efa_label_rel file_lbl 0l;
+    (* Include the additional 64-bits of location information which didn't pack
+       in the main 64-bit word *)
+    Option.iter emit_loc loc;
     a.efa_string defname
   in
-  let pack_info fd_raise d has_next =
-    let line = min 0xFFFFF d.Debuginfo.dinfo_line
-    and char_start = min 0xFF d.Debuginfo.dinfo_char_start
-    and char_end = min 0x3FF d.Debuginfo.dinfo_char_end
+  let fully_pack_info fd_raise d has_next =
+    (* See format in caml_debuginfo_location in runtime/backtrace-nat.c *)
+    let open Debuginfo in
+    let kind = if fd_raise then 1 else 0
+    and has_next = if has_next then 1 else 0
+    and char_end = d.dinfo_char_end + d.dinfo_start_bol - d.dinfo_end_bol in
+    let char_end_offset = d.dinfo_end_bol - d.dinfo_start_bol in
+    Int64.(add (shift_left (of_int d.dinfo_line) 51)
+             (add (shift_left (of_int (d.dinfo_end_line - d.dinfo_line)) 48)
+                (add (shift_left (of_int d.dinfo_char_start) 42)
+                   (add (shift_left (of_int char_end) 35)
+                      (add (shift_left (of_int char_end_offset) 26)
+                         (add (shift_left (of_int kind) 1)
+                            (of_int has_next)))))))
+  in
+  let partially_pack_info fd_raise d has_next =
+    (* Partially packed debuginfo:
+       1lllllllllmmmmmmmmddddddddddddkn
+         1           - d points to a name_and_loc_info struct
+         l (19 bits) - start line number
+         m (18 bits) - offset of end line number from start
+         d (24 bits) - memory offset to name_and_loc_info struct
+         k (1 bit)   - fd_raise flag
+         n (1 bit)   - has_next flag *)
+    let open Debuginfo in
+    let start_line = Int.min 0x7FFFF d.dinfo_line
+    and end_line = Int.min 0x3FFFF (d.dinfo_end_line - d.dinfo_line)
     and kind = if fd_raise then 1 else 0
     and has_next = if has_next then 1 else 0 in
-    Int64.(add (shift_left (of_int line) 44)
-             (add (shift_left (of_int char_start) 36)
-                (add (shift_left (of_int char_end) 26)
+    Int64.(add (shift_left Int64.one 63)
+             (add (shift_left (of_int start_line) 44)
+                (add (shift_left (of_int end_line) 26)
                    (add (shift_left (of_int kind) 1)
                       (of_int has_next)))))
   in
@@ -256,10 +301,31 @@ let emit_frames a =
     a.efa_def_label lbl;
     let rec emit rs d rest =
       let open Debuginfo in
-      let info = pack_info rs d (rest <> []) in
       let defname = Scoped_location.string_of_scopes d.dinfo_scopes in
+      let char_end = d.dinfo_char_end + d.dinfo_start_bol - d.dinfo_end_bol in
+      let is_fully_packable =
+        d.dinfo_line <= 0xFFF
+        && d.dinfo_end_line - d.dinfo_line <= 0x7
+        && d.dinfo_char_start <= 0x3F
+        && char_end <= 0x7F
+        && d.dinfo_end_bol - d.dinfo_start_bol <= 0x1FF
+      in
+      let info =
+        if is_fully_packable then
+          fully_pack_info rs d (rest <> [])
+        else
+          partially_pack_info rs d (rest <> [])
+      in
+      let loc =
+        if is_fully_packable then
+          None
+        else
+          Some (Int.min 0xFFFF d.dinfo_char_start,   (* start_chr *)
+                Int.min 0xFFFF char_end,             (* end_chr *)
+                Int.min 0x3FFFFFFF d.dinfo_char_end) (* end_offset *)
+      in
       a.efa_label_rel
-        (label_defname d.dinfo_file defname)
+        (label_defname d.dinfo_file defname loc)
         (Int64.to_int32 info);
       a.efa_32 (Int64.to_int32 (Int64.shift_right info 32));
       match rest with
@@ -301,12 +367,25 @@ let cfi_endproc () =
   if is_cfi_enabled () then
     emit_string "\t.cfi_endproc\n"
 
+let cfi_remember_state () =
+  if is_cfi_enabled () then
+    emit_string "\t.cfi_remember_state\n"
+
+let cfi_restore_state () =
+  if is_cfi_enabled () then
+    emit_string "\t.cfi_restore_state\n"
+
 let cfi_adjust_cfa_offset n =
   if is_cfi_enabled () then
   begin
     emit_string "\t.cfi_adjust_cfa_offset\t"; emit_int n; emit_string "\n";
   end
 
+let cfi_def_cfa_offset n =
+  if is_cfi_enabled () then begin
+    emit_string "\t.cfi_def_cfa_offset\t"; emit_int n; emit_string "\n";
+  end
+
 let cfi_offset ~reg ~offset =
   if is_cfi_enabled () then begin
     emit_string "\t.cfi_offset ";
@@ -316,6 +395,13 @@ let cfi_offset ~reg ~offset =
     emit_string "\n"
   end
 
+let cfi_def_cfa_register ~reg =
+  if is_cfi_enabled () then begin
+    emit_string "\t.cfi_def_cfa_register ";
+    emit_int reg;
+    emit_string "\n"
+  end
+
 (* Emit debug information *)
 
 (* This assoc list is expected to be very short *)
@@ -370,3 +456,43 @@ let reset () =
 
 let binary_backend_available = ref false
 let create_asm_file = ref true
+
+let report_error_doc ppf = function
+  | Stack_frame_too_large n ->
+      Format_doc.fprintf ppf "stack frame too large (%d bytes)" n
+
+let () =
+  Location.register_error_of_exn
+    (function
+      | Error err -> Some (Location.error_of_printer_file report_error_doc err)
+      | _ -> None
+    )
+
+let report_error = Format_doc.compat report_error_doc
+
+let mk_env f : Emitenv.per_function_env =
+  {
+    f;
+    stack_offset = 0;
+    call_gc_sites = [];
+    bound_error_sites = [];
+    bound_error_call = None;
+    call_gc_label = 0;
+    jumptables_lbl = None;
+    jumptables = [];
+    float_literals = [];
+    int_literals = [];
+  }
+
+let emit_named_text_section func_name prefix_char =
+  if !Clflags.function_sections then begin
+    emit_string "\t.section .text.caml.";
+    emit_symbol func_name;
+    emit_char ',';
+    emit_string_literal "ax";
+    emit_char ',';
+    emit_char prefix_char;
+    emit_string "progbits\n";
+  end
+  else
+    emit_string "\t.text\n"
