@@ -17,10 +17,16 @@ open Misc
 
 (* Link .cmo files and produce a bytecode executable. *)
 
+module Dep : Set.OrderedType with
+  type t = Cmo_format.compunit * Cmo_format.compunit
+module DepSet : Set.S with type elt = Dep.t
+
 val link : filepath list -> filepath -> unit
 val reset : unit -> unit
 
 val check_consistency: filepath -> Cmo_format.compilation_unit -> unit
+val linkdeps_unit :
+  Linkdeps.t -> filename:string -> Cmo_format.compilation_unit -> unit
 
 val extract_crc_interfaces: unit -> crcs
 
@@ -33,11 +39,10 @@ type error =
   | Custom_runtime
   | File_exists of filepath
   | Cannot_open_dll of filepath
-  | Required_module_unavailable of modname
   | Camlheader of string * filepath
+  | Link_error of Linkdeps.error
 
 exception Error of error
 
-open Format
-
-val report_error: formatter -> error -> unit
+val report_error: error Format_doc.format_printer
+val report_error_doc: error Format_doc.printer
