@@ -21,7 +21,7 @@ type code_event =
   { ev_frag : int;
     ev_ev : Instruct.debug_event }
 
-val get_pos : debug_event -> Lexing.position;;
+val get_pos : debug_event -> Lexing.position
 
 (** Current events. **)
 
