@@ -30,10 +30,8 @@ let prompt = ref true
 let time = ref true
 let version = ref true
 
-let topdirs_path = ref (Filename.concat Config.standard_library "compiler-libs")
-
 let add_path dir =
-  Load_path.add_dir dir;
+  Load_path.add_dir ~hidden:false dir;
   Envaux.reset_cache()
 
 let add_path_for mdl dir =
