@@ -181,7 +181,7 @@ let new_checkpoint_list checkpoint_count accepted rejected =
     let (k, l) =
       list_truncate2 (checkpoint_count - List.length accepted) rejected
     in
-      (List.merge (fun {c_time = t1} {c_time = t2} -> compare t2 t1) accepted k,
+      (List.merge (fun t1 t2 -> compare t2.c_time t1.c_time) accepted k,
        l)
 
 (* Clean the checkpoint list. *)
@@ -380,8 +380,8 @@ let new_checkpoint pid fd =
      c_parent = root;
      c_breakpoint_version = 0;
      c_breakpoints = [];
-     c_trap_barrier = 0;
-     c_code_fragments = [0]}
+     c_trap_barrier = Sp.null;
+     c_code_fragments = [main_frag]}
   in
     insert_checkpoint new_checkpoint
 
@@ -506,7 +506,7 @@ let rec back_to time time_max =
   let
     {c_time = t} = find_checkpoint_before (pre64 time_max)
   in
-    go_to (max time t);
+    go_to (Int64.max time t);
     let (new_time, break) = find_last_breakpoint time_max in
     if break <> None || (new_time <= time) then begin
       go_to new_time;
@@ -520,7 +520,7 @@ let rec back_to time time_max =
 let step_backward duration =
   let time = current_time () in
     if time > _0 then
-      back_to (max _0 (time -- duration)) time
+      back_to (Int64.max _0 (time -- duration)) time
 
 (* Run the program from current time. *)
 (* Stop at the first breakpoint, or at the end of the program. *)
@@ -555,7 +555,7 @@ let finish () =
   | Some {ev_ev={ev_stacksize}} ->
       set_initial_frame();
       let (frame, pc) = up_frame ev_stacksize in
-      if frame < 0 then begin
+      if frame = Sp.null then begin
         prerr_endline "`finish' not meaningful in outermost frame.";
         raise Toplevel
       end;
@@ -598,8 +598,9 @@ let next_1 () =
         | Some {ev_ev={ev_stacksize=ev_stacksize2}} ->
             let (frame2, _pc2) = initial_frame() in
             (* Call `finish' if we've entered a function. *)
-            if frame1 >= 0 && frame2 >= 0 &&
-               frame2 - ev_stacksize2 > frame1 - ev_stacksize1
+            if frame1 <> Sp.null && frame2 <> Sp.null &&
+               Sp.(compare (base frame2 ev_stacksize2)
+                     (base frame1 ev_stacksize1)) > 0
             then finish()
       end
 
@@ -622,7 +623,7 @@ let start () =
   | Some {ev_ev={ev_stacksize}} ->
       let (frame, _) = initial_frame() in
       let (frame', pc) = up_frame ev_stacksize in
-      if frame' < 0 then begin
+      if frame' = Sp.null then begin
         prerr_endline "`start not meaningful in outermost frame.";
         raise Toplevel
       end;
@@ -644,7 +645,7 @@ let start () =
             step _minus1;
             (not !interrupted)
               &&
-            (frame' - nargs > frame - ev_stacksize)
+            Sp.(compare (base frame' nargs) (base frame ev_stacksize)) > 0
         | _ ->
             false
       do
@@ -666,8 +667,9 @@ let previous_1 () =
         | Some {ev_ev={ev_stacksize=ev_stacksize2}} ->
             let (frame2, _pc2) = initial_frame() in
             (* Call `start' if we've entered a function. *)
-            if frame1 >= 0 && frame2 >= 0 &&
-               frame2 - ev_stacksize2 > frame1 - ev_stacksize1
+            if frame1 <> Sp.null && frame2 <> Sp.null &&
+              Sp.(compare (base frame2 ev_stacksize2)
+                    (base frame1 ev_stacksize1)) > 0
             then start()
       end
 
