@@ -235,13 +235,6 @@ module Inconstants (P:Param) (Backend:Backend_intf.S) = struct
     | Let_mutable { initial_value = var; body } ->
       mark_var var curr;
       mark_loop ~toplevel curr body
-    | Let_rec(defs, body) ->
-      List.iter (fun (var, def) ->
-          mark_named ~toplevel [Var var] def;
-          (* adds 'var in NC => curr in NC' same remark as let case *)
-          mark_var var curr)
-        defs;
-      mark_loop ~toplevel curr body
     | Var var -> mark_var var curr
     (* Not constant cases: we mark directly 'curr in NC' and mark
        bound variables as in NC also *)
@@ -269,9 +262,9 @@ module Inconstants (P:Param) (Backend:Backend_intf.S) = struct
       mark_curr curr;
       mark_loop ~toplevel [] f1;
       mark_loop ~toplevel:false [] body
-    | If_then_else (f1,f2,f3) ->
+    | If_then_else (cond,f2,f3) ->
       mark_curr curr;
-      mark_curr [Var f1];
+      mark_var cond curr;
       mark_loop ~toplevel [] f2;
       mark_loop ~toplevel [] f3
     | Static_raise (_,l) ->
