@@ -85,12 +85,13 @@ let lambda_smaller' lam ~than:threshold =
       lambda_named_size defining_expr;
       lambda_size body
     | Let_mutable { body } -> lambda_size body
-    | Let_rec (bindings, body) ->
-      List.iter (fun (_, lam) -> lambda_named_size lam) bindings;
-      lambda_size body
     | Switch (_, sw) ->
-      let aux = function _::_::_ -> size := !size + 5 | _ -> () in
-      aux sw.consts; aux sw.blocks;
+      let cost cases =
+        let size = List.length cases in
+        if size <= 1 then 0
+        else 3 + size
+      in
+      size := !size + cost sw.consts + cost sw.blocks;
       List.iter (fun (_, lam) -> lambda_size lam) sw.consts;
       List.iter (fun (_, lam) -> lambda_size lam) sw.blocks;
       Option.iter lambda_size sw.failaction
@@ -172,7 +173,7 @@ module Threshold = struct
     | Never_inline, _ -> Never_inline
     | _, Never_inline -> Never_inline
     | Can_inline_if_no_larger_than i1, Can_inline_if_no_larger_than i2 ->
-      Can_inline_if_no_larger_than (min i1 i2)
+      Can_inline_if_no_larger_than (Int.min i1 i2)
 
   let equal t1 t2 =
     match t1, t2 with
@@ -262,7 +263,7 @@ module Benefit = struct
     | Switch _ | String_switch _ | Static_raise _ | Try_with _
     | If_then_else _ | While _ | For _ -> b := remove_branch !b
     | Apply _ | Send _ -> b := remove_call !b
-    | Let _ | Let_mutable _ | Let_rec _ | Proved_unreachable | Var _
+    | Let _ | Let_mutable _ | Proved_unreachable | Var _
     | Static_catch _ -> ()
 
   let remove_code_helper_named b (named : Flambda.named) =
@@ -674,7 +675,7 @@ let maximum_interesting_size_of_function_body_base =
         let inline_call_cost = cost !Clflags.inline_call_cost ~round in
         direct_call_size + (inline_call_cost * benefit_factor)
       in
-      max_cost := max !max_cost max_size
+      max_cost := Int.max !max_cost max_size
     done;
     !max_cost
   end
@@ -687,7 +688,7 @@ let maximum_interesting_size_of_function_body_multiplier =
         let inline_prim_cost = cost !Clflags.inline_prim_cost ~round in
         inline_prim_cost * benefit_factor
       in
-      max_cost := max !max_cost max_size
+      max_cost := Int.max !max_cost max_size
     done;
     !max_cost
   end
