@@ -107,7 +107,6 @@ let pbytessetu = "Pbytessetu"
 let pccall = "Pccall"
 let pctconst = "Pctconst"
 let pcvtbint = "Pcvtbint"
-let pdirapply = "Pdirapply"
 let pdivbint = "Pdivbint"
 let pdivfloat = "Pdivfloat"
 let pdivint = "Pdivint"
@@ -119,7 +118,6 @@ let pfloatcomp = "Pfloatcomp"
 let pfloatfield = "Pfloatfield"
 let pfloatofint = "Pfloatofint"
 let pgetglobal = "Pgetglobal"
-let pidentity = "Pidentity"
 let pignore = "Pignore"
 let pint_as_pointer = "Pint_as_pointer"
 let pintcomp = "Pintcomp"
@@ -153,7 +151,6 @@ let porbint = "Porbint"
 let porint = "Porint"
 let praise = "Praise"
 let predef_exn = "predef_exn"
-let prevapply = "Prevapply"
 let project_closure = "project_closure"
 let psequand = "Psequand"
 let psequor = "Psequor"
@@ -172,6 +169,17 @@ let psubfloat = "Psubfloat"
 let psubint = "Psubint"
 let pxorbint = "Pxorbint"
 let pxorint = "Pxorint"
+let patomic_cas = "Patomic_cas"
+let patomic_exchange = "Patomic_exchange"
+let patomic_fetch_add = "Patomic_fetch_add"
+let patomic_load = "Patomic_load"
+let prunstack = "Prunstack"
+let pperform = "Pperform"
+let presume = "Presume"
+let preperform = "Preperform"
+let pdls_get = "Pdls_get"
+let ppoll = "Ppoll"
+
 let pabsfloat_arg = "Pabsfloat_arg"
 let paddbint_arg = "Paddbint_arg"
 let paddfloat_arg = "Paddfloat_arg"
@@ -209,7 +217,6 @@ let pbytessetu_arg = "Pbytessetu_arg"
 let pccall_arg = "Pccall_arg"
 let pctconst_arg = "Pctconst_arg"
 let pcvtbint_arg = "Pcvtbint_arg"
-let pdirapply_arg = "Pdirapply_arg"
 let pdivbint_arg = "Pdivbint_arg"
 let pdivfloat_arg = "Pdivfloat_arg"
 let pdivint_arg = "Pdivint_arg"
@@ -221,7 +228,6 @@ let pfloatcomp_arg = "Pfloatcomp_arg"
 let pfloatfield_arg = "Pfloatfield_arg"
 let pfloatofint_arg = "Pfloatofint_arg"
 let pgetglobal_arg = "Pgetglobal_arg"
-let pidentity_arg = "Pidentity_arg"
 let pignore_arg = "Pignore_arg"
 let pint_as_pointer_arg = "Pint_as_pointer_arg"
 let pintcomp_arg = "Pintcomp_arg"
@@ -253,7 +259,6 @@ let popaque_arg = "Popaque_arg"
 let porbint_arg = "Porbint_arg"
 let porint_arg = "Porint_arg"
 let praise_arg = "Praise_arg"
-let prevapply_arg = "Prevapply_arg"
 let psequand_arg = "Psequand_arg"
 let psequor_arg = "Psequor_arg"
 let psetfield_arg = "Psetfield_arg"
@@ -277,6 +282,17 @@ let psubfloat_arg = "Psubfloat_arg"
 let psubint_arg = "Psubint_arg"
 let pxorbint_arg = "Pxorbint_arg"
 let pxorint_arg = "Pxorint_arg"
+let patomic_cas_arg = "Patomic_cas_arg"
+let patomic_exchange_arg = "Patomic_exchange_arg"
+let patomic_fetch_add_arg = "Patomic_fetch_add_arg"
+let patomic_load_arg = "Patomic_load_arg"
+let prunstack_arg = "Prunstack_arg"
+let pperform_arg = "Pperform_arg"
+let presume_arg = "Presume_arg"
+let preperform_arg = "Preperform_arg"
+let pdls_get_arg = "Pdls_get_arg"
+let ppoll_arg = "Ppoll_arg"
+
 let raise = "raise"
 let raise_arg = "raise_arg"
 let read_mutable = "read_mutable"
@@ -310,12 +326,9 @@ let anon_fn_with_loc (sloc: Lambda.scoped_location) =
       (Filename.basename file) line pp_chars
 
 let of_primitive : Lambda.primitive -> string = function
-  | Pidentity -> pidentity
   | Pbytes_of_string -> pbytes_of_string
   | Pbytes_to_string -> pbytes_to_string
   | Pignore -> pignore
-  | Prevapply -> prevapply
-  | Pdirapply -> pdirapply
   | Pgetglobal _ -> pgetglobal
   | Psetglobal _ -> psetglobal
   | Pmakeblock _ -> pmakeblock
@@ -414,14 +427,21 @@ let of_primitive : Lambda.primitive -> string = function
   | Pbbswap _ -> pbbswap
   | Pint_as_pointer -> pint_as_pointer
   | Popaque -> popaque
+  | Patomic_cas -> patomic_cas
+  | Patomic_exchange -> patomic_exchange
+  | Patomic_fetch_add -> patomic_fetch_add
+  | Patomic_load _ -> patomic_load
+  | Prunstack -> prunstack
+  | Pperform -> pperform
+  | Presume -> presume
+  | Preperform -> preperform
+  | Pdls_get -> pdls_get
+  | Ppoll -> ppoll
 
 let of_primitive_arg : Lambda.primitive -> string = function
-  | Pidentity -> pidentity_arg
   | Pbytes_of_string -> pbytes_of_string_arg
   | Pbytes_to_string -> pbytes_to_string_arg
   | Pignore -> pignore_arg
-  | Prevapply -> prevapply_arg
-  | Pdirapply -> pdirapply_arg
   | Pgetglobal _ -> pgetglobal_arg
   | Psetglobal _ -> psetglobal_arg
   | Pmakeblock _ -> pmakeblock_arg
@@ -520,3 +540,13 @@ let of_primitive_arg : Lambda.primitive -> string = function
   | Pbbswap _ -> pbbswap_arg
   | Pint_as_pointer -> pint_as_pointer_arg
   | Popaque -> popaque_arg
+  | Patomic_cas -> patomic_cas_arg
+  | Patomic_exchange -> patomic_exchange_arg
+  | Patomic_fetch_add -> patomic_fetch_add_arg
+  | Patomic_load _ -> patomic_load_arg
+  | Prunstack -> prunstack_arg
+  | Pperform -> pperform_arg
+  | Presume -> presume_arg
+  | Preperform -> preperform_arg
+  | Pdls_get -> pdls_get_arg
+  | Ppoll -> ppoll_arg
