@@ -90,7 +90,7 @@ module Typedtree_search :
    The module uses the module {!Odoc_sig.Analyser}.
    @param My_ir The module used to retrieve comments and special comments.*)
 module Analyser :
-  functor (My_ir : Odoc_sig.Info_retriever) ->
+  Odoc_sig.Info_retriever ->
     sig
       (** This function takes a file name, a file containing the code and
          the typed tree obtained from the compiler.
