@@ -17,9 +17,6 @@
 
 open Asttypes
 open Types
-
-let print_DEBUG s = print_string s ; print_newline ();;
-
 open Odoc_parameter
 open Odoc_value
 open Odoc_type
@@ -107,8 +104,8 @@ module Signature_search =
       type_expr
 
     let search_method_type name class_sig =
-      let fields = Odoc_misc.get_fields class_sig.Types.csig_self in
-      List.assoc name fields
+      let (_, _, type_expr) = Types.Meths.find name class_sig.Types.csig_meths in
+      type_expr
   end
 
 module type Info_retriever =
@@ -121,6 +118,76 @@ module type Info_retriever =
         (Odoc_types.text -> 'a) -> string -> string -> (Odoc_types.info option * 'a list)
   end
 
+let alert_of_attribute attr =
+  let open Parsetree in
+  let load_constant_string = function
+    | { pexp_desc = Pexp_constant
+            { pconst_desc = Pconst_string (text, _, _); _ }; _ } ->
+        Some text
+    | _ -> None
+  in
+  let load_alert_name name = Longident.last name.Location.txt in
+  let deprecated_payload = function
+    | PStr [ { pstr_desc = Pstr_eval (s, _); _ } ] -> load_constant_string s
+    | _ -> None
+  in
+  let alert_payload = function
+    | PStr [ { pstr_desc = Pstr_eval ({ pexp_desc; _ }, _); _ } ] -> (
+        match pexp_desc with
+        | Pexp_apply ({ pexp_desc = Pexp_ident name; _ }, [ (_, payload) ]) ->
+            Some (load_alert_name name, load_constant_string payload)
+        | Pexp_ident name -> Some (load_alert_name name, None)
+        | _ -> None)
+    | _ -> None
+  in
+  match attr.attr_name.txt with
+  | "deprecated" | "ocaml.deprecated" ->
+      Some
+        {
+          alert_name = "deprecated";
+          alert_payload = deprecated_payload attr.attr_payload;
+        }
+  | "alert" | "ocaml.alert" -> (
+      match alert_payload attr.attr_payload with
+      | Some (alert_name, alert_payload) -> Some { alert_name; alert_payload }
+      | None -> None)
+  | _ -> None
+
+let analyze_alerts info attrs =
+  (* Remove any deprecated alert if the tag is present. *)
+  let unify_deprecated alert (depr_tag, acc) =
+    match alert.alert_name with
+    | "deprecated" ->
+        let depr_tag =
+          match (depr_tag, alert.alert_payload) with
+          | Some (_ :: _), _ -> depr_tag
+          | _, Some text -> Some [ Odoc_types.Raw text ]
+          | _, None -> Some []
+        in
+        (depr_tag, acc)
+    | _ -> (depr_tag, alert :: acc)
+  in
+  match List.filter_map alert_of_attribute attrs with
+  | [] -> info
+  | _ :: _ as alerts ->
+      let info = Option.value ~default:Odoc_types.dummy_info info in
+      let i_deprecated, i_alerts =
+        List.fold_right unify_deprecated alerts (info.i_deprecated, [])
+      in
+      Some { info with i_deprecated; i_alerts }
+
+(** Look for signature-item alerts ([\[@@@...\]]) at the beginning of a
+    signature. Stop at any other item. These alerts are considered attached to
+    the entire signature, similarly to the first comment, which becomes the
+    preamble. *)
+let analyze_toplevel_alerts info ast =
+  let rec extract_attributes = function
+    | { Parsetree.psig_desc = Parsetree.Psig_attribute attr; _ } :: tl ->
+        attr :: extract_attributes tl
+    | _ :: _ | [] -> []
+  in
+  analyze_alerts info (extract_attributes ast)
+
 module Analyser =
   functor (My_ir : Info_retriever) ->
   struct
@@ -194,6 +261,13 @@ module Analyser =
 
     let merge_infos = Odoc_merge.merge_info_opt Odoc_types.all_merge_options
 
+    (** Fetch the comment just after an item then merge the info with
+        [comment_opt]. If [attrs] is passed, alerts will be read from it. *)
+    let get_info ?(attrs = []) comment_opt start stop =
+      let maybe_more, info_after_opt = just_after_special start stop in
+      let comment_opt = merge_infos comment_opt info_after_opt in
+      maybe_more, analyze_alerts comment_opt attrs
+
     (** Module for extracting documentation comments for record from different
         tree types *)
     module Record = struct
@@ -344,7 +418,7 @@ module Analyser =
 
 
     let manifest_structure env name_comment_list type_expr =
-      match type_expr.desc with
+      match get_desc type_expr with
       | Tobject (fields, _) ->
         let f (field_name, _, type_expr) =
           let comment_opt =
@@ -359,12 +433,13 @@ module Analyser =
         Object_type (List.map f @@ fst @@ Ctype.flatten_fields fields)
       | _ -> Other (Odoc_env.subst_type env type_expr)
 
-    let get_field env name_comment_list {Types.ld_id=field_name;ld_mutable=mutable_flag;ld_type=type_expr} =
+    let get_field env name_comment_list {Types.ld_id=field_name;ld_mutable=mutable_flag;ld_type=type_expr;ld_attributes} =
       let field_name = Ident.name field_name in
       let comment_opt =
         try List.assoc field_name name_comment_list
         with Not_found -> None
       in
+      let comment_opt = analyze_alerts comment_opt ld_attributes in
       {
         rf_name = field_name ;
         rf_mutable = mutable_flag = Mutable ;
@@ -374,10 +449,10 @@ module Analyser =
 
     let get_type_kind env name_comment_list type_kind =
       match type_kind with
-        Types.Type_abstract ->
+        Types.Type_abstract _ ->
           Odoc_type.Type_abstract
-      | Types.Type_variant l ->
-          let f {Types.cd_id=constructor_name;cd_args;cd_res=ret_type} =
+      | Types.Type_variant (l,_) ->
+          let f {Types.cd_id=constructor_name;cd_args;cd_res=ret_type;cd_attributes} =
             let constructor_name = Ident.name constructor_name in
             let comment_opt =
               try match List.assoc constructor_name name_comment_list with
@@ -385,14 +460,21 @@ module Analyser =
                 | x -> x
               with Not_found -> None
             in
+            let comment_opt = analyze_alerts comment_opt cd_attributes in
             let vc_args =
               match cd_args with
               | Cstr_tuple l -> Cstr_tuple (List.map (Odoc_env.subst_type env) l)
               | Cstr_record l ->
                   Cstr_record (List.map (get_field env name_comment_list) l)
             in
+            let vc_name = match constructor_name with
+              | "::" ->
+                  (* The only infix constructor is always printed (::) *)
+                  "(::)"
+              | s -> s
+            in
             {
-              vc_name = constructor_name ;
+              vc_name;
               vc_args;
               vc_ret =  Option.map (Odoc_env.subst_type env) ret_type;
               vc_text = comment_opt
@@ -448,11 +530,14 @@ module Analyser =
     let erased_names_of_constraints constraints acc =
       List.fold_right (fun constraint_ acc ->
         match constraint_ with
-        | Parsetree.Pwith_type _ | Parsetree.Pwith_module _ -> acc
+        | Parsetree.Pwith_type _ | Parsetree.Pwith_module _ | Parsetree.Pwith_modtype _ -> acc
         | Parsetree.Pwith_typesubst (s, typedecl) ->
            constraint_for_subitem acc s (fun s -> Parsetree.Pwith_typesubst (s, typedecl))
         | Parsetree.Pwith_modsubst (s, modpath) ->
-           constraint_for_subitem acc s (fun s -> Parsetree.Pwith_modsubst (s, modpath)))
+           constraint_for_subitem acc s (fun s -> Parsetree.Pwith_modsubst (s, modpath))
+        | Parsetree.Pwith_modtypesubst (s, modpath) ->
+            constraint_for_subitem acc s
+              (fun s -> Parsetree.Pwith_modtypesubst (s, modpath)))
         constraints acc
 
     let is_erased ident map =
@@ -507,6 +592,7 @@ module Analyser =
            end
         | Parsetree.Psig_modtype {Parsetree.pmtd_name=name} as m ->
           if is_erased name.txt erased then acc else take_item m
+        | Parsetree.Psig_modtypesubst _  -> acc
         | Parsetree.Psig_recmodule mods ->
           (match List.filter
                    (fun pmd ->
@@ -767,6 +853,10 @@ module Analyser =
             in
             let name = Name.parens_if_infix name_pre.txt in
             let subst_typ = Odoc_env.subst_type env type_expr in
+            let (maybe_more, comment_opt) =
+              get_info ~attrs:value_desc.Parsetree.pval_attributes comment_opt
+                pos_end_ele pos_limit
+            in
             let v =
               {
                 val_name = Name.concat current_module_name name ;
@@ -778,12 +868,6 @@ module Analyser =
                 val_loc = { loc_impl = None ; loc_inter = Some sig_item_loc } ;
               }
             in
-            let (maybe_more, info_after_opt) =
-              My_ir.just_after_special
-                !file_name
-                (get_string_of_file pos_end_ele pos_limit)
-            in
-            v.val_info <- merge_infos v.val_info info_after_opt ;
             (* update the parameter description *)
             Odoc_value.update_value_parameters_text v;
 
@@ -793,7 +877,8 @@ module Analyser =
         | Parsetree.Psig_typext tyext ->
           let new_env, types_ext_list, last_ext =
             List.fold_left
-              (fun (env_acc, exts_acc, _) -> fun {Parsetree.pext_name = { txt = name }} ->
+              (fun (env_acc, exts_acc, _) ->
+               fun {Parsetree.pext_name = { txt = name }; pext_attributes; _} ->
                 let complete_name = Name.concat current_module_name name in
                 let env_acc = Odoc_env.add_extension env_acc complete_name in
                 let types_ext =
@@ -801,7 +886,7 @@ module Analyser =
                   with Not_found ->
                     raise (Failure (Odoc_messages.extension_not_found current_module_name name))
                 in
-                  env_acc, ((name, types_ext) :: exts_acc), Some types_ext
+                  env_acc, ((name, pext_attributes, types_ext) :: exts_acc), Some types_ext
               )
               (env, [], None)
               tyext.Parsetree.ptyext_constructors
@@ -814,7 +899,7 @@ module Analyser =
           in
           let new_te =
             {
-              te_info = comment_opt;
+              te_info = None;
               te_type_name =
                 Odoc_env.full_type_name new_env (Name.from_path ty_path);
               te_type_parameters =
@@ -834,7 +919,7 @@ module Analyser =
           let rec analyse_extension_constructors maybe_more exts_acc types_ext_list =
             match types_ext_list with
               [] -> (maybe_more, List.rev exts_acc)
-            | (name, types_ext) :: q ->
+            | (name, attrs, types_ext) :: q ->
               let ext_loc_end =  Loc.end_ types_ext.Types.ext_loc in
               let xt_args =
                 match types_ext.ext_args with
@@ -844,6 +929,14 @@ module Analyser =
                     let docs = Record.(doc types ext_loc_end) l in
                     Cstr_record (List.map (get_field new_env docs) l)
               in
+              let (maybe_more, comment_opt) =
+                let pos_limit2 =
+                  match q with
+                    [] -> pos_limit
+                  | (_, _, next) :: _ -> Loc.start (next.Types.ext_loc)
+                in
+                get_info ~attrs None ext_loc_end pos_limit2
+              in
               let new_x =
                 {
                   xt_name = Name.concat current_module_name name ;
@@ -852,33 +945,27 @@ module Analyser =
                   xt_type_extension = new_te;
                   xt_alias = None ;
                   xt_loc = { loc_impl = None ; loc_inter = Some types_ext.Types.ext_loc} ;
-                  xt_text = None;
+                  xt_text = comment_opt;
                 }
               in
-              let pos_limit2 =
-                match q with
-                  [] -> pos_limit
-                | (_, next) :: _ -> Loc.start (next.Types.ext_loc)
-              in
-              let (maybe_more, comment_opt) =
-                just_after_special ext_loc_end pos_limit2 in
-                new_x.xt_text <- comment_opt;
-                analyse_extension_constructors maybe_more (new_x :: exts_acc) q
+              analyse_extension_constructors maybe_more (new_x :: exts_acc) q
           in
           let (maybe_more, exts) = analyse_extension_constructors 0 [] types_ext_list in
-            new_te.te_constructors <- exts;
-            let (maybe_more2, info_after_opt) =
-              just_after_special (pos_end_ele + maybe_more) pos_limit
-            in
-              new_te.te_info <- merge_infos new_te.te_info info_after_opt ;
-              (maybe_more + maybe_more2, new_env, [ Element_type_extension new_te ])
+          new_te.te_constructors <- exts;
+          let (maybe_more2, comment_opt) =
+            get_info ~attrs:tyext.Parsetree.ptyext_attributes comment_opt
+              (pos_end_ele + maybe_more) pos_limit
+          in
+          new_te.te_info <- comment_opt;
+          (maybe_more + maybe_more2, new_env, [ Element_type_extension new_te ])
 
-        | Parsetree.Psig_exception ext ->
-            let name = ext.Parsetree.ptyexn_constructor.Parsetree.pext_name in
+        | Parsetree.Psig_exception {Parsetree.ptyexn_constructor;ptyexn_attributes;_} ->
+            let {Parsetree.pext_name;pext_attributes;_} = ptyexn_constructor in
+            let name = pext_name.txt in
             let types_ext =
-              try Signature_search.search_extension table name.txt
+              try Signature_search.search_extension table name
               with Not_found ->
-                raise (Failure (Odoc_messages.exception_not_found current_module_name name.txt))
+                raise (Failure (Odoc_messages.exception_not_found current_module_name name))
             in
             let ex_args =
               let pos_end = Loc.end_ types_ext.ext_loc in
@@ -888,9 +975,13 @@ module Analyser =
                   let docs = Record.(doc types) pos_end l in
                   Cstr_record (List.map (get_field env docs) l)
             in
+            let (maybe_more, comment_opt) =
+              get_info ~attrs:(ptyexn_attributes @ pext_attributes) comment_opt
+                pos_end_ele pos_limit
+            in
             let e =
               {
-                ex_name = Name.concat current_module_name name.txt ;
+                ex_name = Name.concat current_module_name name ;
                 ex_info = comment_opt ;
                 ex_args;
                 ex_ret = Option.map (Odoc_env.subst_type env) types_ext.ext_ret_type ;
@@ -905,12 +996,6 @@ module Analyser =
                    ) ;
               }
             in
-            let (maybe_more, info_after_opt) =
-              My_ir.just_after_special
-                !file_name
-                (get_string_of_file pos_end_ele pos_limit)
-            in
-            e.ex_info <- merge_infos e.ex_info info_after_opt ;
             let new_env = Odoc_env.add_extension env e.ex_name in
             (maybe_more, new_env, [ Element_exception e ])
 
@@ -954,23 +1039,6 @@ module Analyser =
                       pos_limit2
                       type_decl
                   in
-(* DEBUG *)       begin
-(* DEBUG *)         let comm =
-(* DEBUG *)           match assoc_com with
-(* DEBUG *)           | None -> "sans commentaire"
-(* DEBUG *)           | Some c -> Odoc_misc.string_of_info c
-(* DEBUG *)         in
-(* DEBUG *)         print_DEBUG ("Type "^name.txt^" : "^comm);
-(* DEBUG *)         let f_DEBUG (name, c_opt) =
-(* DEBUG *)           let comm =
-(* DEBUG *)             match c_opt with
-(* DEBUG *)             | None -> "sans commentaire"
-(* DEBUG *)             | Some c -> Odoc_misc.string_of_info c
-(* DEBUG *)           in
-(* DEBUG *)           print_DEBUG ("constructor/field "^name^": "^comm)
-(* DEBUG *)         in
-(* DEBUG *)         List.iter f_DEBUG name_comment_list;
-(* DEBUG *)       end;
                   (* get the information for the type in the signature *)
                   let sig_type_decl =
                     try Signature_search.search_type table name.txt
@@ -982,15 +1050,18 @@ module Analyser =
                   let loc_start = Loc.start type_decl.Parsetree.ptype_loc in
                   let new_end = Loc.end_ type_decl.Parsetree.ptype_loc
                                 + maybe_more in
+                  (* get the comment associated to the whole type *)
+                  let (maybe_more2, assoc_com) =
+                    get_info ~attrs:type_decl.Parsetree.ptype_attributes
+                      assoc_com new_end pos_limit2
+                  in
                   (* associate the comments to each constructor and build the [Type.t_type] *)
                   let new_type =
                     {
                       ty_name = Name.concat current_module_name name.txt ;
                       ty_info = assoc_com ;
                       ty_parameters =
-                        List.map2 (fun p v ->
-                          let (co, cn) = Types.Variance.get_upper v in
-                          (Odoc_env.subst_type env p,co, cn))
+                        List.map2 (fun p v -> Odoc_env.subst_type env p,v)
                         sig_type_decl.Types.type_params
                         sig_type_decl.Types.type_variance;
                       ty_kind = type_kind;
@@ -1011,12 +1082,6 @@ module Analyser =
                         ) ;
                     }
                   in
-                  let (maybe_more2, info_after_opt) =
-                    My_ir.just_after_special
-                      !file_name
-                      (get_string_of_file new_end pos_limit2)
-                  in
-                  new_type.ty_info <- merge_infos new_type.ty_info info_after_opt ;
                   let (new_maybe_more, eles) = f
                       (maybe_more + maybe_more2)
                       (new_end + maybe_more2)
@@ -1062,23 +1127,6 @@ module Analyser =
                       pos_limit2
                       type_decl
                   in
-(* DEBUG *)       begin
-(* DEBUG *)         let comm =
-(* DEBUG *)           match assoc_com with
-(* DEBUG *)           | None -> "sans commentaire"
-(* DEBUG *)           | Some c -> Odoc_misc.string_of_info c
-(* DEBUG *)         in
-(* DEBUG *)         print_DEBUG ("Type "^name.txt^" : "^comm);
-(* DEBUG *)         let f_DEBUG (name, c_opt) =
-(* DEBUG *)           let comm =
-(* DEBUG *)             match c_opt with
-(* DEBUG *)             | None -> "sans commentaire"
-(* DEBUG *)             | Some c -> Odoc_misc.string_of_info c
-(* DEBUG *)           in
-(* DEBUG *)           print_DEBUG ("constructor/field "^name^": "^comm)
-(* DEBUG *)         in
-(* DEBUG *)         List.iter f_DEBUG name_comment_list;
-(* DEBUG *)       end;
                   (* get the information for the type in the signature *)
                   let sig_type_decl =
                     try Signature_search.search_type table name.txt
@@ -1096,9 +1144,7 @@ module Analyser =
                       ty_name = Name.concat current_module_name name.txt ;
                       ty_info = assoc_com ;
                       ty_parameters =
-                        List.map2 (fun p v ->
-                          let (co, cn) = Types.Variance.get_upper v in
-                          (Odoc_env.subst_type env p,co, cn))
+                        List.map2 (fun p v -> Odoc_env.subst_type env p,v)
                         sig_type_decl.Types.type_params
                         sig_type_decl.Types.type_variance;
                       ty_kind = type_kind;
@@ -1151,7 +1197,8 @@ module Analyser =
         | Parsetree.Psig_module {Parsetree.pmd_name={txt=None}} ->
             (0, env, [])
 
-        | Parsetree.Psig_module {Parsetree.pmd_name={txt=Some name}; pmd_type=module_type} ->
+        | Parsetree.Psig_module {Parsetree.pmd_name={txt=Some name};
+                                 pmd_type=module_type; pmd_attributes} ->
             let complete_name = Name.concat current_module_name name in
             (* get the module type in the signature by the module name *)
             let sig_module_type =
@@ -1169,6 +1216,9 @@ module Analyser =
               else
                 None
             in
+            let (maybe_more, comment_opt) =
+              get_info ~attrs:pmd_attributes comment_opt pos_end_ele pos_limit
+            in
             let new_module =
               {
                 m_name = complete_name ;
@@ -1184,12 +1234,6 @@ module Analyser =
                 m_text_only = false ;
               }
             in
-            let (maybe_more, info_after_opt) =
-              My_ir.just_after_special
-                !file_name
-                (get_string_of_file pos_end_ele pos_limit)
-            in
-            new_module.m_info <- merge_infos new_module.m_info info_after_opt ;
             let new_env = Odoc_env.add_module env new_module.m_name in
             let new_env2 =
               match new_module.m_type with (* FIXME : can this be a Tmty_ident? in this case, we wouldn't have the signature *)
@@ -1218,9 +1262,7 @@ module Analyser =
                         (* FIXME : can this be a Tmty_ident? in this case, we wouldn't have the signature *)
                         Types.Mty_signature s ->
                           Odoc_env.add_signature e complete_name ~rel: name s
-                      | _ ->
-                          print_DEBUG "not a Tmty_signature";
-                          e
+                      | _ -> e
                     )
                     env
                     decls
@@ -1255,7 +1297,8 @@ module Analyser =
                   in
                   (maybe_more2, ele_comments @ eles)
 
-              | {Parsetree.pmd_name={txt = Some name}; pmd_type=modtype} :: q ->
+              | {Parsetree.pmd_name={txt = Some name}; pmd_type=modtype;
+                 pmd_attributes} :: q ->
                   let complete_name = Name.concat current_module_name name in
                   let loc = modtype.Parsetree.pmty_loc in
                   let loc_start = Loc.start loc in
@@ -1289,6 +1332,9 @@ module Analyser =
                     else
                       None
                   in
+                  let (maybe_more, assoc_com) =
+                    get_info ~attrs:pmd_attributes assoc_com loc_end pos_limit2
+                  in
                   let new_module =
                     {
                       m_name = complete_name ;
@@ -1304,12 +1350,6 @@ module Analyser =
                       m_text_only = false ;
                     }
                   in
-                  let (maybe_more, info_after_opt) =
-                    My_ir.just_after_special
-                      !file_name
-                      (get_string_of_file loc_end pos_limit2)
-                  in
-                  new_module.m_info <- merge_infos new_module.m_info info_after_opt ;
 
                   let (maybe_more2, eles) = f
                       maybe_more
@@ -1321,7 +1361,12 @@ module Analyser =
             let (maybe_more, mods) = f ~first: true 0 pos_start_ele decls in
             (maybe_more, new_env, mods)
 
-        | Parsetree.Psig_modtype {Parsetree.pmtd_name=name; pmtd_type=pmodtype_decl} ->
+        | Parsetree.Psig_modtype {Parsetree.pmtd_name=name;
+                                  pmtd_type=pmodtype_decl;
+                                  pmtd_attributes=pmodtype_attrs}
+        | Parsetree.Psig_modtypesubst {Parsetree.pmtd_name=name;
+                                       pmtd_type=pmodtype_decl;
+                                       pmtd_attributes=pmodtype_attrs} ->
             let complete_name = Name.concat current_module_name name.txt in
             let sig_mtype =
               try Signature_search.search_module_type table name.txt
@@ -1336,7 +1381,9 @@ module Analyser =
                 | Some sig_mtype -> Some (analyse_module_type_kind env complete_name module_type sig_mtype)
                 | None -> None
             in
-
+            let (maybe_more, comment_opt) =
+              get_info ~attrs:pmodtype_attrs comment_opt pos_end_ele pos_limit
+            in
             let mt =
               {
                 mt_name = complete_name ;
@@ -1348,12 +1395,6 @@ module Analyser =
                 mt_loc = { loc_impl = None ; loc_inter = Some sig_item_loc } ;
               }
             in
-            let (maybe_more, info_after_opt) =
-              My_ir.just_after_special
-                !file_name
-                (get_string_of_file pos_end_ele pos_limit)
-            in
-            mt.mt_info <- merge_infos mt.mt_info info_after_opt ;
             let new_env = Odoc_env.add_module_type env mt.mt_name in
             let new_env2 =
               match sig_mtype with (* FIXME : can this be a Tmty_ident? in this case, we wouldn't have the signature *)
@@ -1443,6 +1484,10 @@ module Analyser =
                      class_desc.Parsetree.pci_expr
                      sig_class_type
                  in
+                 let (maybe_more, assoc_com) =
+                   get_info ~attrs:class_desc.Parsetree.pci_attributes
+                     assoc_com pos_end pos_limit2
+                 in
                  let new_class =
                    {
                      cl_name = complete_name ;
@@ -1455,9 +1500,6 @@ module Analyser =
                      cl_loc = { loc_impl = None ; loc_inter = Some class_desc.Parsetree.pci_loc } ;
                    }
                  in
-                 let (maybe_more, info_after_opt) =
-                   just_after_special pos_end pos_limit2 in
-                 new_class.cl_info <- merge_infos new_class.cl_info info_after_opt ;
                  Odoc_class.class_update_parameters_text new_class ;
                  let (new_maybe_more, eles) =
                    f maybe_more (pos_end + maybe_more) q
@@ -1515,6 +1557,10 @@ module Analyser =
                       ct_decl.Parsetree.pci_expr
                       sig_class_type
                   in
+                  let (maybe_more, assoc_com) =
+                    get_info ~attrs:ct_decl.Parsetree.pci_attributes assoc_com
+                      pos_end pos_limit2
+                  in
                   let ct =
                     {
                       clt_name = complete_name ;
@@ -1526,10 +1572,6 @@ module Analyser =
                       clt_loc = { loc_impl = None ; loc_inter = Some ct_decl.Parsetree.pci_loc } ;
                     }
                   in
-                  let (maybe_more, info_after_opt) =
-                    just_after_special pos_end pos_limit2
-                  in
-                  ct.clt_info <- merge_infos ct.clt_info info_after_opt ;
                   let (new_maybe_more, eles) =
                     f maybe_more (pos_end + maybe_more) q
                   in
@@ -1589,7 +1631,6 @@ module Analyser =
            let loc_start = Loc.start loc in
            let loc_end = Loc.end_ loc in
            let mp_type_code = get_string_of_file loc_start loc_end in
-           print_DEBUG (Printf.sprintf "mp_type_code=%s" mp_type_code);
            match sig_module_type with
              Types.Mty_functor (param, body_module_type) ->
                let mp_name, mp_kind =
@@ -1689,7 +1730,6 @@ module Analyser =
                let loc_start = Loc.start loc in
                let loc_end = Loc.end_ loc in
                let mp_type_code = get_string_of_file loc_start loc_end in
-               print_DEBUG (Printf.sprintf "mp_type_code=%s" mp_type_code);
                let mp_name, mp_kind =
                  match param2, param with
                    Parsetree.Named (_, pmty), Types.Named (Some ident, mty) ->
@@ -1744,7 +1784,6 @@ module Analyser =
       match parse_class_type.Parsetree.pcty_desc, sig_class_type with
         (Parsetree.Pcty_constr (_, _) (*of Longident.t * core_type list *),
          Types.Cty_constr (p, typ_list, _) (*of Path.t * type_expr list * class_type*)) ->
-          print_DEBUG "Cty_constr _";
            let path_name = Name.from_path p in
            let name = Odoc_env.full_class_or_class_type_name env path_name in
            let k =
@@ -1795,7 +1834,6 @@ module Analyser =
       match parse_class_type.Parsetree.pcty_desc, sig_class_type with
         (Parsetree.Pcty_constr (_, _) (*of Longident.t * core_type list *),
          Types.Cty_constr (p, typ_list, _) (*of Path.t * type_expr list * class_type*)) ->
-          print_DEBUG "Cty_constr _";
           Class_type
             {
               cta_name = Odoc_env.full_class_or_class_type_name env (Name.from_path p) ;
@@ -1849,11 +1887,10 @@ module Analyser =
         (ast : Parsetree.signature) (signat : Types.signature) =
       prepare_file source_file input_file;
       (* We create the t_module for this file. *)
-      let mod_name = String.capitalize_ascii
-          (Filename.basename (try Filename.chop_extension source_file with _ -> source_file))
-      in
+      let mod_name = Unit_info.lax_modname_from_source source_file in
       let len, info_opt = preamble !file_name !file
           (fun x -> x.Parsetree.psig_loc) ast in
+      let info_opt = analyze_toplevel_alerts info_opt ast in
       let elements =
         analyse_parsetree Odoc_env.empty signat mod_name len (String.length !file) ast
       in
