@@ -20,7 +20,6 @@ module Name = Odoc_name
 type private_flag = Asttypes.private_flag =
     Private | Public
 
-(** Description of a record type field. *)
 type record_field = {
     rf_name : string ;
     rf_mutable : bool ; (** true if mutable *)
@@ -32,7 +31,6 @@ type constructor_args =
   | Cstr_record of record_field list
   | Cstr_tuple of Types.type_expr list
 
-(** Description of a variant type constructor. *)
 type variant_constructor = {
     vc_name : string ;
     vc_args : constructor_args ;
@@ -40,7 +38,6 @@ type variant_constructor = {
     mutable vc_text : Odoc_types.info option ; (** optional user description *)
   }
 
-(** The various kinds of type. *)
 type type_kind =
     Type_abstract
   | Type_variant of variant_constructor list
@@ -56,15 +53,13 @@ type object_field = {
 }
 
 type type_manifest =
-  | Other of Types.type_expr (** Type manifest directly taken from Typedtre. *)
+  | Other of Types.type_expr (** Type manifest directly taken from Typedtree. *)
   | Object_type of object_field list
 
-(** Representation of a type. *)
 type t_type = {
     ty_name : Name.t ;
     mutable ty_info : Odoc_types.info option ; (** optional user information *)
-    ty_parameters : (Types.type_expr * bool * bool) list ;
-                    (** type parameters: (type, covariant, contravariant) *)
+    ty_parameters : (Types.type_expr * Types.Variance.t ) list ;
     ty_kind : type_kind ;
     ty_private : private_flag;
     ty_manifest : type_manifest option;
