@@ -22,7 +22,7 @@ let skip_with_reason reason =
     let result = Result.skip_with_reason reason in
     (result, env)
   in
-  Actions.make "skip" code
+  Actions.make ~name:"skip" ~description:"Skip the test" code
 
 let pass_or_skip test pass_reason skip_reason _log env =
   let open Result in
@@ -275,6 +275,12 @@ let run_script log env =
           Printf.sprintf "error in script response: unknown variable %s" name
         in
         (Result.fail_with_reason reason, newenv)
+      | exception Variables.Recursive_variable_definition name ->
+        let reason =
+          Printf.sprintf "error in script response: \
+            recursive variable definition %s" name
+        in
+        (Result.fail_with_reason reason, newenv)
     end else begin
       let reason = String.trim (Sys.string_of_file response_file) in
       let newresult = { result with Result.reason = Some reason } in
@@ -321,6 +327,12 @@ let run_hook hook_name log input_env =
           Printf.sprintf "error in script response: unknown variable %s" name
         in
         (Result.fail_with_reason reason, hookenv)
+      | exception Variables.Recursive_variable_definition name ->
+        let reason =
+          Printf.sprintf "error in script response: \
+            recursive variable definition %s" name
+        in
+        (Result.fail_with_reason reason, hookenv)
       end
     | _ ->
       Printf.fprintf log "Hook returned %d" exit_status;
