@@ -15,15 +15,15 @@
 
 (* Flags used in OCaml commands *)
 
-let stdlib ocamlsrcdir =
-  let stdlib_path = Ocaml_directories.stdlib ocamlsrcdir in
+let stdlib =
+  let stdlib_path = Ocaml_directories.stdlib in
   "-nostdlib -I " ^ stdlib_path
 
-let include_toplevel_directory ocamlsrcdir =
-  "-I " ^ (Ocaml_directories.toplevel ocamlsrcdir)
+let include_toplevel_directory =
+  "-I " ^ Ocaml_directories.toplevel
 
-let c_includes ocamlsrcdir =
-  let dir = Ocaml_directories.runtime ocamlsrcdir in
+let c_includes =
+  let dir = Ocaml_directories.runtime in
   "-ccopt -I" ^ dir
 
 let runtime_variant_flags () = match Ocaml_files.runtime_variant() with
@@ -31,9 +31,9 @@ let runtime_variant_flags () = match Ocaml_files.runtime_variant() with
   | Ocaml_files.Debug -> " -runtime-variant d"
   | Ocaml_files.Instrumented -> " -runtime-variant i"
 
-let runtime_flags ocamlsrcdir env backend c_files =
+let runtime_flags env backend c_files =
   let runtime_library_flags = "-I " ^
-    (Ocaml_directories.runtime ocamlsrcdir) in
+    Ocaml_directories.runtime in
   let rt_flags = match backend with
     | Ocaml_backends.Native -> runtime_variant_flags ()
     | Ocaml_backends.Bytecode ->
@@ -46,16 +46,15 @@ let runtime_flags ocamlsrcdir env backend c_files =
           in
           if use_runtime = Some false
           then ""
-          else "-use-runtime " ^ (Ocaml_files.ocamlrun ocamlsrcdir)
+          else "-use-runtime " ^ Ocaml_files.ocamlrun
         end
       end in
   rt_flags ^ " " ^ runtime_library_flags
 
 let toplevel_default_flags = "-noinit -no-version -noprompt"
 
-let ocamldebug_default_flags ocamlsrcdir =
+let ocamldebug_default_flags =
   "-no-version -no-prompt -no-time -no-breakpoint-message " ^
-  ("-I " ^ (Ocaml_directories.stdlib ocamlsrcdir) ^ " ") ^
-  ("-topdirs-path " ^ (Ocaml_directories.toplevel ocamlsrcdir))
+  ("-I " ^ Ocaml_directories.stdlib ^ " ")
 
 let ocamlobjinfo_default_flags = "-null-crc"
