@@ -26,10 +26,18 @@ val binary_modules : Variables.t
 val bytecc_libs : Variables.t
 (** Libraries to link with for bytecode *)
 
-val c_preprocessor : Variables.t
+val cpp : Variables.t
+
+val cppflags : Variables.t
+
+val cc : Variables.t
+
+val cflags : Variables.t
 
 val caml_ld_library_path : Variables.t
 
+val codegen_exit_status : Variables.t
+
 val compare_programs : Variables.t
 
 val compiler_directory_suffix : Variables.t
@@ -74,6 +82,7 @@ val nativecc_libs : Variables.t
 (** Libraries to link with for native code *)
 
 val objext : Variables.t
+val libext : Variables.t
 val asmext : Variables.t
 
 val ocamlc_byte : Variables.t
