@@ -21,12 +21,14 @@ let show_objects title string_of_object objects =
   List.iter print_object objects;
   exit 0
 
-let string_of_action = Actions.name
+let string_of_action a =
+  Printf.sprintf "%s: %s" (Actions.name a) (Actions.description a)
 
 let string_of_test test =
-  if test.Tests.test_run_by_default
-  then (test.Tests.test_name ^ " (run by default)")
-  else test.Tests.test_name
+  if test.Tests.test_run_by_default then
+    test.Tests.test_name ^ " (run by default): " ^ test.Tests.test_description
+  else
+    test.Tests.test_name ^ ": " ^ test.Tests.test_description
 
 let string_of_variable v =
   Printf.sprintf "%s: %s"
@@ -49,12 +51,21 @@ let log_to_stderr = ref false
 
 let promote = ref false
 
+let default_timeout = ref 0
+
 let keep_test_dir_on_success = ref false
 
 let find_test_dirs = ref []
 
 let list_tests = ref []
 
+let show_timings = ref false
+
+let translate = ref false
+let style = ref Translate.Plain
+let compact = ref false
+
+
 let add_to_list r x =
   r := !r @ [x]
 
@@ -66,17 +77,44 @@ let commandline_options =
   ("-show-actions", Arg.Unit show_actions, " Show available actions.");
   ("-show-tests", Arg.Unit show_tests, " Show available tests.");
   ("-show-variables", Arg.Unit show_variables, " Show available variables.");
+  ("-show-timings", Arg.Set show_timings,
+   " Show the wall clock time taken for each test file.");
+  ("-timeout",
+     Arg.Int (fun t -> if t >= 0
+                       then default_timeout := t
+                       else raise (Arg.Bad "negative timeout")),
+     "<seconds> Set maximal execution time for every command (in seconds)");
   ("-find-test-dirs", Arg.String (add_to_list find_test_dirs),
    " Find directories that contain tests (recursive).");
   ("-list-tests", Arg.String (add_to_list list_tests),
    " List tests in given directory.");
   ("-keep-test-dir-on-success", Arg.Set keep_test_dir_on_success,
    " Keep the test directory (with the generated test artefacts) on success.");
+  ("-translate", Arg.Set translate,
+   " Translate the test script from old to new syntax");
+  ("-compact", Arg.Set compact,
+   " If translating, output the new script in compact mode.");
+  ("-keep-lines", Arg.Unit (fun () -> style := Translate.Lines),
+   " If translating, preserve line numbers in the output.");
+  ("-keep-chars", Arg.Unit (fun () -> style := Translate.Chars),
+   " If translating, preserve char offsets in the output.");
 ]
 
 let files_to_test = ref []
 
 let usage = "Usage: " ^ Sys.argv.(0) ^ " options files to test"
 
-let _ =
+let () =
   Arg.parse (Arg.align commandline_options) (add_to_list files_to_test) usage
+
+let log_to_stderr = !log_to_stderr
+let files_to_test = !files_to_test
+let promote = !promote
+let default_timeout = !default_timeout
+let find_test_dirs = !find_test_dirs
+let list_tests = !list_tests
+let keep_test_dir_on_success = !keep_test_dir_on_success
+let show_timings = !show_timings
+let translate = !translate
+let style = !style
+let compact = !compact
