@@ -18,19 +18,19 @@
 #include <caml/mlvalues.h>
 #include <caml/memory.h>
 #include <caml/osdeps.h>
-#include "unixsupport.h"
+#include "caml/unixsupport.h"
 
-CAMLprim value unix_execv(value path, value args)
+CAMLprim value caml_unix_execv(value path, value args)
 {
   char_os * wpath;
   char_os ** argv;
   caml_unix_check_path(path, "execv");
-  argv = cstringvect(args, "execv");
+  argv = caml_unix_cstringvect(args, "execv");
   wpath = caml_stat_strdup_to_os(String_val(path));
   (void) execv_os(wpath, EXECV_CAST argv);
   caml_stat_free(wpath);
-  cstringvect_free(argv);
-  uerror("execv", path);
+  caml_unix_cstringvect_free(argv);
+  caml_uerror("execv", path);
   return Val_unit;                  /* never reached, but suppress warnings */
                                 /* from smart compilers */
 }
