@@ -15,15 +15,15 @@
 
 #include <caml/mlvalues.h>
 #include <caml/alloc.h>
-#include "unixsupport.h"
+#include "caml/unixsupport.h"
 #include <errno.h>
 
 extern char * getlogin(void);
 
-CAMLprim value unix_getlogin(value unit)
+CAMLprim value caml_unix_getlogin(value unit)
 {
   char * name;
   name = getlogin();
-  if (name == NULL) unix_error(ENOENT, "getlogin", Nothing);
+  if (name == NULL) caml_unix_error(ENOENT, "getlogin", Nothing);
   return caml_copy_string(name);
 }
