@@ -17,15 +17,16 @@
 
 #include <caml/mlvalues.h>
 #include <caml/fail.h>
-#include "unixsupport.h"
+#include "caml/unixsupport.h"
 #include <signal.h>
 #include <caml/signals.h>
 
-CAMLprim value unix_kill(value pid, value signal)
+CAMLprim value caml_unix_kill(value pid, value signal)
 {
   int sig;
   sig = caml_convert_signal_number(Int_val(signal));
   if (kill(Int_val(pid), sig) == -1)
-    uerror("kill", Nothing);
+    caml_uerror("kill", Nothing);
+  caml_process_pending_actions();
   return Val_unit;
 }
