@@ -26,6 +26,9 @@
 #ifdef HAS_UNISTD
 #include <unistd.h>
 #endif
+#ifdef _WIN32
+#include <io.h>
+#endif
 
 #include "caml/mlvalues.h"
 #include "caml/alloc.h"
@@ -37,7 +40,7 @@
 #include "caml/fix_code.h"
 #include "caml/memory.h"
 #include "caml/startup.h"
-#include "caml/stacks.h"
+#include "caml/fiber.h"
 #include "caml/sys.h"
 #include "caml/backtrace.h"
 #include "caml/fail.h"
@@ -73,17 +76,6 @@ enum {
   POS_CNUM = 3
 };
 
-/* Runtime representation of the debug information, optimized
-   for quick lookup */
-struct ev_info {
-  code_t ev_pc;
-  char *ev_filename;
-  char *ev_defname;
-  int ev_lnum;
-  int ev_startchr;
-  int ev_endchr;
-};
-
 struct debug_info {
   code_t start;
   code_t end;
@@ -94,8 +86,7 @@ struct debug_info {
 
 static struct debug_info *find_debug_info(code_t pc)
 {
-  int i;
-  for (i = 0; i < caml_debug_info.size; i++) {
+  for (int i = 0; i < caml_debug_info.size; i++) {
     struct debug_info *di = caml_debug_info.contents[i];
     if (pc >= di->start && pc < di->end)
       return di;
@@ -113,25 +104,27 @@ static int cmp_ev_info(const void *a, const void *b)
   int num_b;
 
   /* Perform a full lexicographic comparison to make sure the resulting order is
-     the same under all implementations of qsort (which is not stable). */
+     the same under all implementations of qsort (which is not stable).
+     NB a->ev_end_offset == b->ev_end_offset =>
+           a->ev_end_lnum == b->ev_end_lnum && a->ev_end_chr == b->ev_end_chr */
 
   if (pc_a > pc_b) return 1;
   if (pc_a < pc_b) return -1;
 
-  num_a = ev_a->ev_lnum;
-  num_b = ev_b->ev_lnum;
+  num_a = ev_a->ev_start_lnum;
+  num_b = ev_b->ev_start_lnum;
 
   if (num_a > num_b) return 1;
   if (num_a < num_b) return -1;
 
-  num_a = ev_a->ev_startchr;
-  num_b = ev_b->ev_startchr;
+  num_a = ev_a->ev_start_chr;
+  num_b = ev_b->ev_start_chr;
 
   if (num_a > num_b) return 1;
   if (num_a < num_b) return -1;
 
-  num_a = ev_a->ev_endchr;
-  num_b = ev_b->ev_endchr;
+  num_a = ev_a->ev_end_offset;
+  num_b = ev_b->ev_end_offset;
 
   if (num_a > num_b) return 1;
   if (num_a < num_b) return -1;
@@ -144,13 +137,13 @@ static struct ev_info *process_debug_events(code_t code_start,
                                             mlsize_t *num_events)
 {
   CAMLparam1(events_heap);
-  CAMLlocal3(l, ev, ev_start);
-  mlsize_t i, j;
+  CAMLlocal4(l, ev, ev_start, ev_end);
+  mlsize_t j;
   struct ev_info *events;
 
   /* Compute the size of the required event buffer. */
   *num_events = 0;
-  for (i = 0; i < caml_array_length(events_heap); i++)
+  for (mlsize_t i = 0; i < caml_array_length(events_heap); i++)
     for (l = Field(events_heap, i); l != Val_int(0); l = Field(l, 1))
       (*num_events)++;
 
@@ -162,7 +155,7 @@ static struct ev_info *process_debug_events(code_t code_start,
     caml_fatal_error ("caml_add_debug_info: out of memory");
 
   j = 0;
-  for (i = 0; i < caml_array_length(events_heap); i++) {
+  for (mlsize_t i = 0; i < caml_array_length(events_heap); i++) {
     for (l = Field(events_heap, i); l != Val_int(0); l = Field(l, 1)) {
       ev = Field(l, 0);
 
@@ -170,6 +163,7 @@ static struct ev_info *process_debug_events(code_t code_start,
                                  + Long_val(Field(ev, EV_POS)));
 
       ev_start = Field(Field(ev, EV_LOC), LOC_START);
+      ev_end = Field(Field(ev, EV_LOC), LOC_END);
 
       {
         const char *fname = String_val(Field(ev_start, POS_FNAME));
@@ -188,13 +182,14 @@ static struct ev_info *process_debug_events(code_t code_start,
         events[j].ev_defname = "<old bytecode>";
       }
 
-      events[j].ev_lnum = Int_val(Field(ev_start, POS_LNUM));
-      events[j].ev_startchr =
-        Int_val(Field(ev_start, POS_CNUM))
-        - Int_val(Field(ev_start, POS_BOL));
-      events[j].ev_endchr =
-        Int_val(Field(Field(Field(ev, EV_LOC), LOC_END), POS_CNUM))
-        - Int_val(Field(ev_start, POS_BOL));
+      events[j].ev_start_lnum = Int_val(Field(ev_start, POS_LNUM));
+      events[j].ev_start_chr =
+        Int_val(Field(ev_start, POS_CNUM)) - Int_val(Field(ev_start, POS_BOL));
+      events[j].ev_end_lnum = Int_val(Field(ev_end, POS_LNUM));
+      events[j].ev_end_chr =
+        Int_val(Field(ev_end, POS_CNUM)) - Int_val(Field(ev_end, POS_BOL));
+      events[j].ev_end_offset =
+        Int_val(Field(ev_end, POS_CNUM)) - Int_val(Field(ev_start, POS_BOL));
 
       j++;
     }
@@ -209,8 +204,7 @@ static struct ev_info *process_debug_events(code_t code_start,
 
 /* Processes a (Instruct.debug_event list array) into a form suitable
    for quick lookup and registers it for the (code_start,code_size) pc range. */
-CAMLprim value caml_add_debug_info(code_t code_start, value code_size,
-                                   value events_heap)
+value caml_add_debug_info(code_t code_start, value code_size, value events_heap)
 {
   CAMLparam1(events_heap);
   struct debug_info *debug_info;
@@ -238,13 +232,12 @@ CAMLprim value caml_add_debug_info(code_t code_start, value code_size,
   CAMLreturn(Val_unit);
 }
 
-CAMLprim value caml_remove_debug_info(code_t start)
+value caml_remove_debug_info(code_t start)
 {
   CAMLparam0();
   CAMLlocal2(dis, prev);
 
-  int i;
-  for (i = 0; i < caml_debug_info.size; i++) {
+  for (int i = 0; i < caml_debug_info.size; i++) {
     struct debug_info *di = caml_debug_info.contents[i];
     if (di->start == start) {
       /* note that caml_ext_table_remove calls caml_stat_free on the
@@ -258,7 +251,8 @@ CAMLprim value caml_remove_debug_info(code_t start)
   CAMLreturn(Val_unit);
 }
 
-int caml_alloc_backtrace_buffer(void){
+int caml_alloc_backtrace_buffer (void)
+{
   CAMLassert(Caml_state->backtrace_pos == 0);
   Caml_state->backtrace_buffer =
     caml_stat_alloc_noexc(BACKTRACE_BUFFER_SIZE * sizeof(code_t));
@@ -266,14 +260,21 @@ int caml_alloc_backtrace_buffer(void){
   return 0;
 }
 
+void caml_free_backtrace_buffer(backtrace_slot *backtrace_buffer) {
+  if (backtrace_buffer != NULL)
+    caml_stat_free(backtrace_buffer);
+}
+
 /* Store the return addresses contained in the given stack fragment
    into the backtrace array */
 
 void caml_stash_backtrace(value exn, value * sp, int reraise)
 {
+  value *trap_sp;
+
   if (exn != Caml_state->backtrace_last_exn || !reraise) {
     Caml_state->backtrace_pos = 0;
-    Caml_state->backtrace_last_exn = exn;
+    caml_modify_generational_global_root(&Caml_state->backtrace_last_exn, exn);
   }
 
   if (Caml_state->backtrace_buffer == NULL &&
@@ -282,7 +283,8 @@ void caml_stash_backtrace(value exn, value * sp, int reraise)
 
   /* Traverse the stack and put all values pointing into bytecode
      into the backtrace buffer. */
-  for (/*nothing*/; sp < Caml_state->trapsp; sp++) {
+  trap_sp = Stack_high(Caml_state->current_stack) + Caml_state->trap_sp_off;
+  for (/*nothing*/; sp < trap_sp; sp++) {
     code_t p;
     if (Is_long(*sp)) continue;
     p = (code_t) *sp;
@@ -293,18 +295,19 @@ void caml_stash_backtrace(value exn, value * sp, int reraise)
 }
 
 /* returns the next frame pointer (or NULL if none is available);
-   updates *sp to point to the following one, and *trsp to the next
+   updates *sp to point to the following one, and *trap_spoff to the next
    trap frame, which we will skip when we reach it  */
 
-code_t caml_next_frame_pointer(value ** sp, value ** trsp)
+code_t caml_next_frame_pointer(value* stack_high, value ** sp,
+                          intnat * trap_spoff)
 {
-  while (*sp < Caml_state->stack_high) {
+  while (*sp < stack_high) {
     value *spv = (*sp)++;
     code_t *p;
     if (Is_long(*spv)) continue;
     p = (code_t*) spv;
-    if(&Trap_pc(*trsp) == p) {
-      *trsp = Trap_link(*trsp);
+    if((code_t*)&Trap_pc(stack_high + *trap_spoff) == p) {
+      *trap_spoff = Trap_link(stack_high + *trap_spoff);
       continue;
     }
 
@@ -314,40 +317,126 @@ code_t caml_next_frame_pointer(value ** sp, value ** trsp)
   return NULL;
 }
 
-#define Default_callstack_size 32
-intnat caml_collect_current_callstack(value** ptrace, intnat* plen,
-                                      intnat max_frames, int alloc_idx)
+/* minimum size to allocate a backtrace (in slots) */
+#define MIN_BACKTRACE_SIZE 16
+
+/* Stores up to [max_slots] backtrace slots of the current call stack
+   to return to the user in [*backtrace_p] (with the allocated size in
+   [*alloc_size_p]). Returns the number of frames stored.  Instead of
+   using a bounded buffer as [Caml_state->stash_backtrace], we
+   dynamically grow the allocated space as required. */
+
+static size_t get_callstack(value* sp, intnat trap_spoff,
+                            struct stack_info* stack,
+                            intnat max_slots,
+                            backtrace_slot **backtrace_p,
+                            size_t *alloc_size_p)
 {
-  value * sp = Caml_state->extern_sp;
-  value * trsp = Caml_state->trapsp;
-  intnat trace_pos = 0;
-  CAMLassert(alloc_idx == 0 || alloc_idx == -1);
+  backtrace_slot *backtrace = *backtrace_p;
+  size_t alloc_size = *alloc_size_p;
+  struct stack_info* parent = Stack_parent(stack);
+  value *stack_high = Stack_high(stack);
+  CAMLnoalloc;
 
-  if (max_frames <= 0) return 0;
-  if (*plen == 0) {
-    value* trace =
-      caml_stat_alloc_noexc(Default_callstack_size * sizeof(value));
-    if (trace == NULL) return 0;
-    *ptrace = trace;
-    *plen = Default_callstack_size;
-  }
+  size_t slots = 0;
+  while (slots < max_slots) {
+    code_t p = caml_next_frame_pointer(stack_high, &sp, &trap_spoff);
+    if (!p) {
+      if (!parent) break;
+      sp = parent->sp;
+      trap_spoff = Long_val(sp[0]);
+      stack_high = Stack_high(parent);
+      parent = Stack_parent(parent);
+    } else {
+      if (slots == alloc_size) {
+        size_t new_size = alloc_size ? alloc_size * 2 : MIN_BACKTRACE_SIZE;
+        backtrace = caml_stat_resize_noexc(backtrace,
+                                           sizeof(backtrace_slot) * new_size);
 
-  while (trace_pos < max_frames) {
-    code_t p = caml_next_frame_pointer(&sp, &trsp);
-    if (p == NULL) break;
-    if (trace_pos == *plen) {
-      intnat new_len = *plen * 2;
-      value * trace = caml_stat_resize_noexc(*ptrace, new_len * sizeof(value));
-      if (trace == NULL) break;
-      *ptrace = trace;
-      *plen = new_len;
+        if (!backtrace) { /* allocation failed */
+          *backtrace_p = NULL;
+          *alloc_size_p = 0;
+          return 0;
+        }
+        alloc_size = new_size;
+      }
+
+      backtrace[slots++] = p;
     }
-    (*ptrace)[trace_pos++] = Val_backtrace_slot(p);
+  }
+  *backtrace_p = backtrace;
+  *alloc_size_p = alloc_size;
+  return slots;
+}
+
+static value alloc_callstack(backtrace_slot *trace, size_t slots)
+{
+  CAMLparam0();
+  CAMLlocal1(callstack);
+  callstack = caml_alloc(slots, 0);
+  for (int i = 0; i < slots; i++)
+    Store_field(callstack, i, Val_backtrace_slot(trace[i]));
+  caml_stat_free(trace);
+  CAMLreturn(callstack);
+}
+
+/* Obtain up to [max_slots] of the callstack of the current domain,
+ * including parent fibers. The callstack is written into [*buffer_p],
+ * current size [*alloc_size_p], which should be reallocated (on the C
+ * heap) if required. Returns the number of slots obtained.
+ *
+ * [alloc_idx] is ignored, and must be negative (this interface is
+ * also used by the native-code runtime, in which [alloc_idx] is
+ * meaningful.
+ */
+
+size_t caml_get_callstack(size_t max_slots,
+                          backtrace_slot **buffer_p,
+                          size_t *alloc_size_p,
+                          ptrdiff_t alloc_idx)
+{
+  CAMLassert(alloc_idx < 1); /* allocation indexes not used in bytecode */
+  return get_callstack(Caml_state->current_stack->sp,
+                       Caml_state->trap_sp_off,
+                       Caml_state->current_stack,
+                       max_slots,
+                       buffer_p, alloc_size_p);
+}
+
+CAMLprim value caml_get_current_callstack (value max_frames_value)
+{
+  backtrace_slot *backtrace = NULL;
+  size_t trace_size = 0;
+  size_t slots = get_callstack(Caml_state->current_stack->sp,
+                               Caml_state->trap_sp_off,
+                               Caml_state->current_stack,
+                               Long_val(max_frames_value),
+                               &backtrace, &trace_size);
+  return alloc_callstack(backtrace, slots);
+}
+
+CAMLprim value caml_get_continuation_callstack (value cont, value max_frames)
+{
+  backtrace_slot *backtrace = NULL;
+  size_t trace_size = 0;
+  size_t slots;
+  struct stack_info *stack;
+  value *sp;
+
+  stack = Ptr_val(caml_continuation_use(cont));
+  {
+    CAMLnoalloc; /* GC must not see the stack outside the cont */
+    sp = stack->sp;
+    slots = get_callstack(sp, Long_val(sp[0]),
+                          stack, Long_val(max_frames),
+                          &backtrace, &trace_size);
+    caml_continuation_replace(cont, stack);
   }
 
-  return trace_pos;
+  return alloc_callstack(backtrace, slots);
 }
 
+
 /* Read the debugging info contained in the current bytecode executable. */
 
 static void read_main_debug_info(struct debug_info *di)
@@ -355,7 +444,7 @@ static void read_main_debug_info(struct debug_info *di)
   CAMLparam0();
   CAMLlocal3(events, evl, l);
   char_os *exec_name;
-  int fd, num_events, orig, i;
+  int fd, num_events, orig;
   struct channel *chan;
   struct exec_trailer trail;
 
@@ -367,18 +456,19 @@ static void read_main_debug_info(struct debug_info *di)
 
      See  https://github.com/ocaml/ocaml/issues/9344 for details.
   */
-  if (caml_cds_file == NULL && caml_byte_program_mode == COMPLETE_EXE)
+  if (caml_params->cds_file == NULL && caml_byte_program_mode == COMPLETE_EXE)
     CAMLreturn0;
 
-  if (caml_cds_file != NULL) {
-    exec_name = caml_cds_file;
+  if (caml_params->cds_file != NULL) {
+    exec_name = (char_os*) caml_params->cds_file;
   } else {
-    exec_name = caml_exe_name;
+    exec_name = (char_os*) caml_params->exe_name;
   }
 
   fd = caml_attempt_open(&exec_name, &trail, 1);
-  if (fd < 0){
-    caml_fatal_error ("executable program file not found");
+  if (fd < 0) {
+    /* Record the failure of caml_attempt_open in di->already-read */
+    di->already_read = fd;
     CAMLreturn0;
   }
 
@@ -389,14 +479,14 @@ static void read_main_debug_info(struct debug_info *di)
     num_events = caml_getword(chan);
     events = caml_alloc(num_events, 0);
 
-    for (i = 0; i < num_events; i++) {
+    for (int i = 0; i < num_events; i++) {
       orig = caml_getword(chan);
       evl = caml_input_val(chan);
       caml_input_val(chan); /* Skip the list of absolute directory names */
       /* Relocate events in event list */
       for (l = evl; l != Val_int(0); l = Field(l, 1)) {
         value ev = Field(l, 0);
-        Field(ev, EV_POS) = Val_long(Long_val(Field(ev, EV_POS)) + orig);
+        Store_field (ev, EV_POS, Val_long(Long_val(Field(ev, EV_POS)) + orig));
       }
       /* Record event list */
       Store_field(events, i, evl);
@@ -405,6 +495,8 @@ static void read_main_debug_info(struct debug_info *di)
     caml_close_channel(chan);
 
     di->events = process_debug_events(caml_start_code, events, &di->num_events);
+  } else {
+    close(fd);
   }
 
   CAMLreturn0;
@@ -416,11 +508,27 @@ CAMLexport void caml_init_debug_info(void)
   caml_add_debug_info(caml_start_code, Val_long(caml_code_size), Val_unit);
 }
 
+CAMLexport void caml_load_main_debug_info(void)
+{
+  if (caml_params->backtrace_enabled > 1) {
+    read_main_debug_info(caml_debug_info.contents[0]);
+  }
+}
+
 int caml_debug_info_available(void)
 {
   return (caml_debug_info.size != 0);
 }
 
+int caml_debug_info_status(void)
+{
+  if (!caml_debug_info_available()) {
+    return 0;
+  } else {
+    return ((struct debug_info *)caml_debug_info.contents[0])->already_read;
+  }
+}
+
 /* Search the event index for the given PC.  Return -1 if not found. */
 
 static struct ev_info *event_for_location(code_t pc)
@@ -456,6 +564,16 @@ static struct ev_info *event_for_location(code_t pc)
   return NULL;
 }
 
+/* Search for event info at the exact given PC. */
+struct ev_info * caml_exact_event_for_location(code_t pc)
+{
+  struct ev_info *ev = event_for_location(pc);
+  if (ev && ev->ev_pc == pc)
+    return ev;
+
+  return NULL;
+}
+
 /* Extract location information for the given PC */
 
 void caml_debuginfo_location(debuginfo dbg,
@@ -474,9 +592,11 @@ void caml_debuginfo_location(debuginfo dbg,
   li->loc_is_inlined = 0;
   li->loc_filename = event->ev_filename;
   li->loc_defname = event->ev_defname;
-  li->loc_lnum = event->ev_lnum;
-  li->loc_startchr = event->ev_startchr;
-  li->loc_endchr = event->ev_endchr;
+  li->loc_start_lnum = event->ev_start_lnum;
+  li->loc_start_chr = event->ev_start_chr;
+  li->loc_end_lnum = event->ev_end_lnum;
+  li->loc_end_chr = event->ev_end_chr;
+  li->loc_end_offset = event->ev_end_offset;
 }
 
 debuginfo caml_debuginfo_extract(backtrace_slot slot)
