@@ -20,13 +20,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include "caml/callback.h"
-#include "caml/backtrace.h"
 #include "caml/custom.h"
+#include "caml/codefrag.h"
 #include "caml/debugger.h"
-#include "caml/domain.h"
-#include "caml/eventlog.h"
+#include "caml/runtime_events.h"
+#include "caml/fiber.h"
 #include "caml/fail.h"
-#include "caml/freelist.h"
 #include "caml/gc.h"
 #include "caml/gc_ctrl.h"
 #include "caml/intext.h"
@@ -35,119 +34,86 @@
 #include "caml/mlvalues.h"
 #include "caml/osdeps.h"
 #include "caml/printexc.h"
+#include "caml/signals.h"
 #include "caml/stack.h"
 #include "caml/startup_aux.h"
 #include "caml/sys.h"
-#ifdef WITH_SPACETIME
-#include "caml/spacetime.h"
-#endif
-#ifdef HAS_UI
-#include "caml/ui.h"
-#endif
 
-extern int caml_parser_trace;
-char * caml_code_area_start, * caml_code_area_end;
-struct ext_table caml_code_fragments_table;
+extern char caml_system__code_begin, caml_system__code_end;
+/* The two symbols above are defined in runtime/$ARCH.S.
+   They use the old `__` separator convention because the new convention
+   gives `caml_system.code_begin`, which is not a valid C identifier. */
 
-/* Initialize the atom table and the static data and code area limits. */
+/* Initialize the static data and code area limits. */
 
 struct segment { char * begin; char * end; };
 
-static void init_static(void)
+static void init_segments(void)
 {
-  extern struct segment caml_data_segments[], caml_code_segments[];
-  int i;
-  struct code_fragment * cf;
-
-  caml_init_atom_table ();
-
-  for (i = 0; caml_data_segments[i].begin != 0; i++) {
-    /* PR#5509: we must include the zero word at end of data segment,
-       because pointers equal to caml_data_segments[i].end are static data. */
-    if (caml_page_table_add(In_static_data,
-                            caml_data_segments[i].begin,
-                            caml_data_segments[i].end + sizeof(value)) != 0)
-      caml_fatal_error("not enough memory for initial page table");
-  }
+  extern struct segment caml_code_segments[];
+  char * caml_code_area_start, * caml_code_area_end;
 
   caml_code_area_start = caml_code_segments[0].begin;
   caml_code_area_end = caml_code_segments[0].end;
-  for (i = 1; caml_code_segments[i].begin != 0; i++) {
+  for (int i = 1; caml_code_segments[i].begin != 0; i++) {
     if (caml_code_segments[i].begin < caml_code_area_start)
       caml_code_area_start = caml_code_segments[i].begin;
     if (caml_code_segments[i].end > caml_code_area_end)
       caml_code_area_end = caml_code_segments[i].end;
   }
   /* Register the code in the table of code fragments */
-  cf = caml_stat_alloc(sizeof(struct code_fragment));
-  cf->code_start = caml_code_area_start;
-  cf->code_end = caml_code_area_end;
-  cf->digest_computed = 0;
-  caml_ext_table_init(&caml_code_fragments_table, 8);
-  caml_ext_table_add(&caml_code_fragments_table, cf);
+  caml_register_code_fragment(caml_code_area_start,
+                              caml_code_area_end,
+                              DIGEST_LATER, NULL);
+  /* Also register the glue code written in assembly */
+  caml_register_code_fragment(&caml_system__code_begin,
+                              &caml_system__code_end,
+                              DIGEST_IGNORE, NULL);
 }
 
-/* These are termination hooks used by the systhreads library */
-struct longjmp_buffer caml_termination_jmpbuf;
-void (*caml_termination_hook)(void *) = NULL;
-
 extern value caml_start_program (caml_domain_state*);
-extern void caml_init_ieee_floats (void);
-extern void caml_init_signals (void);
 #ifdef _WIN32
 extern void caml_win32_overflow_detection (void);
 #endif
 
-#if defined(_MSC_VER) && __STDC_SECURE_LIB__ >= 200411L
+#ifdef _MSC_VER
 
 /* PR 4887: avoid crash box of windows runtime on some system calls */
-extern void caml_install_invalid_parameter_handler();
+extern void caml_install_invalid_parameter_handler(void);
 
 #endif
 
 value caml_startup_common(char_os **argv, int pooling)
 {
   char_os * exe_name, * proc_self_exe;
-  char tos;
+  value res;
 
-  /* Initialize the domain */
-  caml_init_domain();
   /* Determine options */
-#ifdef DEBUG
-  caml_verb_gc = 0x3F;
-#endif
   caml_parse_ocamlrunparam();
-  CAML_EVENTLOG_INIT();
-#ifdef DEBUG
-  caml_gc_message (-1, "### OCaml runtime: debug mode ###\n");
-#endif
-  if (caml_cleanup_on_exit)
+
+  if (caml_params->cleanup_on_exit)
     pooling = 1;
   if (!caml_startup_aux(pooling))
     return Val_unit;
 
-#ifdef WITH_SPACETIME
-  caml_spacetime_initialize();
-#endif
-  caml_init_frame_descriptors();
-  caml_init_ieee_floats();
+  caml_init_codefrag();
   caml_init_locale();
-#if defined(_MSC_VER) && __STDC_SECURE_LIB__ >= 200411L
+#ifdef _MSC_VER
   caml_install_invalid_parameter_handler();
 #endif
   caml_init_custom_operations();
-  Caml_state->top_of_stack = &tos;
-  caml_init_gc (caml_init_minor_heap_wsz, caml_init_heap_wsz,
-                caml_init_heap_chunk_sz, caml_init_percent_free,
-                caml_init_max_percent_free, caml_init_major_window,
-                caml_init_custom_major_ratio, caml_init_custom_minor_ratio,
-                caml_init_custom_minor_max_bsz);
-  init_static();
+  caml_init_os_params();
+  caml_init_gc ();
+
+  /* runtime_events's init can cause a stop-the-world pause, so it must be done
+     after we've initialised the garbage collector */
+  CAML_RUNTIME_EVENTS_INIT();
+
+  init_segments();
   caml_init_signals();
 #ifdef _WIN32
   caml_win32_overflow_detection();
 #endif
-  caml_init_backtrace();
   caml_debugger_init (); /* force debugger.o stub to be linked */
   exe_name = argv[0];
   if (exe_name == NULL) exe_name = T("");
@@ -157,11 +123,10 @@ value caml_startup_common(char_os **argv, int pooling)
   else
     exe_name = caml_search_exe_in_path(exe_name);
   caml_sys_init(exe_name, argv);
-  if (sigsetjmp(caml_termination_jmpbuf.buf, 0)) {
-    if (caml_termination_hook != NULL) caml_termination_hook(NULL);
-    return Val_unit;
-  }
-  return caml_start_program(Caml_state);
+  caml_maybe_expand_stack();
+  res = caml_start_program(Caml_state);
+  caml_terminate_signals();
+  return res;
 }
 
 value caml_startup_exn(char_os **argv)
