@@ -36,10 +36,13 @@ type spec =
                                   call the function with the symbol. *)
   | Rest of (string -> unit)   (* Stop interpreting keywords and call the
                                   function with each remaining argument *)
+  | Rest_all of (string list -> unit)
+                               (* Stop interpreting keywords and call the
+                                  function with all remaining arguments. *)
   | Expand of (string -> string array) (* If the remaining arguments to process
                                           are of the form
-                                          [["-foo"; "arg"] @ rest] where "foo" is
-                                          registered as [Expand f], then the
+                                          [["-foo"; "arg"] @ rest] where "foo"
+                                          is registered as [Expand f], then the
                                           arguments [f "arg" @ rest] are
                                           processed. Only allowed in
                                           [parse_and_expand_argv_dynamic]. *)
@@ -129,7 +132,8 @@ let float_of_string_opt x =
   try Some (float_of_string x)
   with Failure _ -> None
 
-let parse_and_expand_argv_dynamic_aux allow_expand current argv speclist anonfun errmsg =
+let parse_and_expand_argv_dynamic_aux allow_expand current argv speclist anonfun
+                                      errmsg =
   let initpos = !current in
   let convert_error error =
     (* convert an internal error to a Bad/Help exception
@@ -137,7 +141,8 @@ let parse_and_expand_argv_dynamic_aux allow_expand current argv speclist anonfun
        to an user-raised Bad exception.
     *)
     let b = Buffer.create 200 in
-    let progname = if initpos < (Array.length !argv) then !argv.(initpos) else "(?)" in
+    let progname =
+      if initpos < (Array.length !argv) then !argv.(initpos) else "(?)" in
     begin match error with
       | Unknown "-help" -> ()
       | Unknown "--help" -> ()
@@ -160,7 +165,7 @@ let parse_and_expand_argv_dynamic_aux allow_expand current argv speclist anonfun
   while !current < (Array.length !argv) do
     begin try
       let s = !argv.(!current) in
-      if String.length s >= 1 && s.[0] = '-' then begin
+      if String.starts_with ~prefix:"-" s then begin
         let action, follow =
           try assoc3 s !speclist, None
           with Not_found ->
@@ -186,7 +191,7 @@ let parse_and_expand_argv_dynamic_aux allow_expand current argv speclist anonfun
           | Some _ -> ()
         in
         let rec treat_action = function
-        | Unit f -> f ();
+        | Unit f -> no_arg (); f ();
         | Bool f ->
             let arg = get_arg () in
             begin match bool_of_string_opt arg with
@@ -241,20 +246,33 @@ let parse_and_expand_argv_dynamic_aux allow_expand current argv speclist anonfun
             end;
             consume_arg ();
         | Tuple specs ->
+            no_arg ();
             List.iter treat_action specs;
         | Rest f ->
+            no_arg ();
             while !current < (Array.length !argv) - 1 do
               f !argv.(!current + 1);
               consume_arg ();
             done;
+        | Rest_all f ->
+            no_arg ();
+            let acc = ref [] in
+            while !current < Array.length !argv - 1 do
+              acc := !argv.(!current + 1) :: !acc;
+              consume_arg ();
+            done;
+            f (List.rev !acc)
         | Expand f ->
             if not allow_expand then
-              raise (Invalid_argument "Arg.Expand is is only allowed with Arg.parse_and_expand_argv_dynamic");
+              raise (Invalid_argument "Arg.Expand is is only allowed with \
+                                       Arg.parse_and_expand_argv_dynamic");
             let arg = get_arg () in
             let newarg = f arg in
             consume_arg ();
             let before = Array.sub !argv 0 (!current + 1)
-            and after = Array.sub !argv (!current + 1) ((Array.length !argv) - !current - 1) in
+            and after =
+              Array.sub !argv (!current + 1)
+                        ((Array.length !argv) - !current - 1) in
             argv:= Array.concat [before;newarg;after];
         in
         treat_action action end
@@ -269,7 +287,8 @@ let parse_and_expand_argv_dynamic current argv speclist anonfun errmsg =
   parse_and_expand_argv_dynamic_aux true current argv speclist anonfun errmsg
 
 let parse_argv_dynamic ?(current=current) argv speclist anonfun errmsg =
-  parse_and_expand_argv_dynamic_aux false current (ref argv) speclist anonfun errmsg
+  parse_and_expand_argv_dynamic_aux false current (ref argv) speclist anonfun
+    errmsg
 
 
 let parse_argv ?(current=current) argv speclist anonfun errmsg =
@@ -309,16 +328,25 @@ let second_word s =
     else if s.[n] = ' ' then loop (n+1)
     else n
   in
-  try loop (String.index s ' ')
-  with Not_found -> len
+  match String.index s '\t' with
+  | n -> loop (n+1)
+  | exception Not_found ->
+      begin match String.index s ' ' with
+      | n -> loop (n+1)
+      | exception Not_found -> len
+      end
 
 
 let max_arg_len cur (kwd, spec, doc) =
   match spec with
-  | Symbol _ -> max cur (String.length kwd)
-  | _ -> max cur (String.length kwd + second_word doc)
+  | Symbol _ -> Int.max cur (String.length kwd)
+  | _ -> Int.max cur (String.length kwd + second_word doc)
 
 
+let replace_leading_tab s =
+  let seen = ref false in
+  String.map (function '\t' when not !seen -> seen := true; ' ' | c -> c) s
+
 let add_padding len ksd =
   match ksd with
   | (_, _, "") ->
@@ -327,17 +355,17 @@ let add_padding len ksd =
       ksd
   | (kwd, (Symbol _ as spec), msg) ->
       let cutcol = second_word msg in
-      let spaces = String.make ((max 0 (len - cutcol)) + 3) ' ' in
-      (kwd, spec, "\n" ^ spaces ^ msg)
+      let spaces = String.make ((Int.max 0 (len - cutcol)) + 3) ' ' in
+      (kwd, spec, "\n" ^ spaces ^ replace_leading_tab msg)
   | (kwd, spec, msg) ->
       let cutcol = second_word msg in
       let kwd_len = String.length kwd in
       let diff = len - kwd_len - cutcol in
       if diff <= 0 then
-        (kwd, spec, msg)
+        (kwd, spec, replace_leading_tab msg)
       else
         let spaces = String.make diff ' ' in
-        let prefix = String.sub msg 0 cutcol in
+        let prefix = String.sub (replace_leading_tab msg) 0 cutcol in
         let suffix = String.sub msg cutcol (String.length msg - cutcol) in
         (kwd, spec, prefix ^ spaces ^ suffix)
 
@@ -345,7 +373,7 @@ let add_padding len ksd =
 let align ?(limit=max_int) speclist =
   let completed = add_help speclist in
   let len = List.fold_left max_arg_len 0 completed in
-  let len = min len limit in
+  let len = Int.min len limit in
   List.map (add_padding len) completed
 
 let trim_cr s =
@@ -360,23 +388,19 @@ let read_aux trim sep file =
   let buf = Buffer.create 200 in
   let words = ref [] in
   let stash () =
-    let word =  (Buffer.contents buf) in
+    let word = Buffer.contents buf in
     let word = if trim then trim_cr word else word in
     words := word :: !words;
     Buffer.clear buf
   in
-  let rec read () =
-    try
-      let c = input_char ic in
-      if c = sep then begin
-        stash (); read ()
-      end else begin
-        Buffer.add_char buf c; read ()
-      end
-    with End_of_file ->
-      if Buffer.length buf > 0 then
-        stash () in
-  read ();
+  begin
+    try while true do
+        let c = input_char ic in
+        if c = sep then stash () else Buffer.add_char buf c
+      done
+    with End_of_file -> ()
+  end;
+  if Buffer.length buf > 0 then stash ();
   close_in ic;
   Array.of_list (List.rev !words)
 
