@@ -23,7 +23,9 @@ val parent_dir_name : string
    (e.g. [..] in Unix). *)
 
 val dir_sep : string
-(** The directory separator (e.g. [/] in Unix). @since 3.11.2 *)
+(** The directory separator (e.g. [/] in Unix).
+
+    @since 3.11.2 *)
 
 val concat : string -> string -> string
 (** [concat dir file] returns a file name that designates file
@@ -51,9 +53,8 @@ val check_suffix : string -> string -> bool
 
 val chop_suffix : string -> string -> string
 (** [chop_suffix name suff] removes the suffix [suff] from
-   the filename [name]. The behavior is undefined if [name] does not
-   end with the suffix [suff]. [chop_suffix_opt] is thus recommended
-   instead.
+    the filename [name].
+    @raise Invalid_argument if [name] does not end with the suffix [suff].
 *)
 
 val chop_suffix_opt: suffix:string -> string -> string option
@@ -118,6 +119,12 @@ val dirname : string -> string
    This function conforms to the specification of POSIX.1-2008 for the
    [dirname] utility. *)
 
+val null : string
+(** [null] is ["/dev/null"] on POSIX and ["NUL"] on Windows. It represents a
+    file on the OS that discards all writes and returns end of file on reads.
+
+    @since 4.10 *)
+
 val temp_file : ?temp_dir: string -> string -> string -> string
 (** [temp_file prefix suffix] returns the name of a
    fresh temporary file in the temporary directory.
@@ -129,7 +136,7 @@ val temp_file : ?temp_dir: string -> string -> string -> string
    (readable and writable only by the file owner).  The file is
    guaranteed to be different from any other file that existed when
    [temp_file] was called.
-   Raise [Sys_error] if the file could not be created.
+   @raise Sys_error if the file could not be created.
    @before 3.11.2 no ?temp_dir optional argument
 *)
 
@@ -148,10 +155,30 @@ val open_temp_file :
    writable only by the file owner, [0o600]).
 
    @raise Sys_error if the file could not be opened.
-   @before 4.03.0 no ?perms optional argument
+   @before 4.03 no ?perms optional argument
    @before 3.11.2 no ?temp_dir optional argument
 *)
 
+val temp_dir : ?temp_dir: string -> ?perms:int  -> string -> string -> string
+(** [temp_dir prefix suffix] creates and returns the name of a fresh
+   temporary directory with permissions [perms] (defaults to 0o700)
+   inside [temp_dir].  The base name of the temporary directory is
+   formed by concatenating [prefix], then a suitably chosen integer
+   number, then [suffix].  The optional argument [temp_dir] indicates
+   the temporary directory to use, defaulting to the current result of
+   {!Filename.get_temp_dir_name}.  The temporary directory is created
+   empty, with permissions [0o700] (readable, writable, and searchable
+   only by the file owner).  The directory is guaranteed to be
+   different from any other directory that existed when [temp_dir] was
+   called.
+
+   If temp_dir does not exist, this function does not create it.  Instead,
+   it raises Sys_error.
+
+   @raise Sys_error if the directory could not be created.
+   @since 5.1
+*)
+
 val get_temp_dir_name : unit -> string
 (** The name of the temporary directory:
     Under Unix, the value of the [TMPDIR] environment variable, or "/tmp"
@@ -159,24 +186,15 @@ val get_temp_dir_name : unit -> string
     Under Windows, the value of the [TEMP] environment variable, or "."
     if the variable is not set.
     The temporary directory can be changed with {!Filename.set_temp_dir_name}.
-    @since 4.00.0
+    @since 4.00
 *)
 
 val set_temp_dir_name : string -> unit
 (** Change the temporary directory returned by {!Filename.get_temp_dir_name}
     and used by {!Filename.temp_file} and {!Filename.open_temp_file}.
-    @since 4.00.0
-*)
-
-val temp_dir_name : string
-  [@@ocaml.deprecated "Use Filename.get_temp_dir_name instead"]
-(** The name of the initial temporary directory:
-    Under Unix, the value of the [TMPDIR] environment variable, or "/tmp"
-    if the variable is not set.
-    Under Windows, the value of the [TEMP] environment variable, or "."
-    if the variable is not set.
-    @deprecated You should use {!Filename.get_temp_dir_name} instead.
-    @since 3.09.1
+    The temporary directory is a domain-local value which is inherited
+    by child domains.
+    @since 4.00
 *)
 
 val quote : string -> string
@@ -186,3 +204,35 @@ val quote : string -> string
     with programs that follow the standard Windows quoting
     conventions.
  *)
+
+val quote_command :
+       string -> ?stdin:string -> ?stdout:string -> ?stderr:string
+              -> string list -> string
+(** [quote_command cmd args] returns a quoted command line, suitable
+    for use as an argument to {!Sys.command}, {!Unix.system}, and the
+    {!Unix.open_process} functions.
+
+    The string [cmd] is the command to call.  The list [args] is
+    the list of arguments to pass to this command.  It can be empty.
+
+    The optional arguments [?stdin] and [?stdout] and [?stderr] are
+    file names used to redirect the standard input, the standard
+    output, or the standard error of the command.
+    If [~stdin:f] is given, a redirection [< f] is performed and the
+    standard input of the command reads from file [f].
+    If [~stdout:f] is given, a redirection [> f] is performed and the
+    standard output of the command is written to file [f].
+    If [~stderr:f] is given, a redirection [2> f] is performed and the
+    standard error of the command is written to file [f].
+    If both [~stdout:f] and [~stderr:f] are given, with the exact
+    same file name [f], a [2>&1] redirection is performed so that the
+    standard output and the standard error of the command are interleaved
+    and redirected to the same file [f].
+
+    Under Unix and Cygwin, the command, the arguments, and the redirections
+    if any are quoted using {!Filename.quote}, then concatenated.
+    Under Win32, additional quoting is performed as required by the
+    [cmd.exe] shell that is called by {!Sys.command}.
+    @raise Failure if the command cannot be escaped on the current platform.
+    @since 4.10
+*)
