@@ -17,12 +17,13 @@ type extern_flags =
     No_sharing
   | Closures
   | Compat_32
-(* note: this type definition is used in 'byterun/debugger.c' *)
+
+(* note: this type definition is used in 'runtime/debugger.c' *)
 
 external to_channel: out_channel -> 'a -> extern_flags list -> unit
     = "caml_output_value"
 external to_bytes: 'a -> extern_flags list -> bytes
-    = "caml_output_value_to_string"
+    = "caml_output_value_to_bytes"
 external to_string: 'a -> extern_flags list -> string
     = "caml_output_value_to_string"
 external to_buffer_unsafe:
@@ -41,11 +42,10 @@ let to_buffer buff ofs len v flags =
 *)
 
 external from_channel: in_channel -> 'a = "caml_input_value"
-external from_bytes_unsafe: bytes -> int -> 'a
-                           = "caml_input_value_from_string"
+external from_bytes_unsafe: bytes -> int -> 'a = "caml_input_value_from_bytes"
 external data_size_unsafe: bytes -> int -> int = "caml_marshal_data_size"
 
-let header_size = 20
+let header_size = 16
 let data_size buff ofs =
   if ofs < 0 || ofs > Bytes.length buff - header_size
   then invalid_arg "Marshal.data_size"
