@@ -26,8 +26,6 @@ module type S =
     type elt
     type t
     val empty: t
-    val is_empty: t -> bool
-    val mem: elt -> t -> bool
     val add: elt -> t -> t
     val singleton: elt -> t
     val remove: elt -> t -> t
@@ -35,17 +33,6 @@ module type S =
     val inter: t -> t -> t
     val disjoint: t -> t -> bool
     val diff: t -> t -> t
-    val compare: t -> t -> int
-    val equal: t -> t -> bool
-    val subset: t -> t -> bool
-    val iter: (elt -> unit) -> t -> unit
-    val map: (elt -> elt) -> t -> t
-    val fold: (elt -> 'a -> 'a) -> t -> 'a -> 'a
-    val for_all: (elt -> bool) -> t -> bool
-    val exists: (elt -> bool) -> t -> bool
-    val filter: (elt -> bool) -> t -> t
-    val filter_map: (elt -> elt option) -> t -> t
-    val partition: (elt -> bool) -> t -> t * t
     val cardinal: t -> int
     val elements: t -> elt list
     val min_elt: t -> elt
@@ -54,16 +41,31 @@ module type S =
     val max_elt_opt: t -> elt option
     val choose: t -> elt
     val choose_opt: t -> elt option
-    val split: elt -> t -> t * bool * t
     val find: elt -> t -> elt
     val find_opt: elt -> t -> elt option
     val find_first: (elt -> bool) -> t -> elt
     val find_first_opt: (elt -> bool) -> t -> elt option
     val find_last: (elt -> bool) -> t -> elt
     val find_last_opt: (elt -> bool) -> t -> elt option
+    val iter: (elt -> unit) -> t -> unit
+    val fold: (elt -> 'a -> 'a) -> t -> 'a -> 'a
+    val map: (elt -> elt) -> t -> t
+    val filter: (elt -> bool) -> t -> t
+    val filter_map: (elt -> elt option) -> t -> t
+    val partition: (elt -> bool) -> t -> t * t
+    val split: elt -> t -> t * bool * t
+    val is_empty: t -> bool
+    val mem: elt -> t -> bool
+    val equal: t -> t -> bool
+    val compare: t -> t -> int
+    val subset: t -> t -> bool
+    val for_all: (elt -> bool) -> t -> bool
+    val exists: (elt -> bool) -> t -> bool
+    val to_list : t -> elt list
     val of_list: elt list -> t
     val to_seq_from : elt -> t -> elt Seq.t
     val to_seq : t -> elt Seq.t
+    val to_rev_seq : t -> elt Seq.t
     val add_seq : elt Seq.t -> t -> t
     val of_seq : elt Seq.t -> t
   end
@@ -573,6 +575,8 @@ module Make(Ord: OrderedType) =
       in
       fst (sub (List.length l) l)
 
+    let to_list = elements
+
     let of_list l =
       match l with
       | [] -> empty
@@ -594,6 +598,17 @@ module Make(Ord: OrderedType) =
 
     let to_seq c = seq_of_enum_ (cons_enum c End)
 
+    let rec snoc_enum s e =
+      match s with
+        Empty -> e
+      | Node{l; v; r} -> snoc_enum r (More(v, l, e))
+
+    let rec rev_seq_of_enum_ c () = match c with
+      | End -> Seq.Nil
+      | More (x, t, rest) -> Seq.Cons (x, rev_seq_of_enum_ (snoc_enum t rest))
+
+    let to_rev_seq c = rev_seq_of_enum_ (snoc_enum c End)
+
     let to_seq_from low s =
       let rec aux low s c = match s with
         | Empty -> c
