@@ -1,7 +1,6 @@
-(* TEST
-   flags = "-g"
-   ocamlrunparam += ",b=1"
-   compare_programs = "false"
+(* TEST_BELOW
+(* Blank lines added here to preserve locations. *)
+
 *)
 
 exception Exn
@@ -44,7 +43,11 @@ let run f =
     Printf.printf "---------------------------\n%!"
 
 let _ =
-  Printexc.record_backtrace true;
   run without_reraise;
   run with_reraise;
   run trickier
+
+(* TEST
+ flags = "-g";
+ ocamlrunparam += ",b=1";
+*)
