@@ -1,7 +1,6 @@
-(* TEST
-   flags = "-g"
-   ocamlrunparam += ",b=1"
-   compare_programs = "false"
+(* TEST_BELOW
+(* Blank lines added here to preserve locations. *)
+
 *)
 
 (* A test for stack backtraces *)
@@ -58,9 +57,13 @@ let run args =
           | Some line -> print_endline line)
 
 let _ =
-  Printexc.record_backtrace true;
   run [| "a" |];
   run [| "b" |];
   run [| "c" |];
   run [| "d" |];
   run [| |]
+
+(* TEST
+ flags = "-g";
+ ocamlrunparam += ",b=1";
+*)
