@@ -1,12 +1,9 @@
 (* TEST
-   flags = "-g -inline 0"
-   ocamlrunparam += ",b=1"
-   compare_programs = "false"
-   * native
+ flags = "-g -inline 0";
+ ocamlrunparam += ",b=1";
+ native;
 *)
 
-let () = Printexc.record_backtrace true
-
 let finaliser _ = try raise Exit with _ -> ()
 
 let create () =
