@@ -1,9 +1,8 @@
-(* TEST
-   flags = "-g"
-   ocamlrunparam += ",b=1"
-   ocamlopt_flags = "-inline 0"
-   exit_status = "2"
-   compare_programs = "false"
+(* TEST_BELOW
+(* Blank lines added here to preserve locations. *)
+
+
+
 *)
 
 let why : unit -> unit = fun () -> raise Exit [@@inline never]
@@ -15,5 +14,11 @@ let f () =
   () [@@inline never]
 
 let () =
-  Printexc.record_backtrace true;
   f ()
+
+(* TEST
+ flags = "-g";
+ ocamlrunparam += ",b=1";
+ ocamlopt_flags = "-inline 0";
+ exit_status = "2";
+*)
