@@ -1,7 +1,6 @@
-(* TEST
-   flags = "-g"
-   ocamlrunparam += ",b=1"
-   compare_programs = "false"
+(* TEST_BELOW
+(* Blank lines added here to preserve locations. *)
+
 *)
 
 (* A test for stack backtraces *)
@@ -54,7 +53,6 @@ let run args =
       flush stdout
 
 let _ =
-  Printexc.record_backtrace true;
   run [| "a" |];
   run [| "b" |];
   run [| "c" |];
@@ -62,3 +60,8 @@ let _ =
   run [| "e" |];
   run [| "f" |];
   run [| |]
+
+(* TEST
+ flags = "-g";
+ ocamlrunparam += ",b=1";
+*)
