@@ -1,3 +1,7 @@
+(* TEST
+ include testing;
+*)
+
 let r = ref 0
 
 let true_effect () =
