@@ -1,5 +1,5 @@
 (* TEST
-   * expect
+ expect;
 *)
 
 type tlist = { x: 'a. 'a list };;
@@ -39,9 +39,10 @@ Lines 1-3, characters 0-20:
 1 | match { x = assert false } with
 2 | | { x = 3 } -> ()
 3 | | { x = None } -> ()
-Warning 8: this pattern-matching is not exhaustive.
+Warning 8 [partial-match]: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 {x=Some _}
+
 Exception: Assert_failure ("", 1, 12).
 |}];;
 
@@ -54,9 +55,10 @@ Lines 1-3, characters 0-18:
 1 | match { x = assert false } with
 2 | | { x = None } -> ()
 3 | | { x = "" } -> ()
-Warning 8: this pattern-matching is not exhaustive.
+Warning 8 [partial-match]: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 {x="*"}
+
 Exception: Assert_failure ("", 1, 12).
 |}];;
 
@@ -69,9 +71,10 @@ Lines 1-3, characters 0-18:
 1 | match { x = assert false } with
 2 | | { x = None } -> ()
 3 | | { x = `X } -> ()
-Warning 8: this pattern-matching is not exhaustive.
+Warning 8 [partial-match]: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 {x=`AnyOtherTag}
+
 Exception: Assert_failure ("", 1, 12).
 |}];;
 
@@ -84,9 +87,10 @@ Lines 1-3, characters 0-17:
 1 | match { x = assert false } with
 2 | | { x = [||] } -> ()
 3 | | { x = 3 } -> ()
-Warning 8: this pattern-matching is not exhaustive.
+Warning 8 [partial-match]: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 {x=0}
+
 Exception: Assert_failure ("", 1, 12).
 |}];;
 
@@ -99,9 +103,10 @@ Lines 1-3, characters 0-17:
 1 | match { x = assert false } with
 2 | | { x = `X } -> ()
 3 | | { x = 3 } -> ()
-Warning 8: this pattern-matching is not exhaustive.
+Warning 8 [partial-match]: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 {x=0}
+
 Exception: Assert_failure ("", 1, 12).
 |}];;
 
@@ -114,9 +119,10 @@ Lines 1-3, characters 0-17:
 1 | match { x = assert false } with
 2 | | { x = `X "lol" } -> ()
 3 | | { x = 3 } -> ()
-Warning 8: this pattern-matching is not exhaustive.
+Warning 8 [partial-match]: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 {x=0}
+
 Exception: Assert_failure ("", 1, 12).
 |}];;
 
@@ -131,8 +137,9 @@ Lines 1-4, characters 0-17:
 2 | | { x = (2., "") } -> ()
 3 | | { x = None } -> ()
 4 | | { x = 3 } -> ()
-Warning 8: this pattern-matching is not exhaustive.
+Warning 8 [partial-match]: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 {x=0}
+
 Exception: Assert_failure ("", 1, 12).
 |}];;
