@@ -14,9 +14,9 @@
 /**************************************************************************/
 
 #include <signal.h>
-#include "caml/mlvalues.h"
-#include "caml/memory.h"
-#include "caml/callback.h"
+#include <caml/mlvalues.h>
+#include <caml/memory.h>
+#include <caml/callback.h>
 
 value mycallback1(value fun, value arg)
 {
