@@ -1,11 +1,14 @@
 (* TEST
-   include unix
-   * libunix
-   ** bytecode
-   ** native
+ include unix;
+ modules = "callbackprim.c";
+ libunix;
+ {
+   bytecode;
+ }{
+   native;
+ }
 *)
-
-let pid = Unix.getpid ()
+external raise_sigusr1 : unit -> unit = "raise_sigusr1"
 
 let do_test () =
   let seen_states = Array.make 5 (-1) in
@@ -19,12 +22,13 @@ let do_test () =
   seen_states.(!pos) <- 0; pos := !pos + 1;
   Sys.set_signal Sys.sigusr1 (Sys.Signal_handle sighandler);
   seen_states.(!pos) <- 1; pos := !pos + 1;
-  Unix.kill pid Sys.sigusr1;
+  raise_sigusr1 ();
   seen_states.(!pos) <- 2; pos := !pos + 1;
   let _ = Sys.opaque_identity (ref 1) in
   seen_states.(!pos) <- 4; pos := !pos + 1;
   Sys.set_signal Sys.sigusr1 Sys.Signal_default;
-  assert (seen_states = [|0;1;2;3;4|])
+  Array.iter (Printf.printf "%d") seen_states;
+  print_newline ()
 
 let () =
   for _ = 0 to 10 do do_test () done;
