@@ -1,5 +1,6 @@
 (* TEST
-   * toplevel
+ flags = "-no-strict-formats";
+ toplevel;
 *)
 
 (* test whether padding modifiers are accepted without any padding
