@@ -1,5 +1,5 @@
 (* TEST
-   * expect
+ expect;
 *)
 
 type t = A
@@ -12,7 +12,7 @@ module M = struct
     type t = B of t * t' | C
 end
 [%%expect{|
-module M : sig type t = B of t/1 * t/2 | C end
+module M : sig type t = B of t * t/2 | C end
 |}]
 
 (* test *)
@@ -76,8 +76,7 @@ end
 Line 3, characters 7-20:
 3 |   open M(struct end)
            ^^^^^^^^^^^^^
-Error: This module is not a structure; it has type
-       functor (X : sig end) -> sig end
+Error: This module is not a structure; it has type "(X : sig end) -> sig end"
 |}]
 
 open struct
@@ -103,9 +102,11 @@ include struct open struct type t = T end let x = T end
 Line 1, characters 15-41:
 1 | include struct open struct type t = T end let x = T end
                    ^^^^^^^^^^^^^^^^^^^^^^^^^^
-Error: The type t/337 introduced by this open appears in the signature
-       Line 1, characters 46-47:
-         The value x has no valid type if t/337 is hidden
+Error: The type "t" introduced by this open appears in the signature.
+Line 1, characters 46-47:
+1 | include struct open struct type t = T end let x = T end
+                                                  ^
+  The value "x" has no valid type if "t" is hidden.
 |}];;
 
 module A = struct
@@ -123,9 +124,11 @@ Lines 3-6, characters 4-7:
 4 |       type t = T
 5 |       let x = T
 6 |     end
-Error: The type t/342 introduced by this open appears in the signature
-       Line 7, characters 8-9:
-         The value y has no valid type if t/342 is hidden
+Error: The type "t" introduced by this open appears in the signature.
+Line 7, characters 8-9:
+7 |     let y = x
+            ^
+  The value "y" has no valid type if "t" is hidden.
 |}];;
 
 module A = struct
@@ -142,9 +145,11 @@ Lines 3-5, characters 4-7:
 3 | ....open struct
 4 |       type t = T
 5 |     end
-Error: The type t/347 introduced by this open appears in the signature
-       Line 6, characters 8-9:
-         The value y has no valid type if t/347 is hidden
+Error: The type "t" introduced by this open appears in the signature.
+Line 6, characters 8-9:
+6 |     let y = T
+            ^
+  The value "y" has no valid type if "t" is hidden.
 |}]
 
 (* It was decided to not allow this anymore. *)
@@ -209,8 +214,8 @@ module F(X:S) : T = X
 module G(X:T) : S = X
 [%%expect{|
 module type T = sig type s = int end
-module F : functor (X : S) -> T
-module G : functor (X : T) -> S
+module F : (X : S) -> T
+module G : (X : T) -> S
 |}]
 
 module Counter : sig val inc : unit -> unit val current : unit -> int val z : int val zz : int end = struct
@@ -355,8 +360,8 @@ let x = let open struct type t = T end in T
 Line 1, characters 42-43:
 1 | let x = let open struct type t = T end in T
                                               ^
-Error: This expression has type t but an expression was expected of type 'a
-       The type constructor t would escape its scope
+Error: The constructor "T" has type "t" but an expression was expected of type "'a"
+       The type constructor "t" would escape its scope
 |}]
 
 module type Print = sig
@@ -377,7 +382,7 @@ let print_list_of_int = let open Print_list(Print_int) in print
 module type Print = sig type t val print : t -> unit end
 module Print_int : sig type t = int val print : t -> unit end
 module Print_list :
-  functor (P : Print) -> sig type t = P.t list val print : t -> unit end
+  (P : Print) -> sig type t = P.t list val print : t -> unit end
 val print_list_of_int : Print_int.t list -> unit = <fun>
 |}]
 
@@ -387,6 +392,5 @@ let f () = let open functor(X: sig end) -> struct end in ();;
 Line 1, characters 27-53:
 1 | let f () = let open functor(X: sig end) -> struct end in ();;
                                ^^^^^^^^^^^^^^^^^^^^^^^^^^
-Error: This module is not a structure; it has type
-       functor (X : sig end) -> sig end
+Error: This module is not a structure; it has type "(X : sig end) -> sig end"
 |}]
