@@ -13,12 +13,12 @@
 /*                                                                        */
 /**************************************************************************/
 
-#include "caml/mlvalues.h"
-#include "caml/memory.h"
-#include "caml/alloc.h"
+#include <caml/mlvalues.h>
+#include <caml/memory.h>
+#include <caml/alloc.h>
 #include <stdio.h>
 
-extern value stub1(void);
+CAMLextern value stub1(void);
 
 value stub2(void) {
   printf("This is stub2, calling stub1:\n"); fflush(stdout);
