@@ -1,3 +1,8 @@
+(* TEST
+ include testing;
+ flags = "-no-strict-formats";
+*)
+
 (*
 
 A test file for the Format module.
@@ -47,10 +52,10 @@ try
   test (sprintf "%*u" (-4) 42 = "42  ");
 
   say "\nu negative\n%!";
-  begin match Sys.word_size with
-  | 32 ->
+  begin match Sys.int_size with
+  | 31 ->
      test (sprintf "%u" (-1) = "2147483647");
-  | 64 ->
+  | 63 ->
      test (sprintf "%u" (-1) = "9223372036854775807");
   | _ -> test false
   end;
@@ -71,10 +76,10 @@ try
   test (sprintf "%-0+ #*x" 5 42 = "0x2a ");
 
   say "\nx negative\n%!";
-  begin match Sys.word_size with
-  | 32 ->
+  begin match Sys.int_size with
+  | 31 ->
      test (sprintf "%x" (-42) = "7fffffd6");
-  | 64 ->
+  | 63 ->
      test (sprintf "%x" (-42) = "7fffffffffffffd6");
   | _ -> test false
   end;
@@ -91,10 +96,10 @@ try
   test (sprintf "%-0+ #*X" 5 42 = "0X2A ");
 
   say "\nx negative\n%!";
-  begin match Sys.word_size with
-  | 32 ->
+  begin match Sys.int_size with
+  | 31 ->
      test (sprintf "%X" (-42) = "7FFFFFD6");
-  | 64 ->
+  | 63 ->
      test (sprintf "%X" (-42) = "7FFFFFFFFFFFFFD6");
   | _ -> test false
   end;
@@ -111,10 +116,10 @@ try
   test (sprintf "%-0+ #*o" 5 42 = "052  ");
 
   say "\no negative\n%!";
-  begin match Sys.word_size with
-  | 32 ->
+  begin match Sys.int_size with
+  | 31 ->
      test (sprintf "%o" (-42) = "17777777726");
-  | 64 ->
+  | 63 ->
      test (sprintf "%o" (-42) = "777777777777777777726");
   | _ -> test false
   end;
@@ -521,6 +526,14 @@ try
   test (sprintf "@@" = "@");
   test (sprintf "@@@@" = "@@");
   test (sprintf "@@%%" = "@%");
+
+  say "\nDelayed printf\n%!";
+  let t1 = dprintf "%i - %s" 1 "bar" in
+  test (asprintf "foo %t" t1 = "foo 1 - bar");
+  let t2 = dprintf "%a@]" (pp_print_list pp_print_int) [1 ; 2 ; 3] in
+  test (asprintf "foo @[<v>%t@,%s" t2 "bar" = "foo 1\n    2\n    3\nbar");
+  test (asprintf "%t @[<h>%t" t1 t2 = "1 - bar 123");
+
   say "\nend of tests\n%!";
 
 with e ->
