@@ -1,12 +1,16 @@
 (* TEST
+ include systhreads;
+ hassysthreads;
+ not-windows;
+ {
+   bytecode;
+ }{
+   native;
+ }
+*)
+
    (* Test that yielding between busy threads reliably triggers a thread
       switch. *)
-   * hassysthreads
-   include systhreads
-   ** not-windows
-   *** bytecode
-   *** native
-*)
 
 let threads = 4
 
