@@ -1,5 +1,5 @@
 (* TEST
-   * expect
+ expect;
 *)
 
 let test f =
@@ -10,6 +10,6 @@ let test f =
 Line 2, characters 2-43:
 2 |   match f () with exception Not_found -> ()
       ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-Error: None of the patterns in this 'match' expression match values.
+Error: None of the patterns in this "match" expression match values.
 |}]
 ;;
