@@ -1,5 +1,5 @@
 (* TEST
-  * expect
+ expect;
 *)
 
 		let x = abc
@@ -8,6 +8,6 @@
 Line 1, characters 10-13:
 1 | 		let x = abc
     		        ^^^
-Error: Unbound value abc
-Hint: Did you mean abs?
+Error: Unbound value "abc"
+Hint: Did you mean "abs"?
 |}];;
