@@ -1,8 +1,8 @@
 (* TEST
-   flags = "-i"
-   * setup-ocamlc.byte-build-env
-   ** ocamlc.byte
-   *** check-ocamlc.byte-output
+ flags = "-i";
+ setup-ocamlc.byte-build-env;
+ ocamlc.byte;
+ check-ocamlc.byte-output;
 *)
 
 (* This test is valid OCaml code.
