@@ -6,8 +6,14 @@ let empty_record loc = H.Exp.record ~loc [] None
 let empty_apply loc f =
   H.Exp.apply ~loc f []
 
+let missing_rhs loc =
+  let name = Location.mkloc "T" loc in
+  let mtd = H.Mtd.mk ~loc name in
+  H.Sig.modtype_subst ~loc mtd
+
 let empty_let loc = H.Str.value ~loc Asttypes.Nonrecursive []
 let empty_type loc = H.Str.type_ ~loc Asttypes.Nonrecursive []
+let empty_poly_binder loc = H.Typ.(poly ~loc [] (any ~loc ()))
 let functor_id loc = Location.mkloc
     (Longident.( Lapply (Lident "F", Lident "X"))) loc
 let complex_record loc =
@@ -28,11 +34,23 @@ let pat mapper p =
       complex_record loc
   | _ -> super.M.pat mapper p
 
+let typ mapper ty =
+  match ty.ptyp_desc with
+  | Ptyp_extension ({txt="empty_poly_binder";loc},_) ->
+      empty_poly_binder loc
+  | _ -> super.M.typ mapper ty
+
+
 let structure_item mapper stri = match stri.pstr_desc with
   | Pstr_extension (({Location.txt="empty_let";loc},_),_) -> empty_let loc
   | Pstr_extension (({Location.txt="empty_type";loc},_),_) -> empty_type loc
   | _ -> super.structure_item mapper stri
 
+let signature_item mapper stri = match stri.psig_desc with
+  | Psig_extension (({Location.txt="missing_rhs";loc},_),_) -> missing_rhs loc
+  | _ -> super.signature_item mapper stri
+
+
 let () = M.register "illegal ppx" (fun _ ->
-    { super with expr; pat; structure_item }
+    { super with expr; pat; structure_item; signature_item; typ }
   )
