@@ -28,8 +28,6 @@ let () =
         !Clflags.debug;
       Printf.eprintf "use_threads: %B\n"
         !Clflags.use_threads;
-      Printf.eprintf "use_vmthreads: %B\n"
-        !Clflags.use_vmthreads;
       Printf.eprintf "recursive_types: %B\n"
         !Clflags.recursive_types;
       Printf.eprintf "principal: %B\n"
@@ -38,8 +36,6 @@ let () =
         !Clflags.transparent_modules;
       Printf.eprintf "unboxed_types: %B\n"
         !Clflags.unboxed_types;
-      Printf.eprintf "unsafe_string: %B\n"
-        !Clflags.unsafe_string;
       Printf.eprintf "</ppx-context>\n";
       flush stderr;
       default_mapper);
