@@ -1,9 +1,9 @@
 (* TEST
-   modules = "calc_parser.mly calc_lexer.mll"
-   ocamllex_flags = " -q "
-   ocamlyacc_flags = " -q "
-   readonly_files = "calc_input.txt"
-   stdin = "calc_input.txt"
+ modules = "calc_parser.mly calc_lexer.mll";
+ ocamllex_flags = " -q ";
+ ocamlyacc_flags = " -q ";
+ readonly_files = "calc_input.txt";
+ stdin = "calc_input.txt";
 *)
 let _ =
   try
