@@ -1,9 +1,9 @@
 (* TEST
-include tool-ocaml-lib
-flags = "-w -a"
-ocaml_script_as_argument = "true"
-* setup-ocaml-build-env
-** ocaml
+ include tool-ocaml-lib;
+ flags = "-w -a";
+ ocaml_script_as_argument = "true";
+ setup-ocaml-build-env;
+ ocaml;
 *)
 
 let _ = () in 3;;
