@@ -1,14 +1,14 @@
 (* TEST
-include tool-ocaml-lib
-flags = "-w a"
-ocaml_script_as_argument = "true"
-* setup-ocaml-build-env
-** ocaml
+ include tool-ocaml-lib;
+ flags = "-w -a";
+ ocaml_script_as_argument = "true";
+ setup-ocaml-build-env;
+ ocaml;
 *)
 
 open Lib;;
 let x = Bytes.of_string "foo" in
-x.[2] <- 'x';
+Bytes.set x 2 'x';
 if Bytes.get x 2 <> 'x' then raise Not_found
 ;;
 
