@@ -1,3 +1,11 @@
+(* TEST
+ include tool-ocaml-lib;
+ flags = "-w -a";
+ ocaml_script_as_argument = "true";
+ setup-ocaml-build-env;
+ ocaml;
+*)
+
 open Lib;;
 type t = {
   mutable a : int;
@@ -11,26 +19,26 @@ x
 
 (**
        0 CONSTINT 42
-       2 PUSHACC0 
+       2 PUSHACC0
        3 MAKEBLOCK1 0
        5 POP 1
        7 SETGLOBAL Lib
        9 CONSTINT 7
       11 MAKEBLOCK1 0
       13 PUSHCONSTINT 11
-      15 PUSHACC1 
-      16 SETFIELD0 
+      15 PUSHACC1
+      16 SETFIELD0
       17 CONSTINT 11
-      19 PUSHACC1 
-      20 GETFIELD0 
-      21 NEQ 
+      19 PUSHACC1
+      20 GETFIELD0
+      21 NEQ
       22 BRANCHIFNOT 29
       24 GETGLOBAL Not_found
       26 MAKEBLOCK1 0
-      28 RAISE 
-      29 ACC0 
+      28 RAISE
+      29 ACC0
       30 POP 1
-      32 ATOM0 
+      32 ATOM0
       33 SETGLOBAL T210-setfield0
-      35 STOP 
+      35 STOP
 **)
