@@ -1,3 +1,7 @@
+(* TEST
+ toplevel;
+*)
+
 type t = A | B;;
 type u = C of t;;
 let print_t out = function A -> Format.fprintf out "A";;
