@@ -1,5 +1,5 @@
 (* TEST
-   * expect
+ expect;
 *)
 
 [@@@ocaml.warning "+3"];;
@@ -20,6 +20,7 @@ Line 7, characters 9-10:
 7 |   val x: t [@@ocaml.deprecated]
              ^
 Alert deprecated: t
+
 module X : sig type t type s type u val x : t end
 |}]
 
@@ -30,6 +31,7 @@ Line 1, characters 9-12:
 1 | type t = X.t
              ^^^
 Alert deprecated: X.t
+
 type t = X.t
 |}]
 
@@ -40,6 +42,7 @@ Line 1, characters 8-11:
 1 | let x = X.x
             ^^^
 Alert deprecated: X.x
+
 val x : X.t = <abstr>
 |}]
 
@@ -54,6 +57,7 @@ Line 3, characters 0-3:
 3 | foo;;
     ^^^
 Alert deprecated: foo
+
 - : unit = ()
 |}]
 
@@ -74,6 +78,7 @@ Line 2, characters 33-36:
 2 |   | bar, cho [@deprecated], _ -> cho + 1
                                      ^^^
 Alert deprecated: cho
+
 val f : 'a * int * 'b -> int = <fun>
 |}]
 
@@ -87,6 +92,7 @@ Line 3, characters 12-15:
 3 |     val h = foo
                 ^^^
 Alert deprecated: foo
+
 class c : 'a * int -> object val h : int end
 |}]
 
@@ -99,10 +105,12 @@ Line 1, characters 9-12:
 1 | type t = X.t * X.s
              ^^^
 Alert deprecated: X.t
+
 Line 1, characters 15-18:
 1 | type t = X.t * X.s
                    ^^^
 Alert deprecated: X.s
+
 type t = X.t * X.s
 |}]
 
@@ -120,6 +128,7 @@ Line 2, characters 9-12:
 2 | and t2 = X.s
              ^^^
 Alert deprecated: X.s
+
 type t1 = X.t
 and t2 = X.s
 |}]
@@ -131,6 +140,7 @@ Line 1, characters 14-15:
 1 | type t = A of t [@@ocaml.deprecated]
                   ^
 Alert deprecated: t
+
 type t = A of t
 |}]
 
@@ -157,6 +167,7 @@ Line 1, characters 39-42:
 1 | type t = (X.t [@ocaml.warning "-3"]) * X.s
                                            ^^^
 Alert deprecated: X.s
+
 type t = X.t * X.s
 |}]
 
@@ -177,6 +188,7 @@ Line 1, characters 22-25:
 1 | let _ = function (_ : X.t) -> ()
                           ^^^
 Alert deprecated: X.t
+
 - : X.t -> unit = <fun>
 |}]
 
@@ -196,6 +208,7 @@ Line 1, characters 26-29:
 1 | module M = struct let x = X.x end
                               ^^^
 Alert deprecated: X.x
+
 module M : sig val x : X.t end
 |}]
 
@@ -218,10 +231,12 @@ Line 1, characters 26-29:
 1 | module rec M : sig val x: X.t end = struct let x = X.x end
                               ^^^
 Alert deprecated: X.t
+
 Line 1, characters 51-54:
 1 | module rec M : sig val x: X.t end = struct let x = X.x end
                                                        ^^^
 Alert deprecated: X.x
+
 module rec M : sig val x : X.t end
 |}]
 
@@ -248,6 +263,7 @@ Line 3, characters 17-20:
 3 |   struct let x = X.x end
                      ^^^
 Alert deprecated: X.x
+
 module rec M : sig val x : X.t end
 |}]
 
@@ -260,6 +276,7 @@ Line 1, characters 29-32:
 1 | module type S = sig type t = X.t end
                                  ^^^
 Alert deprecated: X.t
+
 module type S = sig type t = X.t end
 |}]
 
@@ -285,6 +302,7 @@ Line 1, characters 28-31:
 1 | class c = object method x = X.x end
                                 ^^^
 Alert deprecated: X.x
+
 class c : object method x : X.t end
 |}]
 
@@ -316,6 +334,7 @@ Line 1, characters 33-36:
 1 | class type c = object method x : X.t end
                                      ^^^
 Alert deprecated: X.t
+
 class type c = object method x : X.t end
 |}]
 
@@ -348,6 +367,7 @@ Line 1, characters 22-25:
 1 | external foo: unit -> X.t = "foo"
                           ^^^
 Alert deprecated: X.t
+
 external foo : unit -> X.t = "foo"
 |}]
 
@@ -367,6 +387,7 @@ Line 1, characters 0-3:
 1 | X.x
     ^^^
 Alert deprecated: X.x
+
 - : X.t = <abstr>
 |}]
 
@@ -429,6 +450,7 @@ Line 2, characters 9-12:
 2 |   | A of X.t
              ^^^
 Alert deprecated: X.t
+
 type ext += A of X.t | B of X.s | C of X.u
 |}]
 
@@ -448,6 +470,7 @@ Line 1, characters 17-20:
 1 | exception Foo of X.t
                      ^^^
 Alert deprecated: X.t
+
 exception Foo of X.t
 |}]
 
@@ -470,6 +493,7 @@ Line 2, characters 9-12:
 2 |   | A of X.t
              ^^^
 Alert deprecated: X.t
+
 type t = A of X.t | B of X.s | C of X.u
 |}]
 
@@ -485,6 +509,7 @@ Line 3, characters 7-10:
 3 |     a: X.t;
            ^^^
 Alert deprecated: X.t
+
 type t = { a : X.t; b : X.s; c : X.u; }
 |}]
 
@@ -501,6 +526,7 @@ Line 3, characters 7-10:
 3 |     a: X.t;
            ^^^
 Alert deprecated: X.t
+
 type t = < a : X.t; b : X.s; c : X.u >
 |}]
 
@@ -517,6 +543,7 @@ Line 3, characters 10-13:
 3 |   | `A of X.t
               ^^^
 Alert deprecated: X.t
+
 type t = [ `A of X.t | `B of X.s | `C of X.u ]
 |}]
 
@@ -530,7 +557,7 @@ type t = [ `A of X.t | `B of X.s | `C of X.u ]
 Line 1, characters 20-33:
 1 | [@@@ocaml.ppwarning "Pp warning!"]
                         ^^^^^^^^^^^^^
-Warning 22: Pp warning!
+Warning 22 [preprocessor]: Pp warning!
 |}]
 
 
@@ -541,11 +568,13 @@ let x = () [@ocaml.ppwarning "Pp warning 1!"]
 Line 2, characters 24-39:
 2 |     [@@ocaml.ppwarning  "Pp warning 2!"]
                             ^^^^^^^^^^^^^^^
-Warning 22: Pp warning 2!
+Warning 22 [preprocessor]: Pp warning 2!
+
 Line 1, characters 29-44:
 1 | let x = () [@ocaml.ppwarning "Pp warning 1!"]
                                  ^^^^^^^^^^^^^^^
-Warning 22: Pp warning 1!
+Warning 22 [preprocessor]: Pp warning 1!
+
 val x : unit = ()
 |}]
 
@@ -556,7 +585,8 @@ type t = unit
 Line 2, characters 22-35:
 2 |     [@ocaml.ppwarning "Pp warning!"]
                           ^^^^^^^^^^^^^
-Warning 22: Pp warning!
+Warning 22 [preprocessor]: Pp warning!
+
 type t = unit
 |}]
 
@@ -574,7 +604,8 @@ end
 Line 8, characters 22-36:
 8 |   [@@@ocaml.ppwarning "Pp warning2!"]
                           ^^^^^^^^^^^^^^
-Warning 22: Pp warning2!
+Warning 22 [preprocessor]: Pp warning2!
+
 module X : sig end
 |}]
 
@@ -586,7 +617,8 @@ let x =
 Line 3, characters 23-38:
 3 |     [@ocaml.ppwarning  "Pp warning 2!"]
                            ^^^^^^^^^^^^^^^
-Warning 22: Pp warning 2!
+Warning 22 [preprocessor]: Pp warning 2!
+
 val x : unit = ()
 |}]
 
@@ -599,11 +631,13 @@ type t =
 Line 4, characters 21-36:
 4 |   [@@ocaml.ppwarning "Pp warning 3!"]
                          ^^^^^^^^^^^^^^^
-Warning 22: Pp warning 3!
+Warning 22 [preprocessor]: Pp warning 3!
+
 Line 3, characters 21-36:
 3 |   [@ocaml.ppwarning  "Pp warning 2!"]
                          ^^^^^^^^^^^^^^^
-Warning 22: Pp warning 2!
+Warning 22 [preprocessor]: Pp warning 2!
+
 type t = unit
 |}]
 
@@ -613,11 +647,12 @@ let ([][@ocaml.ppwarning "XX"]) = []
 Line 1, characters 25-29:
 1 | let ([][@ocaml.ppwarning "XX"]) = []
                              ^^^^
-Warning 22: XX
+Warning 22 [preprocessor]: XX
+
 Line 1, characters 4-31:
 1 | let ([][@ocaml.ppwarning "XX"]) = []
         ^^^^^^^^^^^^^^^^^^^^^^^^^^^
-Warning 8: this pattern-matching is not exhaustive.
+Warning 8 [partial-match]: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 _::_
 |}]
