@@ -1,5 +1,5 @@
 (* TEST
-   * expect
+ expect;
 *)
 
 type ('env, 'a) var =
@@ -30,6 +30,6 @@ Line 15, characters 5-6:
 15 |    | _ -> .   (* error *)
           ^
 Error: This match case could not be refuted.
-       Here is an example of a value that would reach it: (Tint, Tvar Zero)
+       Here is an example of a value that would reach it: "(Tint, Tvar Zero)"
 |}];;
 (* let x = f Tint (Tvar Zero) ;; *)
