@@ -1,5 +1,5 @@
 (* TEST
-   * expect
+ expect;
 *)
 
 module Add (T : sig type two end) =
@@ -17,11 +17,12 @@ Lines 7-9, characters 43-24:
 7 | ...........................................function
 8 |     | One, One -> "two"
 9 |     | Two, Two -> "four"
-Warning 8: this pattern-matching is not exhaustive.
+Warning 8 [partial-match]: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
-(Two, One)
+(One, Two)
+
 module Add :
-  functor (T : sig type two end) ->
+  (T : sig type two end) ->
     sig
       type _ t = One : [ `One ] t | Two : T.two t
       val add : 'a t * 'a t -> string
