@@ -1,5 +1,5 @@
 (* TEST
-   * expect
+ expect;
 *)
 
 type tag = [`TagA | `TagB | `TagC];;
@@ -42,11 +42,11 @@ type _ wrapPoly =
 Line 25, characters 23-27:
 25 |     | WrapPoly ATag -> intA
                             ^^^^
-Error: This expression has type [< `TagA of 'a ] -> 'a
-       but an expression was expected of type a -> int
-       Type [< `TagA of 'a ] is not compatible with type
-         a = [< `TagA of int | `TagB ]
-       The first variant type does not allow tag(s) `TagB
+Error: The value "intA" has type "[< `TagA of 'a ] -> 'a"
+       but an expression was expected of type "a -> int"
+       Type "[< `TagA of 'a ]" is not compatible with type
+         "a" = "[< `TagA of int | `TagB ]"
+       The first variant type does not allow tag(s) "`TagB"
 |}];;
 
 let _ =  example6 (WrapPoly AandBTags) `TagB (* This causes a seg fault *)
@@ -55,5 +55,5 @@ let _ =  example6 (WrapPoly AandBTags) `TagB (* This causes a seg fault *)
 Line 1, characters 9-17:
 1 | let _ =  example6 (WrapPoly AandBTags) `TagB (* This causes a seg fault *)
              ^^^^^^^^
-Error: Unbound value example6
+Error: Unbound value "example6"
 |}];;
