@@ -1,5 +1,5 @@
 (* TEST
-   * expect
+ expect;
 *)
 
 module F(S : sig type 'a t end) = struct
@@ -18,8 +18,9 @@ Lines 7-8, characters 47-21:
 Warning 8 [partial-match]: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 (A, A)
+
 module F :
-  functor (S : sig type 'a t end) ->
+  (S : sig type 'a t end) ->
     sig
       type _ ab = A : int S.t ab | B : float S.t ab
       val f : int S.t ab -> float S.t ab -> string
@@ -45,8 +46,9 @@ Lines 10-11, characters 15-21:
 Warning 8 [partial-match]: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 (A, A)
+
 module F :
-  functor (S : sig type 'a t end) ->
+  (S : sig type 'a t end) ->
     sig
       type a = int * int
       type b = int -> int
