@@ -1,5 +1,5 @@
 (* TEST
-   * expect
+ expect;
 *)
 
 type (_, _) t =
@@ -28,9 +28,10 @@ module M : sig type s = private [> `A ] val eq : (s, [ `A | `B ]) t end
 Lines 16-17, characters 39-16:
 16 | .......................................function
 17 |   | Any -> "Any"
-Warning 8: this pattern-matching is not exhaustive.
+Warning 8 [partial-match]: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 Eq
+
 val f : (M.s, [ `A | `B ]) t -> string = <fun>
 Exception: Match_failure ("", 16, 39).
 |}];;
@@ -58,8 +59,9 @@ module N :
 Lines 12-13, characters 49-16:
 12 | .................................................function
 13 |   | Any -> "Any"
-Warning 8: this pattern-matching is not exhaustive.
+Warning 8 [partial-match]: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 Eq
+
 val f : (N.s, < a : int; b : bool >) t -> string = <fun>
 |}];;
