@@ -1,5 +1,5 @@
 (* TEST
-   * expect
+ expect;
 *)
 
 type (_, _) comp =
@@ -28,6 +28,7 @@ Line 16, characters 0-33:
 Warning 8 [partial-match]: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 Eq
+
 Exception: Match_failure ("", 16, 0).
 |}];;
 
@@ -51,5 +52,6 @@ Line 11, characters 0-33:
 Warning 8 [partial-match]: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 Eq
+
 Exception: Match_failure ("", 11, 0).
 |}];;
