@@ -1,3 +1,7 @@
+(* TEST
+ expect;
+*)
+
 type _ nat =
     Zero : [`Zero] nat
   | Succ : 'a nat -> [`Succ of 'a] nat;;
@@ -20,8 +24,10 @@ type aux =
     Aux :
       [ `Succ of [< [< [< [ `Zero ] pre_nat ] pre_nat ] pre_nat ] ] nat ->
       aux
-Line _, characters 4-5:
+Line 14, characters 4-5:
+14 |   | _ -> .  (* error *)
+         ^
 Error: This match case could not be refuted.
        Here is an example of a value that would reach it:
-       Succ (Succ (Succ (Succ (Succ Zero))))
+       "Succ (Succ (Succ (Succ (Succ Zero))))"
 |}];;
