@@ -1,5 +1,5 @@
 (* TEST
-   * expect
+ expect;
 *)
 
 type +'a n = private int
@@ -23,20 +23,9 @@ type _ t = Nil : nil t | Cons : ('x, 'fx) elt * 'x t -> 'fx t
 Line 9, characters 11-18:
 9 |   let Cons(Elt dim, _) = sh in ()
                ^^^^^^^
-Error: This pattern matches values of type ($Cons_'x, 'a -> $Cons_'x) elt
+Error: This pattern matches values of type "($x, 'a -> $x) elt"
        but a pattern was expected which matches values of type
-         ($Cons_'x, 'a -> $'b -> nil) elt
-       The type constructor $'b would escape its scope
-|}, Principal{|
-type +'a n = private int
-type nil = private Nil_type
-type (_, _) elt =
-    Elt_fine : 'nat n -> ('l, 'nat * 'l) elt
-  | Elt : 'nat n -> ('l, 'nat -> 'l) elt
-type _ t = Nil : nil t | Cons : ('x, 'fx) elt * 'x t -> 'fx t
-Line 9, characters 6-22:
-9 |   let Cons(Elt dim, _) = sh in ()
-          ^^^^^^^^^^^^^^^^
-Error: This pattern matches values of type ('a -> $0 -> nil) t
-       The type constructor $0 would escape its scope
+         "($x, 'a -> $'b -> nil) elt"
+       The type constructor "$'b" would escape its scope
+       Hint: "$x" is an existential type bound by the constructor "Cons".
 |}];;
