@@ -29,6 +29,5 @@ Warning 8 [partial-match]: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 Eq
 
-module F :
-  functor (T : sig type _ t end) -> sig val f : ('a, 'a T.t) eq -> int end
+module F : (T : sig type _ t end) -> sig val f : ('a, 'a T.t) eq -> int end
 |}];;
