@@ -1,5 +1,5 @@
 (* TEST
-   * expect
+ expect;
 *)
 
 type bar = < bar: unit >
@@ -19,13 +19,12 @@ class foo =
 type bar = < bar : unit >
 type _ ty = Int : int ty
 type dyn = Dyn : 'a ty -> dyn
-Line 7, characters 0-108:
- 7 | class foo =
- 8 |   object (this)
+Lines 8-12, characters 2-5:
+ 8 | ..object (this)
  9 |     method foo (Dyn ty) =
 10 |       match ty with
 11 |       | Int -> (this :> bar)
 12 |   end.................................
-Error: This class should be virtual.
-       The following methods are undefined : bar
+Error: This non-virtual class has undeclared virtual methods.
+       The following methods were not declared : "bar"
 |}];;
