@@ -1,5 +1,5 @@
 (* TEST
-   * expect
+ expect;
 *)
 
 module X = struct
@@ -18,13 +18,13 @@ end;; (* should fail *)
 Lines 2-3, characters 2-37:
 2 | ..type t = X.t =
 3 |     | A : 'a * 'b * ('b -> unit) -> t
-Error: This variant or record definition does not match that of type X.t
+Error: This variant or record definition does not match that of type "X.t"
        Constructors do not match:
-         A : 'a * 'b * ('a -> unit) -> X.t
+         "A : 'a * 'b * ('a -> unit) -> X.t"
        is not the same as:
-         A : 'a * 'b * ('b -> unit) -> X.t
-       The type 'a -> unit is not equal to the type 'b -> unit
-       Type 'a is not equal to type 'b
+         "A : 'a * 'b * ('b -> unit) -> X.t"
+       The type "'a -> unit" is not equal to the type "'b -> unit"
+       Type "'a" is not equal to type "'b"
 |}]
 
 (* would segfault
