@@ -1,5 +1,5 @@
 (* TEST
-   * expect
+ expect;
 *)
 
 #labels false;;
@@ -24,9 +24,10 @@ let f : [`L of (s, t) eql | `R of silly] -> 'a =
 Line 2, characters 2-30:
 2 |   function `R {silly} -> silly
       ^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-Warning 8: this pattern-matching is not exhaustive.
+Warning 8 [partial-match]: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 `L Refl
+
 val f : [ `L of (s, t) eql | `R of silly ] -> 'a = <fun>
 |}]
 
