@@ -212,7 +212,7 @@ module F(X : sig type 'a t end) = struct
 end
 [%%expect{|
 module F :
-  functor (X : sig type 'a t end) ->
+  (X : sig type 'a t end) ->
     sig type 'a u = 'b X.t constraint 'a = < b : 'b X.t > end
 |}]
 (* But not too clever *)
@@ -232,7 +232,7 @@ module F(X : sig type 'a t end) = struct
 end
 [%%expect{|
 module F :
-  functor (X : sig type 'a t end) ->
+  (X : sig type 'a t end) ->
     sig type 'a u = 'b X.t constraint 'a = < b : 'b X.t > end
 |}, Principal{|
 Line 2, characters 2-51:
@@ -396,7 +396,7 @@ let coerce : type a b. (a, b) eql -> a -> b = fun Refl x -> x;;
 [%%expect{|
 type (_, _) eql = Refl : ('a, 'a) eql
 module Uninj :
-  functor (X : sig type !'a t end) ->
+  (X : sig type !'a t end) ->
     sig val uninj : ('a X.t, 'b X.t) eql -> ('a, 'b) eql end
 val coerce : ('a, 'b) eql -> 'a -> 'b = <fun>
 |}]
