@@ -1,6 +1,6 @@
 (* TEST
-   * expect
- *)
+ expect;
+*)
 
 type t = int
 module M : sig type t end with type t = [`T of t] =
@@ -18,16 +18,16 @@ Error: Signature mismatch:
        Modules do not match:
          sig type t = [ `T of t ] end
        is not included in
-         sig type t = [ `T of t ] end
+         sig type t = [ `T of t/2 ] end
        Type declarations do not match:
-         type t = [ `T of t/2 ]
+         type t = [ `T of t ]
        is not included in
-         type t = [ `T of t/1 ]
-       The type [ `T of t/1 ] is not equal to the type [ `T of t/2 ]
-       Type t/1 = [ `T of t/1 ] is not equal to type t/2 = int
-       Types for tag `T are incompatible
+         type t = [ `T of t/3 ]
+       The type "[ `T of t ]" is not equal to the type "[ `T of t/2 ]"
+       Type "t" = "[ `T of t ]" is not equal to type "t/2" = "int"
+       Types for tag "`T" are incompatible
        Line 4, characters 2-20:
-         Definition of type t/1
+         Definition of type "t"
        Line 1, characters 0-12:
-         Definition of type t/2
+         Definition of type "t/2"
 |}]
