@@ -1,9 +1,9 @@
-(* TEST
-flags = " -w a "
-ocamlc_byte_exit_status = "2"
-* setup-ocamlc.byte-build-env
-** ocamlc.byte
-*** check-ocamlc.byte-output
+(* TEST_BELOW
+(* Blank lines added here to preserve locations. *)
+
+
+
+
 *)
 
 let flag = ref false
@@ -26,3 +26,11 @@ let () = flag := true
 module M2 = M(Float)(Int)
 
 let _ = [| M2.X.x; M1.X.x |]
+
+(* TEST
+ flags = " -w -a ";
+ ocamlc_byte_exit_status = "2";
+ setup-ocamlc.byte-build-env;
+ ocamlc.byte;
+ check-ocamlc.byte-output;
+*)
