@@ -57,7 +57,7 @@ module C4 = F(struct end);;
 C4.chr 66;;
 [%%expect{|
 module F :
-  functor (X : sig end) ->
+  (X : sig end) ->
     sig
       external code : char -> int = "%identity"
       val chr : int -> char
@@ -91,7 +91,7 @@ module C4 :
 module G(X:sig end) = struct module M = X end;; (* does not alias X *)
 module M = G(struct end);;
 [%%expect{|
-module G : functor (X : sig end) -> sig module M : sig end end
+module G : (X : sig end) -> sig module M : sig end end
 module M : sig module M : sig end end
 |}];;
 
@@ -141,9 +141,8 @@ module M5 = G(struct end);;
 M5.N'.x;;
 [%%expect{|
 module F :
-  functor (X : sig end) ->
-    sig module N : sig val x : int end module N' = N end
-module G : functor (X : sig end) -> sig module N' : sig val x : int end end
+  (X : sig end) -> sig module N : sig val x : int end module N' = N end
+module G : (X : sig end) -> sig module N' : sig val x : int end end
 module M5 : sig module N' : sig val x : int end end
 - : int = 1
 |}];;
@@ -265,7 +264,7 @@ val pow : t -> t -> t = <fun>
 
 module F(X:sig module C = Char end) = struct module C = X.C end;;
 [%%expect{|
-module F : functor (X : sig module C = Char end) -> sig module C = Char end
+module F : (X : sig module C = Char end) -> sig module C = Char end
 |}];;
 
 (* Applicative functors *)
@@ -383,7 +382,7 @@ end;;
 include T;;
 let f (x : t) : T.t = x ;;
 [%%expect{|
-module F : functor (M : sig end) -> sig type t end
+module F : (M : sig end) -> sig type t end
 module T : sig module M : sig end type t = F(M).t end
 module M = T.M
 type t = F(M).t
@@ -470,11 +469,11 @@ module G = F (M.Y);;
 module N = G (M);;
 module N = F (M.Y) (M);;
 [%%expect{|
-module FF : functor (X : sig end) -> sig type t end
+module FF : (X : sig end) -> sig type t end
 module M :
   sig module X : sig end module Y : sig type t = FF(X).t end type t = Y.t end
-module F : functor (Y : sig type t end) (M : sig type t = Y.t end) -> sig end
-module G : functor (M : sig type t = M.Y.t end) -> sig end
+module F : (Y : sig type t end) (M : sig type t = Y.t end) -> sig end
+module G : (M : sig type t = M.Y.t end) -> sig end
 module N : sig end
 module N : sig end
 |}];;
@@ -488,7 +487,7 @@ end
 include T
 let f (x : t) : T.t = x
 [%%expect {|
-module F : functor (M : sig end) -> sig type t end
+module F : (M : sig end) -> sig type t end
 module T : sig module M : sig end type t = F(M).t end
 module M = T.M
 type t = F(M).t
@@ -511,7 +510,7 @@ module A1 : sig end
 module A2 : sig end
 module L1 : sig module X = A1 end
 module L2 : sig module X = A2 end
-module F : functor (L : sig module X : sig end end) -> sig end
+module F : (L : sig module X : sig end end) -> sig end
 module F1 : sig end
 module F2 : sig end
 |}];;
@@ -688,7 +687,7 @@ module type A = Alias with module N := F(List);;
 module rec Bad : A = Bad;;
 [%%expect{|
 module type Alias = sig module N : sig end module M = N end
-module F : functor (X : sig end) -> sig type t end
+module F : (X : sig end) -> sig type t end
 Line 1:
 Error: Module type declarations do not match:
          module type A = sig module M = F(List) end
@@ -780,7 +779,7 @@ end = struct
   type a = Foo.b
 end;;
 [%%expect{|
-module F : functor (X : sig end) -> sig type t end
+module F : (X : sig end) -> sig type t end
 module M :
   sig type a module Foo : sig module Bar : sig end type b = a end end
 |}];;
@@ -880,7 +879,7 @@ module M :
         module A : sig val x : string end
         module B : sig val x : int end
       end
-    module F : functor (X : sig module A = N.A end) -> sig val s : string end
+    module F : (X : sig module A = N.A end) -> sig val s : string end
   end
 module N : sig val s : string end
 val s : string = "hello"
