@@ -32,8 +32,7 @@ Error: Modules do not match:
 module F(X : sig type t = M.t val equal : unit end)
   = struct type t end
 [%%expect{|
-module F :
-  functor (X : sig type t = M.t val equal : unit end) -> sig type t end
+module F : (X : sig type t = M.t val equal : unit end) -> sig type t end
 |} ]
 
 type t = F(M).t
@@ -55,7 +54,7 @@ Error: Modules do not match:
 (* MPR#7611 *)
 module Generative() = struct type t end
 [%%expect{|
-module Generative : functor () -> sig type t end
+module Generative : () -> sig type t end
 |}]
 
 type t = Generative(M).t
