@@ -6,7 +6,7 @@ module F(X : sig type t end) = struct
   let f (_ : X.t) = ()
 end;;
 [%%expect{|
-module F : functor (X : sig type t end) -> sig val f : X.t -> unit end
+module F : (X : sig type t end) -> sig val f : X.t -> unit end
 |}]
 
 module M = F(struct type t = T end);;
@@ -15,7 +15,7 @@ Line 1, characters 11-35:
 1 | module M = F(struct type t = T end);;
                ^^^^^^^^^^^^^^^^^^^^^^^^
 Error: This functor has type
-       "functor (X : sig type t end) -> sig val f : X.t -> unit end"
+       "(X : sig type t end) -> sig val f : X.t -> unit end"
        The parameter cannot be eliminated in the result type.
        Please bind the argument to a module identifier.
 |}]
@@ -31,7 +31,7 @@ module N = M (struct type 'a t = int constraint 'a = float end)
 
 [%%expect{|
 module M :
-  functor (X : sig type 'a t constraint 'a = float end) ->
+  (X : sig type 'a t constraint 'a = float end) ->
     sig module type S = sig type t = float val foo : t X.t end end
 module N : sig module type S = sig type t = float val foo : int end end
 |}]
@@ -41,7 +41,7 @@ module F (X : sig type t end) = struct type s = X.t always_int end
 module M = F (struct type t = T end)
 [%%expect{|
 type 'a always_int = int
-module F : functor (X : sig type t end) -> sig type s = X.t always_int end
+module F : (X : sig type t end) -> sig type s = X.t always_int end
 module M : sig type s = int end
 |}]
 
@@ -52,7 +52,7 @@ end
 [%%expect{|
 module M :
   sig
-    module F : functor (X : sig type t end) -> sig type t = X.t end
+    module F : (X : sig type t end) -> sig type t = X.t end
     module Not_ok : sig type t end
   end
 |}]
