@@ -1,5 +1,5 @@
 (* TEST
- * expect
+ expect;
 *)
 
 (* From jctis: <https://github.com/ocaml/ocaml/issues/10399> *)
@@ -41,6 +41,6 @@ Error: Signature mismatch:
            val o : t
          end
        Values do not match: val o : c is not included in val o : t
-       The type c is not compatible with the type t
-       The second object type has no method y
+       The type "c" is not compatible with the type "t"
+       The second object type has no method "y"
 |}]
