@@ -1,5 +1,5 @@
 (* TEST
-   * expect
+ expect;
 *)
 
 (* Leo's version *)
@@ -15,7 +15,7 @@ module M = F(struct type t end);;
 module type S = module type of M;;
 [%%expect{|
 module F :
-  functor (X : sig type t end) ->
+  (X : sig type t end) ->
     sig type x = X.t type y = X.t type t = E of x type u = t = E of y end
 module M : sig type x type y type t = E of x type u = t = E of y end
 module type S = sig type x type y type t = E of x type u = t = E of y end
@@ -26,12 +26,12 @@ module rec M1 : S with type x = int and type y = bool = M1;;
 Line 1, characters 0-58:
 1 | module rec M1 : S with type x = int and type y = bool = M1;;
     ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-Error: This variant or record definition does not match that of type M1.t
+Error: This variant or record definition does not match that of type "M1.t"
        Constructors do not match:
-         E of M1.x
+         "E of M1.x"
        is not the same as:
-         E of M1.y
-       The type M1.x = int is not equal to the type M1.y = bool
+         "E of M1.y"
+       The type "M1.x" = "int" is not equal to the type "M1.y" = "bool"
 |}]
 
 let bool_of_int x =
@@ -43,7 +43,7 @@ bool_of_int 3;;
 Line 2, characters 28-32:
 2 |   let (E y : M1.u) = (E x : M1.t) in
                                 ^^^^
-Error: Unbound module M1
+Error: Unbound module "M1"
 |}]
 
 (* Also check the original version *)
@@ -58,7 +58,7 @@ module type S = module type of M;;
 [%%expect{|
 type (_, _) eq = Eq : ('a, 'a) eq
 module F :
-  functor (X : Set.OrderedType) ->
+  (X : Set.OrderedType) ->
     sig
       type x = Set.Make(X).t
       and y = Set.Make(X).t
@@ -78,11 +78,11 @@ cast eq 3;;
 Line 1, characters 0-58:
 1 | module rec M1 : S with type x = int and type y = bool = M1;;
     ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-Error: This variant or record definition does not match that of type M1.t
+Error: This variant or record definition does not match that of type "M1.t"
        Constructors do not match:
-         E of (M1.x, M1.x) eq
+         "E of (M1.x, M1.x) eq"
        is not the same as:
-         E of (M1.x, M1.y) eq
-       The type (M1.x, M1.x) eq is not equal to the type (M1.x, M1.y) eq
-       Type M1.x = int is not equal to type M1.y = bool
+         "E of (M1.x, M1.y) eq"
+       The type "(M1.x, M1.x) eq" is not equal to the type "(M1.x, M1.y) eq"
+       Type "M1.x" = "int" is not equal to type "M1.y" = "bool"
 |}]
