@@ -1,5 +1,5 @@
 (* TEST
-   * expect
+ expect;
 *)
 type u = <x:int>
 type t = private <u; ..>
@@ -11,7 +11,7 @@ type t = private < x : int; .. >
 Line 4, characters 24-25:
 4 | let f (x:t) (y:u) = x = y;;
                             ^
-Error: This expression has type u but an expression was expected of type t
+Error: The value "y" has type "u" but an expression was expected of type "t"
        The second object type has an abstract row, it cannot be closed
 |}]
 
@@ -21,6 +21,6 @@ let g (x:u) (y:t) = x = y;;
 Line 1, characters 24-25:
 1 | let g (x:u) (y:t) = x = y;;
                             ^
-Error: This expression has type t but an expression was expected of type u
+Error: The value "y" has type "t" but an expression was expected of type "u"
        The first object type has an abstract row, it cannot be closed
 |}]
