@@ -1,5 +1,5 @@
 (* TEST
-   * expect
+ expect;
 *)
 
 class type foo_t =
@@ -40,7 +40,7 @@ class foo: foo_t =
   end
 ;;
 [%%expect{|
-Line 2, characters 2-156:
+Lines 2-8, characters 2-5:
 2 | ..object(self)
 3 |     method foo = "foo"
 4 |     method cast: type a. a name -> a =
