@@ -1,5 +1,5 @@
 (* TEST
-   * expect
+ expect;
 *)
 
 class type ['e] t = object('s)
@@ -18,6 +18,5 @@ Line 2, characters 2-27:
       ^^^^^^^^^^^^^^^^^^^^^^^^^
 Error: Some type variables are unbound in this type:
          class base : 'e -> ['e] t
-       The method update has type 'e -> < update : 'a; .. > as 'a where 'e
-       is unbound
+       The method "update" has type "'e -> #base" where "'e" is unbound
 |}];;
