@@ -1,8 +1,8 @@
-(* TEST
-flags = "-i -w +63"
-* setup-ocamlc.byte-build-env
-** ocamlc.byte
-*** check-ocamlc.byte-output
+(* TEST_BELOW
+(* Blank lines added here to preserve locations. *)
+
+
+
 *)
 
 type 'a t = B of 'a t list
@@ -15,3 +15,10 @@ module DT = struct
   type 'a t = {bar : 'a}
   let p t = foo (fun x -> x) t
 end
+
+(* TEST
+ flags = "-i -w +63";
+ setup-ocamlc.byte-build-env;
+ ocamlc.byte;
+ check-ocamlc.byte-output;
+*)
