@@ -1,8 +1,8 @@
 (* TEST
-flags = " -w -a "
-* setup-ocamlc.byte-build-env
-** ocamlc.byte
-*** check-ocamlc.byte-output
+ flags = " -w -a ";
+ setup-ocamlc.byte-build-env;
+ ocamlc.byte;
+ check-ocamlc.byte-output;
 *)
 
 module Order = struct
@@ -93,7 +93,7 @@ module Create(P: Order.Total) = struct
                             List.iter (fun j -> j#emit e) archivers_;
                         cont e
                     in
-                    Printf.kprintf f
+                    Printf.ksprintf f
         end
 end
 
