@@ -1,10 +1,10 @@
 (* TEST
-   * expect
+ expect;
 *)
 
 module type S = sig module M : sig end module N = M end;;
 [%%expect{|
-module type S = sig module M : sig  end module N = M end
+module type S = sig module M : sig end module N = M end
 |}];;
 
 module rec M : S with module M := M = M;;
