@@ -1,3 +1,8 @@
+(* TEST
+ flags = " -short-paths ";
+ toplevel;
+*)
+
 type t = [`A | `B];;
 type 'a u = t;;
 let a : [< int u] = `A;;
