@@ -1,6 +1,6 @@
 (* TEST
-   flags = "-strict-sequence"
-   * expect
+ flags = "-strict-sequence";
+ expect;
 *)
 external a : (int [@untagged]) -> unit = "a" "a_nat"
 external b : (int32 [@unboxed]) -> unit = "b" "b_nat"
@@ -56,8 +56,8 @@ end;;
 Line 11, characters 2-71:
 11 |   external f : (int32 [@unboxed]) -> (int32 [@unboxed]) = "f" "noalloc"
        ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-Error: [@The native code version of the primitive is mandatory
-       when attributes [@untagged] or [@unboxed] are present.
+Error: The native code version of the primitive is mandatory
+       when attributes "[@untagged]" or "[@unboxed]" are present.
 |}]
 
 module Old_style_warning = struct
@@ -74,19 +74,23 @@ Line 3, characters 2-61:
       ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 Alert deprecated: [@@unboxed] + [@@noalloc] should be used
 instead of "float"
+
 Line 4, characters 2-53:
 4 |   external b : float -> float = "b" "noalloc" "b_nat"
       ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 Alert deprecated: [@@noalloc] should be used instead of "noalloc"
+
 Line 5, characters 2-51:
 5 |   external c : float -> float = "c" "c_nat" "float"
       ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 Alert deprecated: [@@unboxed] + [@@noalloc] should be used
 instead of "float"
+
 Line 6, characters 2-45:
 6 |   external d : float -> float = "d" "noalloc"
       ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 Alert deprecated: [@@noalloc] should be used instead of "noalloc"
+
 module Old_style_warning :
   sig
     external a : float -> float = "a" "a_nat" [@@unboxed] [@@noalloc]
@@ -257,7 +261,7 @@ Error: Signature mismatch:
          external f : int -> int = "f" "f_nat" [@@noalloc]
        is not included in
          external f : int -> int = "f" "f_nat"
-       The first primitive is [@@noalloc] but the second is not
+       The first primitive is "[@@noalloc]" but the second is not
 |}]
 
 (* Bad: attributes in the interface but not in the implementation *)
@@ -420,7 +424,7 @@ Error: Signature mismatch:
          external f : int -> int = "f" "f_nat"
        is not included in
          external f : int -> int = "f" "f_nat" [@@noalloc]
-       The second primitive is [@@noalloc] but the first is not
+       The second primitive is "[@@noalloc]" but the first is not
 |}]
 
 (* Bad: claiming something is a primitive when it isn't *)
@@ -460,7 +464,7 @@ end
 
 [%%expect{|
 Line 1:
-Error: The external function `f' is not available
+Error: The external function "f" is not available
 |}]
 
 (* Bad: mismatched names and native names *)
@@ -616,8 +620,8 @@ Error: Signature mismatch:
          external f : int -> int -> int = "f" "f_nat"
        is not included in
          external f : int -> int = "f" "f_nat"
-       The type int -> int -> int is not compatible with the type int -> int
-       Type int -> int is not compatible with type int
+       The type "int -> int -> int" is not compatible with the type "int -> int"
+       Type "int -> int" is not compatible with type "int"
 |}]
 
 (* Bad: unboxed or untagged with the wrong type *)
@@ -627,7 +631,8 @@ external g : (float [@untagged]) -> float = "g" "g_nat";;
 Line 1, characters 14-19:
 1 | external g : (float [@untagged]) -> float = "g" "g_nat";;
                   ^^^^^
-Error: Don't know how to untag this type. Only int can be untagged.
+Error: Don't know how to untag this type. Only "int"
+       and other immediate types can be untagged.
 |}]
 external h : (int [@unboxed]) -> float = "h" "h_nat";;
 [%%expect{|
@@ -635,7 +640,7 @@ Line 1, characters 14-17:
 1 | external h : (int [@unboxed]) -> float = "h" "h_nat";;
                   ^^^
 Error: Don't know how to unbox this type.
-       Only float, int32, int64 and nativeint can be unboxed.
+       Only "float", "int32", "int64", and "nativeint" can be unboxed.
 |}]
 
 (* Bad: unboxing the function type *)
@@ -645,7 +650,7 @@ Line 1, characters 13-25:
 1 | external i : int -> float [@unboxed] = "i" "i_nat";;
                  ^^^^^^^^^^^^
 Error: Don't know how to unbox this type.
-       Only float, int32, int64 and nativeint can be unboxed.
+       Only "float", "int32", "int64", and "nativeint" can be unboxed.
 |}]
 
 (* Bad: unboxing a "deep" sub-type. *)
@@ -654,7 +659,7 @@ external j : int -> (float [@unboxed]) * float = "j" "j_nat";;
 Line 1, characters 21-26:
 1 | external j : int -> (float [@unboxed]) * float = "j" "j_nat";;
                          ^^^^^
-Error: The attribute '@unboxed' should be attached to
+Error: The attribute "@unboxed" should be attached to
        a direct argument or result of the primitive,
        it should not occur deeply into its type.
 |}]
@@ -674,21 +679,21 @@ external l : float -> float = "l" "l_nat" "float" [@@unboxed];;
 Line 1, characters 0-61:
 1 | external l : float -> float = "l" "l_nat" "float" [@@unboxed];;
     ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-Error: Cannot use "float" in conjunction with [@unboxed]/[@untagged].
+Error: Cannot use "float" in conjunction with "[@unboxed]"/"[@untagged]".
 |}]
 external m : (float [@unboxed]) -> float = "m" "m_nat" "float";;
 [%%expect{|
 Line 1, characters 0-62:
 1 | external m : (float [@unboxed]) -> float = "m" "m_nat" "float";;
     ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-Error: Cannot use "float" in conjunction with [@unboxed]/[@untagged].
+Error: Cannot use "float" in conjunction with "[@unboxed]"/"[@untagged]".
 |}]
 external n : float -> float = "n" "noalloc" [@@noalloc];;
 [%%expect{|
 Line 1, characters 0-55:
 1 | external n : float -> float = "n" "noalloc" [@@noalloc];;
     ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-Error: Cannot use "noalloc" in conjunction with [@@noalloc].
+Error: Cannot use "noalloc" in conjunction with "[@@noalloc]".
 |}]
 
 (* Warnings: unboxed / untagged without any native implementation *)
@@ -697,48 +702,48 @@ external o : (float[@unboxed]) -> float = "o";;
 Line 1, characters 0-45:
 1 | external o : (float[@unboxed]) -> float = "o";;
     ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-Error: [@The native code version of the primitive is mandatory
-       when attributes [@untagged] or [@unboxed] are present.
+Error: The native code version of the primitive is mandatory
+       when attributes "[@untagged]" or "[@unboxed]" are present.
 |}]
 external p : float -> (float[@unboxed]) = "p";;
 [%%expect{|
 Line 1, characters 0-45:
 1 | external p : float -> (float[@unboxed]) = "p";;
     ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-Error: [@The native code version of the primitive is mandatory
-       when attributes [@untagged] or [@unboxed] are present.
+Error: The native code version of the primitive is mandatory
+       when attributes "[@untagged]" or "[@unboxed]" are present.
 |}]
 external q : (int[@untagged]) -> float = "q";;
 [%%expect{|
 Line 1, characters 0-44:
 1 | external q : (int[@untagged]) -> float = "q";;
     ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-Error: [@The native code version of the primitive is mandatory
-       when attributes [@untagged] or [@unboxed] are present.
+Error: The native code version of the primitive is mandatory
+       when attributes "[@untagged]" or "[@unboxed]" are present.
 |}]
 external r : int -> (int[@untagged]) = "r";;
 [%%expect{|
 Line 1, characters 0-42:
 1 | external r : int -> (int[@untagged]) = "r";;
     ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-Error: [@The native code version of the primitive is mandatory
-       when attributes [@untagged] or [@unboxed] are present.
+Error: The native code version of the primitive is mandatory
+       when attributes "[@untagged]" or "[@unboxed]" are present.
 |}]
 external s : int -> int = "s" [@@untagged];;
 [%%expect{|
 Line 1, characters 0-42:
 1 | external s : int -> int = "s" [@@untagged];;
     ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-Error: [@The native code version of the primitive is mandatory
-       when attributes [@untagged] or [@unboxed] are present.
+Error: The native code version of the primitive is mandatory
+       when attributes "[@untagged]" or "[@unboxed]" are present.
 |}]
 external t : float -> float = "t" [@@unboxed];;
 [%%expect{|
 Line 1, characters 0-45:
 1 | external t : float -> float = "t" [@@unboxed];;
     ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-Error: [@The native code version of the primitive is mandatory
-       when attributes [@untagged] or [@unboxed] are present.
+Error: The native code version of the primitive is mandatory
+       when attributes "[@untagged]" or "[@unboxed]" are present.
 |}]
 
 (* PR#7424 *)
@@ -764,6 +769,7 @@ versions of the compiler, breaking the primitive implementation.
 You should explicitly annotate the declaration of i
 with [@@boxed] or [@@unboxed], so that its external interface
 remains stable in the future.
+
 external id : i -> i = "%identity"
 |}];;
 
@@ -784,6 +790,7 @@ versions of the compiler, breaking the primitive implementation.
 You should explicitly annotate the declaration of i
 with [@@boxed] or [@@unboxed], so that its external interface
 remains stable in the future.
+
 Line 3, characters 0-34:
 3 | external id : i -> j = "%identity";;
     ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
@@ -795,6 +802,7 @@ versions of the compiler, breaking the primitive implementation.
 You should explicitly annotate the declaration of j
 with [@@boxed] or [@@unboxed], so that its external interface
 remains stable in the future.
+
 external id : i -> j = "%identity"
 |}];;
 
