@@ -1,6 +1,6 @@
 (* TEST
-   flags = " -w +A -strict-sequence "
-   * expect
+ flags = " -w +A -strict-sequence ";
+ expect;
 *)
 
 (* Ignore OCAMLRUNPARAM=b to be reproducible *)
@@ -29,6 +29,7 @@ Line 1, characters 0-1:
     ^
 Warning 41 [ambiguous-name]: A belongs to several types: a exn
 The first one was selected. Please disambiguate if this is wrong.
+
 - : a = A
 |}]
 ;;
@@ -40,6 +41,7 @@ Line 1, characters 6-7:
           ^
 Warning 42 [disambiguated-name]: this use of A relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Exception: A.
 |}]
 ;;
@@ -57,17 +59,20 @@ Line 1, characters 26-27:
                               ^
 Warning 42 [disambiguated-name]: this use of A relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 - : exn -> int = <fun>
 |}, Principal{|
 Line 1, characters 26-27:
 1 | function Not_found -> 1 | A -> 2 | _ -> 3
                               ^
 Warning 18 [not-principal]: this type-based constructor disambiguation is not principal.
+
 Line 1, characters 26-27:
 1 | function Not_found -> 1 | A -> 2 | _ -> 3
                               ^
 Warning 42 [disambiguated-name]: this use of A relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 - : exn -> int = <fun>
 |}]
 ;;
@@ -79,11 +84,13 @@ Line 1, characters 10-11:
               ^
 Warning 42 [disambiguated-name]: this use of A relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 1, characters 17-18:
 1 | try raise A with A -> 2
                      ^
 Warning 42 [disambiguated-name]: this use of A relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 - : int = 2
 |}]
 ;;
