@@ -1,6 +1,6 @@
 (* TEST
-   flags = " -w A -strict-sequence "
-   * expect
+ flags = " -w +A -strict-sequence ";
+ expect;
 *)
 
 type t = A : t;;
@@ -16,7 +16,8 @@ end;;
 Line 2, characters 10-11:
 2 |   let _f ~x (* x unused argument *) = function
               ^
-Warning 27: unused variable x.
+Warning 27 [unused-var-strict]: unused variable x.
+
 module X1 : sig end
 |}]
 
@@ -29,7 +30,8 @@ end;;
 Line 2, characters 6-7:
 2 |   let x = 42 (* unused value *)
           ^
-Warning 32: unused value x.
+Warning 32 [unused-value-declaration]: unused value x.
+
 module X2 : sig end
 |}]
 
@@ -44,10 +46,12 @@ end;;
 Line 2, characters 24-25:
 2 |   module O = struct let x = 42 (* unused *) end
                             ^
-Warning 32: unused value x.
+Warning 32 [unused-value-declaration]: unused value x.
+
 Line 3, characters 2-8:
 3 |   open O (* unused open *)
       ^^^^^^
-Warning 33: unused open O.
+Warning 33 [unused-open]: unused open O.
+
 module X3 : sig end
 |}]
