@@ -1,6 +1,6 @@
 (* TEST
-   flags = " -w A -strict-sequence "
-   * expect
+ flags = " -w +A -strict-sequence ";
+ expect;
 *)
 
 module A = struct type foo end;;
@@ -23,7 +23,8 @@ end = C;;
 Line 2, characters 2-8:
 2 |   open A
       ^^^^^^
-Warning 33: unused open A.
+Warning 33 [unused-open]: unused open A.
+
 module rec C : sig end
 |}]
 
@@ -39,12 +40,14 @@ end = D;;
 Line 5, characters 10-14:
 5 |       let None = None
               ^^^^
-Warning 8: this pattern-matching is not exhaustive.
+Warning 8 [partial-match]: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 Some _
+
 Line 4, characters 6-12:
 4 |       open A
           ^^^^^^
-Warning 33: unused open A.
+Warning 33 [unused-open]: unused open A.
+
 module rec D : sig module M : sig module X : sig end end end
 |}]
