@@ -1,6 +1,6 @@
 (* TEST
-   flags = " -w A -strict-sequence "
-   * expect
+ flags = " -w +A -strict-sequence ";
+ expect;
 *)
 
 (* Use type information *)
@@ -25,58 +25,69 @@ end;;
 Line 3, characters 19-20:
 3 |   let f1 (r:t) = r.x (* ok *)
                        ^
-Warning 42: this use of x relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of x relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 4, characters 29-30:
 4 |   let f2 r = ignore (r:t); r.x (* non principal *)
                                  ^
-Warning 42: this use of x relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of x relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 7, characters 18-19:
 7 |     match r with {x; y} -> y + y (* ok *)
                       ^
-Warning 42: this use of x relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of x relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 7, characters 21-22:
 7 |     match r with {x; y} -> y + y (* ok *)
                          ^
-Warning 42: this use of y relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of y relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 7, characters 18-19:
 7 |     match r with {x; y} -> y + y (* ok *)
                       ^
-Warning 27: unused variable x.
+Warning 27 [unused-var-strict]: unused variable x.
+
 module OK :
   sig val f1 : M1.t -> int val f2 : M1.t -> int val f3 : M1.t -> int end
 |}, Principal{|
 Line 3, characters 19-20:
 3 |   let f1 (r:t) = r.x (* ok *)
                        ^
-Warning 42: this use of x relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of x relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 4, characters 29-30:
 4 |   let f2 r = ignore (r:t); r.x (* non principal *)
                                  ^
-Warning 18: this type-based field disambiguation is not principal.
+Warning 18 [not-principal]: this type-based field disambiguation is not principal.
+
 Line 4, characters 29-30:
 4 |   let f2 r = ignore (r:t); r.x (* non principal *)
                                  ^
-Warning 42: this use of x relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of x relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 7, characters 18-19:
 7 |     match r with {x; y} -> y + y (* ok *)
                       ^
-Warning 42: this use of x relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of x relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 7, characters 21-22:
 7 |     match r with {x; y} -> y + y (* ok *)
                          ^
-Warning 42: this use of y relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of y relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 7, characters 18-19:
 7 |     match r with {x; y} -> y + y (* ok *)
                       ^
-Warning 27: unused variable x.
+Warning 27 [unused-var-strict]: unused variable x.
+
 module OK :
   sig val f1 : M1.t -> int val f2 : M1.t -> int val f3 : M1.t -> int end
 |}]
@@ -89,13 +100,13 @@ end;; (* fails *)
 Line 3, characters 25-31:
 3 |   let f r = match r with {x; y} -> y + y
                              ^^^^^^
-Warning 41: these field labels belong to several types: M1.u M1.t
+Warning 41 [ambiguous-name]: these field labels belong to several types: M1.u M1.t
 The first one was selected. Please disambiguate if this is wrong.
+
 Line 3, characters 35-36:
 3 |   let f r = match r with {x; y} -> y + y
                                        ^
-Error: This expression has type bool but an expression was expected of type
-         int
+Error: The value "y" has type "bool" but an expression was expected of type "int"
 |}]
 
 module F2 = struct
@@ -109,29 +120,45 @@ end;; (* fails for -principal *)
 Line 6, characters 8-9:
 6 |        {x; y} -> y + y
             ^
-Warning 42: this use of x relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of x relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 6, characters 11-12:
 6 |        {x; y} -> y + y
                ^
-Warning 42: this use of y relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of y relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 6, characters 8-9:
 6 |        {x; y} -> y + y
             ^
-Warning 27: unused variable x.
+Warning 27 [unused-var-strict]: unused variable x.
+
 module F2 : sig val f : M1.t -> int end
 |}, Principal{|
+Line 6, characters 8-9:
+6 |        {x; y} -> y + y
+            ^
+Warning 42 [disambiguated-name]: this use of x relies on type-directed disambiguation,
+it will not compile with OCaml 4.00 or earlier.
+
+Line 6, characters 11-12:
+6 |        {x; y} -> y + y
+               ^
+Warning 42 [disambiguated-name]: this use of y relies on type-directed disambiguation,
+it will not compile with OCaml 4.00 or earlier.
+
 Line 6, characters 7-13:
 6 |        {x; y} -> y + y
            ^^^^^^
-Warning 41: these field labels belong to several types: M1.u M1.t
-The first one was selected. Please disambiguate if this is wrong.
-Line 6, characters 7-13:
+Warning 18 [not-principal]: this type-based record disambiguation is not principal.
+
+Line 6, characters 8-9:
 6 |        {x; y} -> y + y
-           ^^^^^^
-Error: This pattern matches values of type M1.u
-       but a pattern was expected which matches values of type M1.t
+            ^
+Warning 27 [unused-var-strict]: unused variable x.
+
+module F2 : sig val f : M1.t -> int end
 |}]
 
 (* Use type information with modules*)
@@ -147,8 +174,9 @@ let f (r:M.t) = r.M.x;; (* ok *)
 Line 1, characters 18-21:
 1 | let f (r:M.t) = r.M.x;; (* ok *)
                       ^^^
-Warning 42: this use of x relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of x relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 val f : M.t -> int = <fun>
 |}]
 let f (r:M.t) = r.x;; (* warning *)
@@ -156,14 +184,16 @@ let f (r:M.t) = r.x;; (* warning *)
 Line 1, characters 18-19:
 1 | let f (r:M.t) = r.x;; (* warning *)
                       ^
-Warning 40: x was selected from type M.t.
+Warning 40 [name-out-of-scope]: x was selected from type M.t.
 It is not visible in the current scope, and will not
 be selected if the type becomes unknown.
+
 Line 1, characters 18-19:
 1 | let f (r:M.t) = r.x;; (* warning *)
                       ^
-Warning 42: this use of x relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of x relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 val f : M.t -> int = <fun>
 |}]
 let f ({x}:M.t) = x;; (* warning *)
@@ -171,14 +201,16 @@ let f ({x}:M.t) = x;; (* warning *)
 Line 1, characters 8-9:
 1 | let f ({x}:M.t) = x;; (* warning *)
             ^
-Warning 42: this use of x relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of x relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 1, characters 7-10:
 1 | let f ({x}:M.t) = x;; (* warning *)
            ^^^
-Warning 40: this record of type M.t contains fields that are
+Warning 40 [name-out-of-scope]: this record of type M.t contains fields that are
 not visible in the current scope: x.
 They will not be selected if the type becomes unknown.
+
 val f : M.t -> int = <fun>
 |}]
 
@@ -203,12 +235,14 @@ end;;
 Line 4, characters 20-21:
 4 |   let f (r:M.t) = r.x
                         ^
-Warning 42: this use of x relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of x relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 3, characters 2-8:
 3 |   open N
       ^^^^^^
-Warning 33: unused open N.
+Warning 33 [unused-open]: unused open N.
+
 module OK : sig val f : M.t -> int end
 |}]
 
@@ -253,14 +287,16 @@ end;; (* ok *)
 Line 3, characters 9-10:
 3 |   let f {x;z} = x,z
              ^
-Warning 42: this use of x relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of x relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 3, characters 8-13:
 3 |   let f {x;z} = x,z
             ^^^^^
-Warning 9: the following labels are not bound in this record pattern:
+Warning 9 [missing-record-field-pattern]: the following labels are not bound in this record pattern:
 y
 Either bind these labels explicitly or add '; _' to the pattern.
+
 module OK : sig val f : M.u -> bool * char end
 |}]
 module F3 = struct
@@ -271,12 +307,13 @@ end;; (* fail for missing label *)
 Line 3, characters 11-12:
 3 |   let r = {x=true;z='z'}
                ^
-Warning 42: this use of x relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of x relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 3, characters 10-24:
 3 |   let r = {x=true;z='z'}
               ^^^^^^^^^^^^^^
-Error: Some record fields are undefined: y
+Error: Some record fields are undefined: "y"
 |}]
 
 module OK = struct
@@ -288,13 +325,15 @@ end;; (* ok *)
 Line 4, characters 11-12:
 4 |   let r = {x=3; y=true}
                ^
-Warning 42: this use of x relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of x relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 4, characters 16-17:
 4 |   let r = {x=3; y=true}
                     ^
-Warning 42: this use of y relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of y relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 module OK :
   sig
     type u = { x : int; y : bool; }
@@ -314,8 +353,8 @@ end;; (* fail but don't warn *)
 Line 4, characters 22-23:
 4 |   let b : bar = {x=3; y=4}
                           ^
-Error: This record expression is expected to have type bar
-       The field y does not belong to type bar
+Error: This record expression is expected to have type "bar"
+       There is no field "y" within type "bar"
 |}]
 
 module M = struct type foo = {x:int;y:int} end;;
@@ -331,8 +370,8 @@ let r = { M.x = 3; N.y = 4; };; (* error: different definitions *)
 Line 1, characters 19-22:
 1 | let r = { M.x = 3; N.y = 4; };; (* error: different definitions *)
                        ^^^
-Error: The record field N.y belongs to the type N.bar
-       but is mixed here with fields of type M.foo
+Error: The record field "N.y" belongs to the type "N.bar"
+       but is mixed here with fields of type "M.foo"
 |}]
 
 module MN = struct include M include N end
@@ -354,18 +393,20 @@ let r = {MN.x = 3; NM.y = 4};; (* error: type would change with order *)
 Line 1, characters 8-28:
 1 | let r = {MN.x = 3; NM.y = 4};; (* error: type would change with order *)
             ^^^^^^^^^^^^^^^^^^^^
-Warning 41: x belongs to several types: MN.bar MN.foo
+Warning 41 [ambiguous-name]: x belongs to several types: MN.bar MN.foo
 The first one was selected. Please disambiguate if this is wrong.
+
 Line 1, characters 8-28:
 1 | let r = {MN.x = 3; NM.y = 4};; (* error: type would change with order *)
             ^^^^^^^^^^^^^^^^^^^^
-Warning 41: y belongs to several types: NM.foo NM.bar
+Warning 41 [ambiguous-name]: y belongs to several types: NM.foo NM.bar
 The first one was selected. Please disambiguate if this is wrong.
+
 Line 1, characters 19-23:
 1 | let r = {MN.x = 3; NM.y = 4};; (* error: type would change with order *)
                        ^^^^
-Error: The record field NM.y belongs to the type NM.foo = M.foo
-       but is mixed here with fields of type MN.bar = N.bar
+Error: The record field "NM.y" belongs to the type "NM.foo" = "M.foo"
+       but is mixed here with fields of type "MN.bar" = "N.bar"
 |}]
 
 (* Lpw25 *)
@@ -389,13 +430,14 @@ end;;
 Line 3, characters 37-38:
 3 |   let f r = ignore (r: foo); {r with x = 2; z = 3}
                                          ^
-Warning 42: this use of x relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of x relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 3, characters 44-45:
 3 |   let f r = ignore (r: foo); {r with x = 2; z = 3}
                                                 ^
-Error: This record expression is expected to have type M.foo
-       The field z does not belong to type M.foo
+Error: This record expression is expected to have type "M.foo"
+       There is no field "z" within type "M.foo"
 |}]
 module M = struct
   include M
@@ -417,13 +459,14 @@ end;;
 Line 3, characters 38-39:
 3 |   let f r = ignore (r: foo); { r with x = 3; a = 4 }
                                           ^
-Warning 42: this use of x relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of x relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 3, characters 45-46:
 3 |   let f r = ignore (r: foo); { r with x = 3; a = 4 }
                                                  ^
-Error: This record expression is expected to have type M.foo
-       The field a does not belong to type M.foo
+Error: This record expression is expected to have type "M.foo"
+       There is no field "a" within type "M.foo"
 |}]
 module F7 = struct
   open M
@@ -434,18 +477,20 @@ end;;
 Line 3, characters 11-12:
 3 |   let r = {x=1; y=2}
                ^
-Warning 42: this use of x relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of x relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 3, characters 16-17:
 3 |   let r = {x=1; y=2}
                     ^
-Warning 42: this use of y relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of y relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 4, characters 18-19:
 4 |   let r: other = {x=1; y=2}
                       ^
-Error: This record expression is expected to have type M.other
-       The field x does not belong to type M.other
+Error: This record expression is expected to have type "M.other"
+       There is no field "x" within type "M.other"
 |}]
 
 module A = struct type t = {x: int} end
@@ -459,8 +504,8 @@ let f (r : B.t) = r.A.x;; (* fail *)
 Line 1, characters 20-23:
 1 | let f (r : B.t) = r.A.x;; (* fail *)
                         ^^^
-Error: The field A.x belongs to the record type A.t
-       but a field was expected belonging to the record type B.t
+Error: The field "A.x" belongs to the record type "A.t"
+       but a field was expected belonging to the record type "B.t"
 |}]
 
 (* Spellchecking *)
@@ -473,9 +518,9 @@ end;;
 Line 3, characters 19-22:
 3 |   let a : t = {x=1;yyz=2}
                        ^^^
-Error: This record expression is expected to have type t
-       The field yyz does not belong to type t
-Hint: Did you mean yyy?
+Error: This record expression is expected to have type "t"
+       There is no field "yyz" within type "t"
+Hint: Did you mean "yyy"?
 |}]
 
 (* PR#6004 *)
@@ -496,8 +541,9 @@ class f (_ : 'a) (_ : 'a) = object end;;
 Line 1, characters 12-13:
 1 | class g = f A;; (* ok *)
                 ^
-Warning 42: this use of A relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of A relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 class g : f
 class f : 'a -> 'a -> object  end
 |}]
@@ -506,29 +552,34 @@ class g = f (A : t) A;; (* warn with -principal *)
 Line 1, characters 13-14:
 1 | class g = f (A : t) A;; (* warn with -principal *)
                  ^
-Warning 42: this use of A relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of A relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 1, characters 20-21:
 1 | class g = f (A : t) A;; (* warn with -principal *)
                         ^
-Warning 42: this use of A relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of A relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 class g : f
 |}, Principal{|
 Line 1, characters 13-14:
 1 | class g = f (A : t) A;; (* warn with -principal *)
                  ^
-Warning 42: this use of A relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of A relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 1, characters 20-21:
 1 | class g = f (A : t) A;; (* warn with -principal *)
                         ^
-Warning 18: this type-based constructor disambiguation is not principal.
+Warning 18 [not-principal]: this type-based constructor disambiguation is not principal.
+
 Line 1, characters 20-21:
 1 | class g = f (A : t) A;; (* warn with -principal *)
                         ^
-Warning 42: this use of A relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of A relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 class g : f
 |}]
 
@@ -547,12 +598,14 @@ end;;
 Line 7, characters 15-16:
 7 |   let y : t = {x = 0}
                    ^
-Warning 42: this use of x relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of x relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 Line 6, characters 2-8:
 6 |   open M  (* this open is unused, it isn't reported as shadowing 'x' *)
       ^^^^^^
-Warning 33: unused open M.
+Warning 33 [unused-open]: unused open M.
+
 module Shadow1 :
   sig
     type t = { x : int; }
@@ -572,12 +625,14 @@ end;;
 Line 6, characters 2-8:
 6 |   open M  (* this open shadows label 'x' *)
       ^^^^^^
-Warning 45: this open statement shadows the label x (which is later used)
+Warning 45 [open-shadow-label-constructor]: this open statement shadows the label x (which is later used)
+
 Line 7, characters 10-18:
 7 |   let y = {x = ""}
               ^^^^^^^^
-Warning 41: these field labels belong to several types: M.s t
+Warning 41 [ambiguous-name]: these field labels belong to several types: M.s t
 The first one was selected. Please disambiguate if this is wrong.
+
 module Shadow2 :
   sig
     type t = { x : int; }
@@ -598,8 +653,9 @@ end;;
 Line 5, characters 37-40:
 5 |   let f (u : u) = match u with `Key {loc} -> loc
                                          ^^^
-Warning 42: this use of loc relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of loc relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 module P6235 :
   sig
     type t = { loc : string; }
@@ -623,8 +679,9 @@ end;;
 Line 7, characters 11-14:
 7 |     |`Key {loc} -> loc
                ^^^
-Warning 42: this use of loc relies on type-directed disambiguation,
+Warning 42 [disambiguated-name]: this use of loc relies on type-directed disambiguation,
 it will not compile with OCaml 4.00 or earlier.
+
 module P6235' :
   sig
     type t = { loc : string; }
@@ -633,23 +690,24 @@ module P6235' :
     val f : u -> string
   end
 |}, Principal{|
+Line 7, characters 11-14:
+7 |     |`Key {loc} -> loc
+               ^^^
+Warning 42 [disambiguated-name]: this use of loc relies on type-directed disambiguation,
+it will not compile with OCaml 4.00 or earlier.
+
 Line 7, characters 10-15:
 7 |     |`Key {loc} -> loc
               ^^^^^
-Warning 41: these field labels belong to several types: v t
-The first one was selected. Please disambiguate if this is wrong.
-Line 7, characters 10-15:
-7 |     |`Key {loc} -> loc
-              ^^^^^
-Warning 9: the following labels are not bound in this record pattern:
-x
-Either bind these labels explicitly or add '; _' to the pattern.
-Line 7, characters 5-15:
-7 |     |`Key {loc} -> loc
-         ^^^^^^^^^^
-Error: This pattern matches values of type [? `Key of v ]
-       but a pattern was expected which matches values of type u
-       Types for tag `Key are incompatible
+Warning 18 [not-principal]: this type-based record disambiguation is not principal.
+
+module P6235' :
+  sig
+    type t = { loc : string; }
+    type v = { loc : string; x : int; }
+    type u = [ `Key of t ]
+    val f : u -> string
+  end
 |}]
 
 (** no candidates after filtering;
@@ -667,6 +725,70 @@ type u = { a : int; }
 Line 5, characters 12-15:
 5 | let _ = ( { M.x=0 } : u );;
                 ^^^
-Error: The field M.x belongs to the record type M.t
-       but a field was expected belonging to the record type u
+Error: The field "M.x" belongs to the record type "M.t"
+       but a field was expected belonging to the record type "u"
+|}]
+
+(* PR#8747 *)
+module M = struct type t = { x : int; y: char } end
+let f (x : M.t) = { x with y = 'a' }
+let g (x : M.t) = { x with y = 'a' } :: []
+let h (x : M.t) = { x with y = 'a' } :: { x with y = 'b' } :: [];;
+[%%expect{|
+module M : sig type t = { x : int; y : char; } end
+Line 2, characters 27-28:
+2 | let f (x : M.t) = { x with y = 'a' }
+                               ^
+Warning 42 [disambiguated-name]: this use of y relies on type-directed disambiguation,
+it will not compile with OCaml 4.00 or earlier.
+
+Line 2, characters 18-36:
+2 | let f (x : M.t) = { x with y = 'a' }
+                      ^^^^^^^^^^^^^^^^^^
+Warning 40 [name-out-of-scope]: this record of type M.t contains fields that are
+not visible in the current scope: y.
+They will not be selected if the type becomes unknown.
+
+val f : M.t -> M.t = <fun>
+Line 3, characters 27-28:
+3 | let g (x : M.t) = { x with y = 'a' } :: []
+                               ^
+Warning 42 [disambiguated-name]: this use of y relies on type-directed disambiguation,
+it will not compile with OCaml 4.00 or earlier.
+
+Line 3, characters 18-36:
+3 | let g (x : M.t) = { x with y = 'a' } :: []
+                      ^^^^^^^^^^^^^^^^^^
+Warning 40 [name-out-of-scope]: this record of type M.t contains fields that are
+not visible in the current scope: y.
+They will not be selected if the type becomes unknown.
+
+val g : M.t -> M.t list = <fun>
+Line 4, characters 27-28:
+4 | let h (x : M.t) = { x with y = 'a' } :: { x with y = 'b' } :: [];;
+                               ^
+Warning 42 [disambiguated-name]: this use of y relies on type-directed disambiguation,
+it will not compile with OCaml 4.00 or earlier.
+
+Line 4, characters 18-36:
+4 | let h (x : M.t) = { x with y = 'a' } :: { x with y = 'b' } :: [];;
+                      ^^^^^^^^^^^^^^^^^^
+Warning 40 [name-out-of-scope]: this record of type M.t contains fields that are
+not visible in the current scope: y.
+They will not be selected if the type becomes unknown.
+
+Line 4, characters 49-50:
+4 | let h (x : M.t) = { x with y = 'a' } :: { x with y = 'b' } :: [];;
+                                                     ^
+Warning 42 [disambiguated-name]: this use of y relies on type-directed disambiguation,
+it will not compile with OCaml 4.00 or earlier.
+
+Line 4, characters 40-58:
+4 | let h (x : M.t) = { x with y = 'a' } :: { x with y = 'b' } :: [];;
+                                            ^^^^^^^^^^^^^^^^^^
+Warning 40 [name-out-of-scope]: this record of type M.t contains fields that are
+not visible in the current scope: y.
+They will not be selected if the type becomes unknown.
+
+val h : M.t -> M.t list = <fun>
 |}]
