@@ -1,10 +1,8 @@
 (* TEST
-include config
-include testing
-binary_modules =
-  "config build_path_prefix_map misc identifiable numbers \
-   strongly_connected_components"
-* bytecode
+ include config;
+ include testing;
+ binary_modules = "config build_path_prefix_map format_doc misc identifiable numbers strongly_connected_components";
+ bytecode;
 *)
 
 module Int = Numbers.Int
