@@ -1,11 +1,11 @@
-(* TEST
+(* TEST_BELOW
+(* Blank lines added here to preserve locations. *)
+
+
+
+
 
-flags = "-w +A-70"
 
-* setup-ocamlc.byte-build-env
-** ocamlc.byte
-compile_only = "true"
-*** check-ocamlc.byte-output
 
 *)
 
@@ -37,3 +37,11 @@ let silent_fragile2 (type t) (r1 : t repr) (r2 : t repr) (t : t) =
   | _, XY, X -> ()
   | AB, _, _ -> ()
   | _, XY, _ -> ()
+
+(* TEST
+ flags = "-w +A-70";
+ setup-ocamlc.byte-build-env;
+ compile_only = "true";
+ ocamlc.byte;
+ check-ocamlc.byte-output;
+*)
