@@ -1,22 +1,22 @@
-(* TEST
+(* TEST_BELOW
+(* Blank lines added here to preserve locations. *)
+
+
+
+
+
+
+
+
+
+
+
+
+
 
-flags = "-w +A-70"
-compile_only = "true"
 
-* setup-ocamlc.byte-build-env
-** ocamlc.byte
-*** check-ocamlc.byte-output
 
-* no-flambda
-** setup-ocamlopt.byte-build-env
-*** ocamlopt.byte
-**** check-ocamlopt.byte-output
 
-* flambda
-compiler_reference = "${test_source_directory}/w55.flambda.reference"
-** setup-ocamlopt.byte-build-env
-*** ocamlopt.byte
-**** check-ocamlopt.byte-output
 
 *)
 
@@ -50,3 +50,24 @@ let h' x = (j [@inlined hint]) x
 let b' x y = (a [@inlined hint]) x y
 
 let d' x = (c [@inlined hint]) x
+
+(* TEST
+ flags = "-w +A-70";
+ compile_only = "true";
+ {
+   setup-ocamlc.byte-build-env;
+   ocamlc.byte;
+   check-ocamlc.byte-output;
+ }{
+   no-flambda;
+   setup-ocamlopt.byte-build-env;
+   ocamlopt.byte;
+   check-ocamlopt.byte-output;
+ }{
+   compiler_reference = "${test_source_directory}/w55.flambda.reference";
+   flambda;
+   setup-ocamlopt.byte-build-env;
+   ocamlopt.byte;
+   check-ocamlopt.byte-output;
+ }
+*)
