@@ -15,16 +15,15 @@
 
 (* Disassembler for executable and .cmo object files *)
 
-open Asttypes
 open Config
 open Instruct
-open Lambda
 open Location
 open Opcodes
 open Opnames
 open Cmo_format
 open Printf
 
+let print_banners = ref true
 let print_locations = ref true
 let print_reloc_info = ref false
 
@@ -48,8 +47,7 @@ let inputs ic =
 (* Global variables *)
 
 type global_table_entry =
-    Empty
-  | Global of Ident.t
+  | Glob of Symtable.Global.t
   | Constant of Obj.t
 
 let start = ref 0                              (* Position of beg. of code *)
@@ -75,44 +73,10 @@ let record_events orig evl =
       Hashtbl.add event_table ev.ev_pos ev)
     evl
 
-(* Print a structured constant *)
-
-let print_float f =
-  if String.contains f '.'
-  then printf "%s" f
-  else printf "%s." f
-;;
-
-let rec print_struct_const = function
-    Const_base(Const_int i) -> printf "%d" i
-  | Const_base(Const_float f) -> print_float f
-  | Const_base(Const_string (s, _, _)) -> printf "%S" s
-  | Const_immstring s -> printf "%S" s
-  | Const_base(Const_char c) -> printf "%C" c
-  | Const_base(Const_int32 i) -> printf "%ldl" i
-  | Const_base(Const_nativeint i) -> printf "%ndn" i
-  | Const_base(Const_int64 i) -> printf "%LdL" i
-  | Const_pointer n -> printf "%da" n
-  | Const_block(tag, args) ->
-      printf "<%d>" tag;
-      begin match args with
-        [] -> ()
-      | [a1] ->
-          printf "("; print_struct_const a1; printf ")"
-      | a1::al ->
-          printf "("; print_struct_const a1;
-          List.iter (fun a -> printf ", "; print_struct_const a) al;
-          printf ")"
-      end
-  | Const_float_array a ->
-      printf "[|";
-      List.iter (fun f -> print_float f; printf "; ") a;
-      printf "|]"
-
 (* Print an obj *)
 
 let same_custom x y =
-  Obj.field x 0 = Obj.field (Obj.repr y) 0
+  Nativeint.equal (Obj.raw_field x 0) (Obj.raw_field (Obj.repr y) 0)
 
 let rec print_obj x =
   if Obj.is_block x then begin
@@ -168,13 +132,25 @@ let find_reloc ic =
 
 (* Symbolic printing of global names, etc *)
 
+let print_unexpected_reloc reloc_constr reloc_arg =
+  printf "<unexpected (%s '%s') reloc>" reloc_constr reloc_arg
+
+let print_unexpected_reloc_literal sc =
+  printf "<unexpected (Reloc_literal ";
+  print_obj sc;
+  printf ") reloc>"
+
 let print_getglobal_name ic =
   if !objfile then begin
     begin try
       match find_reloc ic with
-          Reloc_getglobal id -> print_string (Ident.name id)
-        | Reloc_literal sc -> print_struct_const sc
-        | _ -> print_string "<wrong reloc>"
+        | Reloc_getcompunit (Compunit cu_name) -> print_string cu_name
+        | Reloc_getpredef (Predef_exn predef_exn) -> print_string predef_exn
+        | Reloc_literal sc -> print_obj sc
+        | Reloc_setcompunit (Compunit cu_name) ->
+          print_unexpected_reloc "Reloc_setcompunit" cu_name
+        | Reloc_primitive prim ->
+          print_unexpected_reloc "Reloc_primitive" prim
     with Not_found ->
       print_string "<no reloc>"
     end;
@@ -185,17 +161,25 @@ let print_getglobal_name ic =
     if n >= Array.length !globals || n < 0
     then print_string "<global table overflow>"
     else match !globals.(n) with
-           Global id -> print_string(Ident.name id)
+         | Glob glob ->
+             let desc = Format_doc.compat Symtable.Global.description in
+             print_string (Format.asprintf "%a" desc glob)
          | Constant obj -> print_obj obj
-         | _ -> print_string "???"
   end
 
 let print_setglobal_name ic =
   if !objfile then begin
     begin try
       match find_reloc ic with
-        Reloc_setglobal id -> print_string (Ident.name id)
-      | _ -> print_string "<wrong reloc>"
+      | Reloc_setcompunit (Compunit cu_name) -> print_string cu_name
+      | Reloc_literal sc ->
+        print_unexpected_reloc_literal sc
+      | Reloc_getcompunit (Compunit cu_name) ->
+        print_unexpected_reloc "Reloc_getcompunit" cu_name
+      | Reloc_getpredef (Predef_exn predef_exn) ->
+        print_unexpected_reloc "Reloc_getpredef" predef_exn
+      | Reloc_primitive prim ->
+        print_unexpected_reloc "Reloc_primitive" prim
     with Not_found ->
       print_string "<no reloc>"
     end;
@@ -206,8 +190,10 @@ let print_setglobal_name ic =
     if n >= Array.length !globals || n < 0
     then print_string "<global table overflow>"
     else match !globals.(n) with
-           Global id -> print_string(Ident.name id)
-         | _ -> print_string "???"
+         | Glob glob ->
+             let desc = Format_doc.compat Symtable.Global.description in
+             print_string (Format.asprintf "%a" desc glob)
+         | Constant _ -> print_string "<unexpected constant>"
   end
 
 let print_primitive ic =
@@ -215,7 +201,14 @@ let print_primitive ic =
     begin try
       match find_reloc ic with
         Reloc_primitive s -> print_string s
-      | _ -> print_string "<wrong reloc>"
+      | Reloc_literal sc ->
+        print_unexpected_reloc_literal sc
+      | Reloc_getcompunit (Compunit cu_name) ->
+          print_unexpected_reloc "Reloc_getcompunit" cu_name
+      | Reloc_setcompunit (Compunit cu_name) ->
+        print_unexpected_reloc "Reloc_setcompunit" cu_name
+      | Reloc_getpredef (Predef_exn predef_exn) ->
+        print_unexpected_reloc "Reloc_getpredef" predef_exn
     with Not_found ->
       print_string "<no reloc>"
     end;
@@ -249,7 +242,6 @@ type shape =
   | Switch
   | Closurerec
   | Pubmet
-;;
 
 let op_shapes = [
   opACC0, Nothing;
@@ -297,13 +289,13 @@ let op_shapes = [
   opGRAB, Uint;
   opCLOSURE, Uint_Disp;
   opCLOSUREREC, Closurerec;
-  opOFFSETCLOSUREM2, Nothing;
+  opOFFSETCLOSUREM3, Nothing;
   opOFFSETCLOSURE0, Nothing;
-  opOFFSETCLOSURE2, Nothing;
+  opOFFSETCLOSURE3, Nothing;
   opOFFSETCLOSURE, Sint;  (* was Uint *)
-  opPUSHOFFSETCLOSUREM2, Nothing;
+  opPUSHOFFSETCLOSUREM3, Nothing;
   opPUSHOFFSETCLOSURE0, Nothing;
-  opPUSHOFFSETCLOSURE2, Nothing;
+  opPUSHOFFSETCLOSURE3, Nothing;
   opPUSHOFFSETCLOSURE, Sint; (* was Nothing *)
   opGETGLOBAL, Getglobal;
   opPUSHGETGLOBAL, Getglobal;
@@ -396,12 +388,16 @@ let op_shapes = [
   opUGEINT, Nothing;
   opBULTINT, Uint_Disp;
   opBUGEINT, Uint_Disp;
+  opPERFORM, Nothing;
+  opRESUME, Nothing;
+  opRESUMETERM, Uint;
+  opREPERFORMTERM, Uint;
   opSTOP, Nothing;
   opEVENT, Nothing;
   opBREAK, Nothing;
   opRERAISE, Nothing;
   opRAISE_NOTRACE, Nothing;
-];;
+]
 
 let print_event ev =
   if !print_locations then
@@ -421,7 +417,7 @@ let print_instr ic =
   else print_string names_of_instructions.(op);
   begin try
     let shape = List.assoc op op_shapes in
-    if shape <> Nothing then print_string " ";
+    if shape <> Nothing && shape <> Switch then print_string " ";
     match shape with
     | Uint -> print_int (inputu ic)
     | Sint -> print_int (inputs ic)
@@ -468,8 +464,7 @@ let print_instr ic =
     | Nothing -> ()
   with Not_found -> print_string " (unknown arguments)"
   end;
-  print_string "\n";
-;;
+  print_string "\n"
 
 (* Disassemble a block of code *)
 
@@ -483,10 +478,15 @@ let print_code ic len =
 let print_reloc (info, pos) =
   printf "    %d    (%d)    " pos (pos/4);
   match info with
-    Reloc_literal sc -> print_struct_const sc; printf "\n"
-  | Reloc_getglobal id -> printf "require    %s\n" (Ident.name id)
-  | Reloc_setglobal id -> printf "provide    %s\n" (Ident.name id)
-  | Reloc_primitive s -> printf "prim    %s\n" s
+  | Reloc_literal sc -> print_obj sc; printf "\n"
+  | Reloc_getcompunit (Compunit cu_name) ->
+    printf "require        %s\n" cu_name
+  | Reloc_getpredef (Predef_exn predef_exn) ->
+    printf "require predef %s\n" predef_exn
+  | Reloc_setcompunit (Compunit cu_name) ->
+    printf "provide        %s\n" cu_name
+  | Reloc_primitive s ->
+    printf "prim           %s\n" s
 
 (* Print a .cmo file *)
 
@@ -503,50 +503,46 @@ let dump_obj ic =
     List.iter print_reloc cu.cu_reloc;
   if cu.cu_debug > 0 then begin
     seek_in ic cu.cu_debug;
-    let evl = (input_value ic : debug_event list) in
-    ignore (input_value ic); (* Skip the list of absolute directory names *)
+    let evl = (Compression.input_value ic : debug_event list) in
+    ignore (Compression.input_value ic);
+                (* Skip the list of absolute directory names *)
     record_events 0 evl
   end;
   seek_in ic cu.cu_pos;
   print_code ic cu.cu_codesize
 
-(* Read the primitive table from an executable *)
-
-let read_primitive_table ic len =
-  let p = really_input_string ic len in
-  String.split_on_char '\000' p |> List.filter ((<>) "") |> Array.of_list
-
 (* Print an executable file *)
 
 let dump_exe ic =
-  Bytesections.read_toc ic;
-  let prim_size = Bytesections.seek_section ic "PRIM" in
-  primitives := read_primitive_table ic prim_size;
-  ignore(Bytesections.seek_section ic "DATA");
-  let init_data = (input_value ic : Obj.t array) in
-  globals := Array.make (Array.length init_data) Empty;
-  for i = 0 to Array.length init_data - 1 do
-    !globals.(i) <- Constant (init_data.(i))
-  done;
-  ignore(Bytesections.seek_section ic "SYMB");
-  let sym_table = (input_value ic : Symtable.global_map) in
+  let toc = Bytesections.read_toc ic in
+(* Read the primitive table from an executable *)
+  let prims = Bytesections.read_section_string toc ic Bytesections.Name.PRIM in
+  primitives := Array.of_list (Misc.split_null_terminated prims);
+  let init_data : Obj.t array =
+    Bytesections.read_section_struct toc ic Bytesections.Name.DATA in
+  globals := Array.map (fun x -> Constant x) init_data;
+  let sym_table : Symtable.global_map =
+    Bytesections.read_section_struct toc ic Bytesections.Name.SYMB in
   Symtable.iter_global_map
-    (fun id pos -> !globals.(pos) <- Global id) sym_table;
-  begin try
-    ignore (Bytesections.seek_section ic "DBUG");
-    let num_eventlists = input_binary_int ic in
-    for _i = 1 to num_eventlists do
-      let orig = input_binary_int ic in
-      let evl = (input_value ic : debug_event list) in
-      ignore (input_value ic); (* Skip the list of absolute directory names *)
-      record_events orig evl
-    done
-  with Not_found -> ()
+    (fun global pos -> !globals.(pos) <- Glob global) sym_table;
+  begin
+    match Bytesections.seek_section toc ic Bytesections.Name.DBUG with
+    | exception Not_found -> ()
+    | (_ : int) ->
+        let num_eventlists = input_binary_int ic in
+        for _i = 1 to num_eventlists do
+          let orig = input_binary_int ic in
+          let evl = (Compression.input_value ic : debug_event list) in
+          (* Skip the list of absolute directory names *)
+          ignore (Compression.input_value ic);
+          record_events orig evl
+        done
   end;
-  let code_size = Bytesections.seek_section ic "CODE" in
+  let code_size = Bytesections.seek_section toc ic Bytesections.Name.CODE in
   print_code ic code_size
 
 let arg_list = [
+  "-nobanners", Arg.Clear print_banners, " : don't print banners";
   "-noloc", Arg.Clear print_locations, " : don't print source information";
   "-reloc", Arg.Set print_reloc_info, " : print relocation information";
   "-args", Arg.Expand Arg.read_arg,
@@ -566,16 +562,16 @@ let arg_fun filename =
   let ic = open_in_bin filename in
   if not !first_file then print_newline ();
   first_file := false;
-  printf "## start of ocaml dump of %S\n%!" filename;
+  if !print_banners then printf "## start of ocaml dump of %S\n%!" filename;
   begin try
           objfile := false; dump_exe ic
     with Bytesections.Bad_magic_number ->
       objfile := true; seek_in ic 0; dump_obj ic
   end;
   close_in ic;
-  printf "## end of ocaml dump of %S\n%!" filename
+  if !print_banners then printf "## end of ocaml dump of %S\n%!" filename
 
-let main() =
+let main () =
   Arg.parse_expand arg_list arg_fun arg_usage;
     exit 0
 
