@@ -17,7 +17,6 @@
 
 open Format
 open Misc
-open Longident
 open Types
 open Toploop
 
@@ -76,7 +75,7 @@ let _ = add_directive "quit" (Directive_none dir_quit)
 let dir_directory s =
   let d = expand_directory Config.standard_library s in
   Dll.add_path [d];
-  let dir = Load_path.Dir.create d in
+  let dir = Load_path.Dir.create ~hidden:false d in
   Load_path.prepend_dir dir;
   toplevel_env :=
     Stdlib.String.Set.fold
@@ -96,7 +95,7 @@ let _ = add_directive "directory" (Directive_string dir_directory)
 let dir_remove_directory s =
   let d = expand_directory Config.standard_library s in
   let keep id =
-    match Load_path.find_uncap (Ident.name id ^ ".cmi") with
+    match Load_path.find_normalized (Ident.name id ^ ".cmi") with
     | exception Not_found -> true
     | fn -> Filename.dirname fn <> d
   in
@@ -111,7 +110,7 @@ let _ = add_directive "remove_directory" (Directive_string dir_remove_directory)
     }
 
 let dir_show_dirs () =
-  List.iter print_endline (Load_path.get_paths ())
+  List.iter print_endline (Load_path.get_path_list ())
 
 let _ = add_directive "show_dirs" (Directive_none dir_show_dirs)
     {
@@ -184,7 +183,20 @@ let _ = add_directive "mod_use" (Directive_string (with_error_fmt dir_mod_use))
 
 (* Install, remove a printer *)
 
-exception Bad_printing_function
+module Printer = struct
+  type kind =
+    | Old of Types.type_expr
+      (* 'a -> unit *)
+    | Simple of Types.type_expr
+      (* Format.formatter -> 'a -> unit *)
+    | Generic of { ty_path: Path.t; arity: int; }
+      (* (formatter -> 'a1 -> unit) ->
+         (formatter -> 'a2 -> unit) ->
+         ... ->
+         (formatter -> 'an -> unit) ->
+         formatter -> ('a1, 'a2, ..., 'an) t -> unit
+      *)
+end
 
 let filter_arrow ty =
   let ty = Ctype.expand_head !toplevel_env ty in
@@ -192,142 +204,145 @@ let filter_arrow ty =
   | Tarrow (lbl, l, r, _) when not (Btype.is_optional lbl) -> Some (l, r)
   | _ -> None
 
-let rec extract_last_arrow desc =
-  match filter_arrow desc with
-  | None -> raise Bad_printing_function
-  | Some (_, r as res) ->
-      try extract_last_arrow r
-      with Bad_printing_function -> res
+let extract_last_arrow ty =
+  let rec extract last ty =
+    match filter_arrow ty with
+    | None -> last
+    | Some ((_, rest) as next) -> extract (Some next) rest
+  in extract None ty
+
+let extract_target_type ty =
+  Option.map fst (extract_last_arrow ty)
 
-let extract_target_type ty = fst (extract_last_arrow ty)
 let extract_target_parameters ty =
-  let ty = extract_target_type ty |> Ctype.expand_head !toplevel_env in
-  match get_desc ty with
-  | Tconstr (path, (_ :: _ as args), _)
-      when Ctype.all_distinct_vars !toplevel_env args ->
-        Some (path, args)
-  | _ -> None
+  match extract_target_type ty with
+  | None -> None
+  | Some tgt ->
+      let tgt = Ctype.expand_head !toplevel_env tgt in
+      match get_desc tgt with
+      | Tconstr (path, (_ :: _ as args), _)
+        when Ctype.all_distinct_vars !toplevel_env args ->
+          Some (path, args)
+      | _ -> None
 
-type 'a printer_type_new = Format.formatter -> 'a -> unit
-type 'a printer_type_old = 'a -> unit
-
-let printer_type ppf typename =
-  let printer_type =
-    match
-      Env.find_type_by_name
-        (Ldot(Lident "Topdirs", typename)) !toplevel_env
-    with
-    | path, _ -> path
-    | exception Not_found ->
-        fprintf ppf "Cannot find type Topdirs.%s.@." typename;
-        raise Exit
+let match_simple_printer_type desc ~is_old_style =
+  let make_printer_type =
+    if is_old_style
+    then Topprinters.printer_type_old
+    else Topprinters.printer_type_new
   in
-  printer_type
+  match
+    Ctype.with_local_level_generalize begin fun () ->
+      let ty_arg = Ctype.newvar() in
+      Ctype.unify !toplevel_env
+        (make_printer_type ty_arg)
+        (Ctype.instance desc.val_type);
+      ty_arg
+    end
+  with
+  | exception Ctype.Unify _ -> None
+  | ty_arg ->
+      if is_old_style
+      then Some (Printer.Old ty_arg)
+      else Some (Printer.Simple ty_arg)
 
-let match_simple_printer_type desc printer_type =
-  Ctype.begin_def();
-  let ty_arg = Ctype.newvar() in
-  begin try
-    Ctype.unify !toplevel_env
-      (Ctype.newconstr printer_type [ty_arg])
-      (Ctype.instance desc.val_type);
-  with Ctype.Unify _ ->
-    raise Bad_printing_function
-  end;
-  Ctype.end_def();
-  Ctype.generalize ty_arg;
-  (ty_arg, None)
+let match_generic_printer_type desc ty_path params =
+  let make_printer_type = Topprinters.printer_type_new in
+  match
+    Ctype.with_local_level_generalize begin fun () ->
+      let args = List.map (fun _ -> Ctype.newvar ()) params in
+      let ty_target = Ctype.newty (Tconstr (ty_path, args, ref Mnil)) in
+      let printer_args_ty =
+        List.map (fun ty_var -> make_printer_type ty_var) args in
+      let ty_expected =
+        List.fold_right Topprinters.type_arrow
+          printer_args_ty (make_printer_type ty_target) in
+      Ctype.unify !toplevel_env
+        ty_expected
+        (Ctype.instance desc.val_type);
+      args
+    end
+  with
+  | exception Ctype.Unify _ -> None
+  | args ->
+      if Ctype.all_distinct_vars !toplevel_env args
+      then Some ()
+      else None
 
-let match_generic_printer_type desc path args printer_type =
-  Ctype.begin_def();
-  let args = List.map (fun _ -> Ctype.newvar ()) args in
-  let ty_target = Ctype.newty (Tconstr (path, args, ref Mnil)) in
-  let ty_args =
-    List.map (fun ty_var -> Ctype.newconstr printer_type [ty_var]) args in
-  let ty_expected =
-    List.fold_right
-      (fun ty_arg ty -> Ctype.newty (Tarrow (Asttypes.Nolabel, ty_arg, ty,
-                                             commu_var ())))
-      ty_args (Ctype.newconstr printer_type [ty_target]) in
-  begin try
-    Ctype.unify !toplevel_env
-      ty_expected
-      (Ctype.instance desc.val_type);
-  with Ctype.Unify _ ->
-    raise Bad_printing_function
-  end;
-  Ctype.end_def();
-  Ctype.generalize ty_expected;
-  if not (Ctype.all_distinct_vars !toplevel_env args) then
-    raise Bad_printing_function;
-  (ty_expected, Some (path, ty_args))
+let match_printer_type desc =
+  match match_simple_printer_type desc ~is_old_style:false with
+  | Some _ as res -> res
+  | None ->
+  match match_simple_printer_type desc ~is_old_style:true with
+  | Some _ as res -> res
+  | None ->
+  match extract_target_parameters desc.val_type with
+  | None -> None
+  | Some (ty_path, args) ->
+    match match_generic_printer_type desc ty_path args with
+    | None -> None
+    | Some () ->
+      Some (Printer.Generic { ty_path; arity = List.length args; })
 
-let match_printer_type ppf desc =
-  let printer_type_new = printer_type ppf "printer_type_new" in
-  let printer_type_old = printer_type ppf "printer_type_old" in
-  try
-    (match_simple_printer_type desc printer_type_new, false)
-  with Bad_printing_function ->
-    try
-      (match_simple_printer_type desc printer_type_old, true)
-    with Bad_printing_function as exn ->
-      match extract_target_parameters desc.val_type with
-      | None -> raise exn
-      | Some (path, args) ->
-          (match_generic_printer_type desc path args printer_type_new,
-           false)
-
-let find_printer_type ppf lid =
+let find_printer lid =
   match Env.find_value_by_name lid !toplevel_env with
-  | (path, desc) -> begin
-    match match_printer_type ppf desc with
-    | (ty_arg, is_old_style) -> (ty_arg, path, is_old_style)
-    | exception Bad_printing_function ->
-      fprintf ppf "%a has the wrong type for a printing function.@."
-      Printtyp.longident lid;
-      raise Exit
-  end
   | exception Not_found ->
-      fprintf ppf "Unbound value %a.@." Printtyp.longident lid;
-      raise Exit
+    let report ppf =
+      fprintf ppf "Unbound value %a.@."
+        Printtyp.longident lid
+    in Error report
+  | (path, desc) ->
+    match match_printer_type desc with
+    | None ->
+      let report ppf =
+        fprintf ppf "%a has the wrong type for a printing function.@."
+          Printtyp.longident lid
+      in Error report
+    | Some kind -> Ok (path, kind)
+
+let install_printer_by_kind path kind =
+  let v = eval_value_path !toplevel_env path in
+  match kind with
+  | Printer.Old ty_arg ->
+    install_printer path ty_arg
+      (fun _formatter repr -> Obj.obj v (Obj.obj repr))
+  | Printer.Simple ty_arg ->
+    install_printer path ty_arg
+      (fun formatter repr -> Obj.obj v formatter (Obj.obj repr))
+  | Printer.Generic { ty_path; arity } ->
+     let rec build v = function
+       | 0 ->
+          Zero
+            (fun formatter repr -> Obj.obj v formatter (Obj.obj repr))
+       | n ->
+          Succ
+            (fun fn -> build ((Obj.obj v : _ -> Obj.t) fn) (n - 1)) in
+     install_generic_printer' path ty_path (build v arity)
+
+let remove_installed_printer path =
+  match remove_printer path with
+  | () -> Ok ()
+  | exception Not_found ->
+    let report ppf =
+      fprintf ppf "The printer named %a is not installed.@."
+        Printtyp.path path
+    in Error report
 
 let dir_install_printer ppf lid =
-  try
-    let ((ty_arg, ty), path, is_old_style) =
-      find_printer_type ppf lid in
-    let v = eval_value_path !toplevel_env path in
-    match ty with
-    | None ->
-       let print_function =
-         if is_old_style then
-           (fun _formatter repr -> Obj.obj v (Obj.obj repr))
-         else
-           (fun formatter repr -> Obj.obj v formatter (Obj.obj repr)) in
-       install_printer path ty_arg print_function
-    | Some (ty_path, ty_args) ->
-       let rec build v = function
-         | [] ->
-            let print_function =
-              if is_old_style then
-                (fun _formatter repr -> Obj.obj v (Obj.obj repr))
-              else
-                (fun formatter repr -> Obj.obj v formatter (Obj.obj repr)) in
-            Zero print_function
-         | _ :: args ->
-            Succ
-              (fun fn -> build ((Obj.obj v : _ -> Obj.t) fn) args) in
-       install_generic_printer' path ty_path (build v ty_args)
-  with Exit -> ()
+  match find_printer lid with
+  | Error report ->
+    report ppf
+  | Ok (path, kind) ->
+    install_printer_by_kind path kind
 
 let dir_remove_printer ppf lid =
-  try
-    let (_ty_arg, path, _is_old_style) = find_printer_type ppf lid in
-    begin try
-      remove_printer path
-    with Not_found ->
-      fprintf ppf "No printer named %a.@." Printtyp.longident lid
-    end
-  with Exit -> ()
+  match find_printer lid with
+  | Error report ->
+    report ppf
+  | Ok (path, _kind) ->
+    match remove_installed_printer path with
+    | Ok () -> ()
+    | Error report -> report ppf
 
 let _ = add_directive "install_printer"
     (Directive_ident (with_error_fmt dir_install_printer))
@@ -425,10 +440,11 @@ let is_nonrec_type id td =
           nonrecursive_use:= true
     | _ -> ()
   in
-  let it =  Btype.{type_iterators with it_path } in
   let () =
-    it.it_type_declaration it td;
-    Btype.unmark_iterators.it_type_declaration Btype.unmark_iterators td
+    with_type_mark begin fun mark ->
+      let it = Btype.{(type_iterators mark) with it_path} in
+      it.it_type_declaration it td
+    end
   in
   match !recursive_use, !nonrecursive_use with
   | false, true -> Trec_not
@@ -527,16 +543,15 @@ let is_rec_module id md =
     | Path.Pident id' -> if (Ident.same id id') then raise Exit
     | _ -> ()
   in
-  let it =  Btype.{type_iterators with it_path } in
-  let rs = match it.it_module_declaration it md with
+  with_type_mark begin fun mark ->
+    let it =  Btype.{(type_iterators mark) with it_path} in
+    match it.it_module_declaration it md with
     | () -> Trec_not
     | exception Exit -> Trec_first
-  in
-  Btype.unmark_iterators.it_module_declaration Btype.unmark_iterators md;
-  rs
+  end
 
 let secretly_the_same_path env path1 path2 =
-  let norm path = Printtyp.rewrite_double_underscore_paths env path in
+  let norm path = Out_type.rewrite_double_underscore_paths env path in
   Path.same (norm path1) (norm path2)
 
 let () =
@@ -593,15 +608,13 @@ let () =
 let () =
   reg_show_prim "show_class"
     (fun env loc id lid ->
-       let path, desc_class = Env.lookup_class ~loc lid env in
+       let _path, desc_class = Env.lookup_class ~loc lid env in
        let _path, desc_cltype = Env.lookup_cltype ~loc lid env in
        let _path, typedcl = Env.lookup_type ~loc lid env in
-       let hash_typedcl = Env.find_hash_type path env in
        [
          Sig_class (id, desc_class, Trec_not, Exported);
          Sig_class_type (id, desc_cltype, Trec_not, Exported);
          Sig_type (id, typedcl, Trec_not, Exported);
-         Sig_type (id, hash_typedcl, Trec_not, Exported);
        ]
     )
     "Print the signature of the corresponding class."
@@ -609,13 +622,11 @@ let () =
 let () =
   reg_show_prim "show_class_type"
     (fun env loc id lid ->
-       let path, desc = Env.lookup_cltype ~loc lid env in
+       let _path, desc = Env.lookup_cltype ~loc lid env in
        let _path, typedcl = Env.lookup_type ~loc lid env in
-       let hash_typedcl = Env.find_hash_type path env in
        [
          Sig_class_type (id, desc, Trec_not, Exported);
          Sig_type (id, typedcl, Trec_not, Exported);
-         Sig_type (id, hash_typedcl, Trec_not, Exported);
        ]
     )
     "Print the signature of the corresponding class type."
@@ -654,6 +665,13 @@ let _ = add_directive "print_length"
 
 (* Set various compiler flags *)
 
+let _ = add_directive "debug"
+    (Directive_bool(fun b -> Clflags.debug := b))
+    {
+      section = section_options;
+      doc = "Choose whether to generate debugging events.";
+    }
+
 let _ = add_directive "labels"
     (Directive_bool(fun b -> Clflags.classic := not b))
     {
