@@ -13,7 +13,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-open Format
+open Format_doc
 open Outcometree
 
 exception Ellipsis
@@ -24,6 +24,7 @@ let cautious f ppf arg =
 
 let print_lident ppf = function
   | "::" -> pp_print_string ppf "(::)"
+  | s when Lexer.is_keyword s -> fprintf ppf "\\#%s" s
   | s -> pp_print_string ppf s
 
 let rec print_ident ppf =
@@ -36,32 +37,15 @@ let rec print_ident ppf =
 
 let out_ident = ref print_ident
 
-(* Check a character matches the [identchar_latin1] class from the lexer *)
-let is_ident_char c =
-  match c with
-  | 'A'..'Z' | 'a'..'z' | '_' | '\192'..'\214' | '\216'..'\246'
-  | '\248'..'\255' | '\'' | '0'..'9' -> true
-  | _ -> false
-
-let all_ident_chars s =
-  let rec loop s len i =
-    if i < len then begin
-      if is_ident_char s.[i] then loop s len (i+1)
-      else false
-    end else begin
-      true
-    end
-  in
-  let len = String.length s in
-  loop s len 0
-
 let parenthesized_ident name =
   (List.mem name ["or"; "mod"; "land"; "lor"; "lxor"; "lsl"; "lsr"; "asr"])
-  || not (all_ident_chars name)
+  || not (Misc.Utf8_lexeme.is_valid_identifier name)
 
 let value_ident ppf name =
   if parenthesized_ident name then
     fprintf ppf "( %s )" name
+  else if Lexer.is_keyword name then
+    fprintf ppf "\\#%s" name
   else
     pp_print_string ppf name
 
@@ -153,16 +137,29 @@ let print_out_string ppf s =
   else
     fprintf ppf "%S" s
 
+let print_constr ppf name =
+  match name with
+  | Oide_ident {printed_name = ("true" | "false") as c} ->
+    (* despite being keywords, these are constructor names
+       and should not be escaped *)
+    fprintf ppf "%s" c
+  | Oide_dot (id, ("true"|"false" as s)) ->
+      (* Similarly, M.true is invalid *)
+      fprintf ppf "%a.(%s)" print_ident id s
+  | _ -> print_ident ppf name
+
 let print_out_value ppf tree =
   let rec print_tree_1 ppf =
     function
     | Oval_constr (name, [param]) ->
-        fprintf ppf "@[<1>%a@ %a@]" print_ident name print_constr_param param
+        fprintf ppf "@[<1>%a@ %a@]" print_constr name print_constr_param param
     | Oval_constr (name, (_ :: _ as params)) ->
-        fprintf ppf "@[<1>%a@ (%a)@]" print_ident name
+        fprintf ppf "@[<1>%a@ (%a)@]" print_constr name
           (print_tree_list print_tree_1 ",") params
     | Oval_variant (name, Some param) ->
-        fprintf ppf "@[<2>`%s@ %a@]" name print_constr_param param
+        fprintf ppf "@[<2>`%a@ %a@]" print_lident name print_constr_param param
+    | Oval_lazy param ->
+        fprintf ppf "@[<2>lazy@ %a@]" print_constr_param param
     | tree -> print_simple_tree ppf tree
   and print_constr_param ppf = function
     | Oval_int i -> parenthesize_if_neg ppf "%i" i (i < 0)
@@ -188,6 +185,7 @@ let print_out_value ppf tree =
     | Oval_string (s, maxlen, kind) ->
        begin try
          let len = String.length s in
+         let maxlen = max maxlen 8 in (* always show a little prefix *)
          let s = if len > maxlen then String.sub s 0 maxlen else s in
          begin match kind with
          | Ostr_bytes -> fprintf ppf "Bytes.of_string %S" s
@@ -204,8 +202,8 @@ let print_out_value ppf tree =
         fprintf ppf "@[<1>[%a]@]" (print_tree_list print_tree_1 ";") tl
     | Oval_array tl ->
         fprintf ppf "@[<2>[|%a|]@]" (print_tree_list print_tree_1 ";") tl
-    | Oval_constr (name, []) -> print_ident ppf name
-    | Oval_variant (name, None) -> fprintf ppf "`%s" name
+    | Oval_constr (name, []) -> print_constr ppf name
+    | Oval_variant (name, None) -> fprintf ppf "`%a" print_lident name
     | Oval_stuff s -> pp_print_string ppf s
     | Oval_record fel ->
         fprintf ppf "@[<1>{%a}@]" (cautious (print_fields true)) fel
@@ -235,7 +233,7 @@ let print_out_value ppf tree =
   in
   cautious print_tree_1 ppf tree
 
-let out_value = ref print_out_value
+let out_value = ref (compat print_out_value)
 
 (* Types *)
 
@@ -253,15 +251,25 @@ let rec print_list pr sep ppf =
 let pr_present =
   print_list (fun ppf s -> fprintf ppf "`%s" s) (fun ppf -> fprintf ppf "@ ")
 
-let pr_var = Pprintast.tyvar
+let pr_var = Pprintast.Doc.tyvar
+let ty_var ~non_gen ppf s =
+  pr_var ppf (if non_gen then "_" ^ s else s)
 
 let pr_vars =
   print_list pr_var (fun ppf -> fprintf ppf "@ ")
 
+let print_arg_label ppf (lbl : Asttypes.arg_label) =
+  match lbl with
+  | Nolabel -> ()
+  | Labelled s -> fprintf ppf "%a:" print_lident s
+  | Optional s -> fprintf ppf "?%a:" print_lident s
+
 let rec print_out_type ppf =
   function
-  | Otyp_alias (ty, s) ->
-      fprintf ppf "@[%a@ as %a@]" print_out_type ty pr_var s
+  | Otyp_alias {non_gen; aliased; alias } ->
+      fprintf ppf "@[%a@ as %a@]"
+        print_out_type aliased
+        (ty_var ~non_gen) alias
   | Otyp_poly (sl, ty) ->
       fprintf ppf "@[<hov 2>%a.@ %a@]"
         pr_vars sl
@@ -273,7 +281,7 @@ and print_out_type_1 ppf =
   function
     Otyp_arrow (lab, ty1, ty2) ->
       pp_open_box ppf 0;
-      if lab <> "" then (pp_print_string ppf lab; pp_print_char ppf ':');
+      print_arg_label ppf lab;
       print_out_type_2 ppf ty1;
       pp_print_string ppf " ->";
       pp_print_space ppf ();
@@ -287,19 +295,18 @@ and print_out_type_2 ppf =
   | ty -> print_simple_out_type ppf ty
 and print_simple_out_type ppf =
   function
-    Otyp_class (ng, id, tyl) ->
-      fprintf ppf "@[%a%s#%a@]" print_typargs tyl (if ng then "_" else "")
-        print_ident id
+    Otyp_class (id, tyl) ->
+      fprintf ppf "@[%a#%a@]" print_typargs tyl print_ident id
   | Otyp_constr (id, tyl) ->
       pp_open_box ppf 0;
       print_typargs ppf tyl;
       print_ident ppf id;
       pp_close_box ppf ()
-  | Otyp_object (fields, rest) ->
-      fprintf ppf "@[<2>< %a >@]" (print_fields rest) fields
+  | Otyp_object {fields; open_row} ->
+      fprintf ppf "@[<2>< %a >@]" (print_fields open_row) fields
   | Otyp_stuff s -> pp_print_string ppf s
-  | Otyp_var (ng, s) -> pr_var ppf (if ng then "_" ^ s else s)
-  | Otyp_variant (non_gen, row_fields, closed, tags) ->
+  | Otyp_var (non_gen, s) -> ty_var ~non_gen ppf s
+  | Otyp_variant (row_fields, closed, tags) ->
       let print_present ppf =
         function
           None | Some [] -> ()
@@ -313,8 +320,7 @@ and print_simple_out_type ppf =
         | Ovar_typ typ ->
            print_simple_out_type ppf typ
       in
-      fprintf ppf "%s@[<hov>[%s@[<hv>@[<hv>%a@]%a@]@ ]@]"
-        (if non_gen then "_" else "")
+      fprintf ppf "@[<hov>[%s@[<hv>@[<hv>%a@]%a@]@ ]@]"
         (if closed then if tags = None then " " else "< "
          else if tags = None then "> " else "? ")
         print_fields row_fields
@@ -328,44 +334,39 @@ and print_simple_out_type ppf =
   | Otyp_abstract | Otyp_open
   | Otyp_sum _ | Otyp_manifest (_, _) -> ()
   | Otyp_record lbls -> print_record_decl ppf lbls
-  | Otyp_module (p, n, tyl) ->
+  | Otyp_module (p, fl) ->
       fprintf ppf "@[<1>(module %a" print_ident p;
       let first = ref true in
-      List.iter2
-        (fun s t ->
+      List.iter
+        (fun (s, t) ->
           let sep = if !first then (first := false; "with") else "and" in
           fprintf ppf " %s type %s = %a" sep s print_out_type t
         )
-        n tyl;
+        fl;
       fprintf ppf ")@]"
   | Otyp_attribute (t, attr) ->
       fprintf ppf "@[<1>(%a [@@%s])@]" print_out_type t attr.oattr_name
 and print_record_decl ppf lbls =
   fprintf ppf "{%a@;<1 -2>}"
     (print_list_init print_out_label (fun ppf -> fprintf ppf "@ ")) lbls
-and print_fields rest ppf =
+and print_fields open_row ppf =
   function
     [] ->
-      begin match rest with
-        Some non_gen -> fprintf ppf "%s.." (if non_gen then "_" else "")
-      | None -> ()
-      end
+      if open_row then fprintf ppf "..";
   | [s, t] ->
-      fprintf ppf "%s : %a" s print_out_type t;
-      begin match rest with
-        Some _ -> fprintf ppf ";@ "
-      | None -> ()
-      end;
-      print_fields rest ppf []
+      fprintf ppf "%a : %a" print_lident s print_out_type t;
+      if open_row then fprintf ppf ";@ ";
+      print_fields open_row ppf []
   | (s, t) :: l ->
-      fprintf ppf "%s : %a;@ %a" s print_out_type t (print_fields rest) l
+      fprintf ppf "%s : %a;@ %a" s print_out_type t (print_fields open_row) l
 and print_row_field ppf (l, opt_amp, tyl) =
   let pr_of ppf =
     if opt_amp then fprintf ppf " of@ &@ "
     else if tyl <> [] then fprintf ppf " of@ "
     else fprintf ppf ""
   in
-  fprintf ppf "@[<hv 2>`%s%t%a@]" l pr_of (print_typlist print_out_type " &")
+  fprintf ppf "@[<hv 2>`%a%t%a@]" print_lident l pr_of
+    (print_typlist print_out_type " &")
     tyl
 and print_typlist print_elem sep ppf =
   function
@@ -387,23 +388,31 @@ and print_typargs ppf =
       pp_print_char ppf ')';
       pp_close_box ppf ();
       pp_print_space ppf ()
-and print_out_label ppf (name, mut, arg) =
-  fprintf ppf "@[<2>%s%s :@ %a@];" (if mut then "mutable " else "") name
-    print_out_type arg
+and print_out_label ppf {olab_name; olab_mut; olab_type} =
+  fprintf ppf "@[<2>%s%a :@ %a@];"
+    (match olab_mut with
+     | Mutable -> "mutable "
+     | Immutable -> "")
+    print_lident olab_name
+    print_out_type olab_type
 
 let out_label = ref print_out_label
 
 let out_type = ref print_out_type
 
+let out_type_args = ref print_typargs
+
 (* Class types *)
 
-let print_type_parameter ppf s =
-  if s = "_" then fprintf ppf "_" else pr_var ppf s
+let print_type_parameter ?(non_gen=false) ppf s =
+  if s = "_" then fprintf ppf "_" else ty_var ~non_gen ppf s
 
-let type_parameter ppf (ty, (co, cn)) =
-  fprintf ppf "%s%a"
-    (if not cn then "+" else if not co then "-" else "")
-    print_type_parameter ty
+let type_parameter ppf {ot_non_gen=non_gen; ot_name=ty; ot_variance=var,inj} =
+  let open Asttypes in
+  fprintf ppf "%s%s%a"
+    (match var with Covariant -> "+" | Contravariant -> "-" | NoVariance ->  "")
+    (match inj with Injective -> "!" | NoInjectivity -> "")
+    (print_type_parameter ~non_gen) ty
 
 let print_out_class_params ppf =
   function
@@ -424,7 +433,7 @@ let rec print_out_class_type ppf =
       in
       fprintf ppf "@[%a%a@]" pr_tyl tyl print_ident id
   | Octy_arrow (lab, ty, cty) ->
-      fprintf ppf "@[%s%a ->@ %a@]" (if lab <> "" then lab ^ ":" else "")
+      fprintf ppf "@[%a%a ->@ %a@]" print_arg_label lab
         print_out_type_2 ty print_out_class_type cty
   | Octy_signature (self_ty, csil) ->
       let pr_param ppf =
@@ -441,14 +450,14 @@ and print_out_class_sig_item ppf =
       fprintf ppf "@[<2>constraint %a =@ %a@]" !out_type ty1
         !out_type ty2
   | Ocsg_method (name, priv, virt, ty) ->
-      fprintf ppf "@[<2>method %s%s%s :@ %a@]"
+      fprintf ppf "@[<2>method %s%s%a :@ %a@]"
         (if priv then "private " else "") (if virt then "virtual " else "")
-        name !out_type ty
+        print_lident name !out_type ty
   | Ocsg_value (name, mut, vr, ty) ->
-      fprintf ppf "@[<2>val %s%s%s :@ %a@]"
+      fprintf ppf "@[<2>val %s%s%a :@ %a@]"
         (if mut then "mutable " else "")
         (if vr then "virtual " else "")
-        name !out_type ty
+        print_lident name !out_type ty
 
 let out_class_type = ref print_out_class_type
 
@@ -458,6 +467,8 @@ let out_module_type = ref (fun _ -> failwith "Oprint.out_module_type")
 let out_sig_item = ref (fun _ -> failwith "Oprint.out_sig_item")
 let out_signature = ref (fun _ -> failwith "Oprint.out_signature")
 let out_type_extension = ref (fun _ -> failwith "Oprint.out_type_extension")
+let out_functor_parameters =
+  ref (fun _ -> failwith "Oprint.out_functor_parameters")
 
 (* For anonymous functor arguments, the logic to choose between
    the long-form
@@ -482,50 +493,66 @@ let out_type_extension = ref (fun _ -> failwith "Oprint.out_type_extension")
 (* take a module type that may be a functor type,
    and return the longest prefix list of arguments
    that should be printed in long form. *)
-let collect_functor_arguments mty =
-  let rec collect_args acc = function
-    | Omty_functor (param, mty_res) ->
-       collect_args (param :: acc) mty_res
-    | non_functor -> (acc, non_functor)
-  in
+
+let rec collect_functor_args acc = function
+  | Omty_functor (param, mty_res) ->
+      collect_functor_args (param :: acc) mty_res
+  | non_functor -> (acc, non_functor)
+let collect_functor_args mty =
+  let l, rest = collect_functor_args [] mty in
+  List.rev l, rest
+
+let constructor_of_extension_constructor
+    (ext : out_extension_constructor) : out_constructor
+=
+  {
+    ocstr_name = ext.oext_name;
+    ocstr_args = ext.oext_args;
+    ocstr_return_type = ext.oext_ret_type;
+  }
+
+let split_anon_functor_arguments params =
   let rec uncollect_anonymous_suffix acc rest = match acc with
-      | Some (None, mty_arg) :: acc ->
-          uncollect_anonymous_suffix acc
-            (Omty_functor (Some (None, mty_arg), rest))
-      | _ :: _ | [] ->
-         (acc, rest)
+    | Some (None, mty_arg) :: acc ->
+        uncollect_anonymous_suffix acc
+          (Some (None, mty_arg) :: rest)
+    | _ :: _ | [] ->
+        (acc, rest)
   in
-  let (acc, non_functor) = collect_args [] mty in
-  let (acc, rest) = uncollect_anonymous_suffix acc non_functor in
+  let (acc, rest) = uncollect_anonymous_suffix (List.rev params) [] in
   (List.rev acc, rest)
 
 let rec print_out_module_type ppf mty =
   print_out_functor ppf mty
-and print_out_functor ppf = function
-  | Omty_functor _ as t ->
-     let rec print_functor ppf = function
-       | Omty_functor (Some (None, mty_arg), mty_res) ->
-          fprintf ppf "%a ->@ %a"
-            print_simple_out_module_type mty_arg
-            print_functor mty_res
-       | Omty_functor _ as non_anonymous_functor ->
-          let (args, rest) = collect_functor_arguments non_anonymous_functor in
-          let print_arg ppf = function
-            | None ->
-               fprintf ppf "()"
-            | Some (param, mty) ->
-               fprintf ppf "(%s : %a)"
-                 (Option.value param ~default:"_")
-                 print_out_module_type mty
-          in
-          fprintf ppf "@[<2>functor@ %a@]@ ->@ %a"
-            (pp_print_list ~pp_sep:pp_print_space print_arg) args
-            print_functor rest
-       | non_functor ->
-          print_simple_out_module_type ppf non_functor
-     in
-     fprintf ppf "@[<2>%a@]" print_functor t
-  | t -> print_simple_out_module_type ppf t
+
+and print_out_functor_parameters ppf l =
+  let print_nonanon_arg ppf = function
+    | None ->
+        fprintf ppf "()"
+    | Some (param, mty) ->
+        fprintf ppf "(%s : %a)"
+          (Option.value param ~default:"_")
+          print_out_module_type mty
+  in
+  let rec print_args ppf = function
+    | [] -> ()
+    | Some (None, mty_arg) :: l ->
+        fprintf ppf "%a ->@ %a"
+          print_simple_out_module_type mty_arg
+          print_args l
+    | _ :: _ as non_anonymous_functor ->
+        let args, anons = split_anon_functor_arguments non_anonymous_functor in
+        fprintf ppf "@[%a@]@ ->@ %a"
+          (pp_print_list ~pp_sep:pp_print_space print_nonanon_arg) args
+          print_args anons
+  in
+  print_args ppf l
+
+and print_out_functor ppf t =
+  let params, non_functor = collect_functor_args t in
+  fprintf ppf "@[<2>%a%a@]"
+    print_out_functor_parameters params
+    print_simple_out_module_type non_functor
 and print_simple_out_module_type ppf =
   function
     Omty_abstract -> ()
@@ -549,13 +576,13 @@ and print_out_signature ppf =
         match items with
             Osig_typext(ext, Oext_next) :: items ->
               gather_extensions
-                ((ext.oext_name, ext.oext_args, ext.oext_ret_type) :: acc)
+                (constructor_of_extension_constructor ext :: acc)
                 items
           | _ -> (List.rev acc, items)
       in
       let exts, items =
         gather_extensions
-          [(ext.oext_name, ext.oext_args, ext.oext_ret_type)]
+          [constructor_of_extension_constructor ext]
           items
       in
       let te =
@@ -570,18 +597,18 @@ and print_out_signature ppf =
 and print_out_sig_item ppf =
   function
     Osig_class (vir_flag, name, params, clt, rs) ->
-      fprintf ppf "@[<2>%s%s@ %a%s@ :@ %a@]"
+      fprintf ppf "@[<2>%s%s@ %a%a@ :@ %a@]"
         (if rs = Orec_next then "and" else "class")
         (if vir_flag then " virtual" else "") print_out_class_params params
-        name !out_class_type clt
+        print_lident name !out_class_type clt
   | Osig_class_type (vir_flag, name, params, clt, rs) ->
-      fprintf ppf "@[<2>%s%s@ %a%s@ =@ %a@]"
+      fprintf ppf "@[<2>%s%s@ %a%a@ =@ %a@]"
         (if rs = Orec_next then "and" else "class type")
         (if vir_flag then " virtual" else "") print_out_class_params params
-        name !out_class_type clt
+        print_lident name !out_class_type clt
   | Osig_typext (ext, Oext_exception) ->
       fprintf ppf "@[<2>exception %a@]"
-        print_out_constr (ext.oext_name, ext.oext_args, ext.oext_ret_type)
+        print_out_constr (constructor_of_extension_constructor ext)
   | Osig_typext (ext, _es) ->
       print_out_extension_constructor ppf ext
   | Osig_modtype (name, Omty_abstract) ->
@@ -629,13 +656,15 @@ and print_out_type_decl kwd ppf td =
   in
   let type_defined ppf =
     match td.otype_params with
-      [] -> pp_print_string ppf td.otype_name
-    | [param] -> fprintf ppf "@[%a@ %s@]" type_parameter param td.otype_name
+      [] -> print_lident ppf td.otype_name
+    | [param] ->
+        fprintf ppf "@[%a@ %a@]" type_parameter param
+          print_lident td.otype_name
     | _ ->
-        fprintf ppf "@[(@[%a)@]@ %s@]"
+        fprintf ppf "@[(@[%a)@]@ %a@]"
           (print_list type_parameter (fun ppf -> fprintf ppf ",@ "))
           td.otype_params
-          td.otype_name
+          print_lident td.otype_name
   in
   let print_manifest ppf =
     function
@@ -691,13 +720,18 @@ and print_out_type_decl kwd ppf td =
     print_immediate
     print_unboxed
 
-and print_out_constr ppf (name, tyl,ret_type_opt) =
+and print_out_constr ppf constr =
+  let {
+    ocstr_name = name;
+    ocstr_args = tyl;
+    ocstr_return_type = return_type;
+  } = constr in
   let name =
     match name with
     | "::" -> "(::)"   (* #7200 *)
     | s -> s
   in
-  match ret_type_opt with
+  match return_type with
   | None ->
       begin match tyl with
       | [] ->
@@ -719,36 +753,37 @@ and print_out_constr ppf (name, tyl,ret_type_opt) =
 and print_out_extension_constructor ppf ext =
   let print_extended_type ppf =
       match ext.oext_type_params with
-        [] -> fprintf ppf "%s" ext.oext_type_name
+        [] -> fprintf ppf "%a" print_lident ext.oext_type_name
       | [ty_param] ->
-        fprintf ppf "@[%a@ %s@]"
-          print_type_parameter
+        fprintf ppf "@[%a@ %a@]"
+          (print_type_parameter ~non_gen:false)
           ty_param
-          ext.oext_type_name
+          print_lident ext.oext_type_name
       | _ ->
-        fprintf ppf "@[(@[%a)@]@ %s@]"
+        fprintf ppf "@[(@[%a)@]@ %a@]"
           (print_list print_type_parameter (fun ppf -> fprintf ppf ",@ "))
           ext.oext_type_params
-          ext.oext_type_name
+          print_lident ext.oext_type_name
   in
   fprintf ppf "@[<hv 2>type %t +=%s@;<1 2>%a@]"
     print_extended_type
     (if ext.oext_private = Asttypes.Private then " private" else "")
-    print_out_constr (ext.oext_name, ext.oext_args, ext.oext_ret_type)
+    print_out_constr
+    (constructor_of_extension_constructor ext)
 
 and print_out_type_extension ppf te =
   let print_extended_type ppf =
     match te.otyext_params with
-      [] -> fprintf ppf "%s" te.otyext_name
+      [] -> fprintf ppf "%a" print_lident te.otyext_name
     | [param] ->
-      fprintf ppf "@[%a@ %s@]"
-        print_type_parameter param
-        te.otyext_name
+      fprintf ppf "@[%a@ %a@]"
+        (print_type_parameter ~non_gen:false) param
+        print_lident te.otyext_name
     | _ ->
-        fprintf ppf "@[(@[%a)@]@ %s@]"
+        fprintf ppf "@[(@[%a)@]@ %a@]"
           (print_list print_type_parameter (fun ppf -> fprintf ppf ",@ "))
           te.otyext_params
-          te.otyext_name
+          print_lident te.otyext_name
   in
   fprintf ppf "@[<hv 2>type %t +=%s@;<1 2>%a@]"
     print_extended_type
@@ -761,9 +796,12 @@ let _ = out_module_type := print_out_module_type
 let _ = out_signature := print_out_signature
 let _ = out_sig_item := print_out_sig_item
 let _ = out_type_extension := print_out_type_extension
+let _ = out_functor_parameters := print_out_functor_parameters
 
 (* Phrases *)
 
+open Format
+
 let print_out_exception ppf exn outv =
   match exn with
     Sys.Break -> fprintf ppf "Interrupted.@."
@@ -783,13 +821,13 @@ let rec print_items ppf =
         match items with
             (Osig_typext(ext, Oext_next), None) :: items ->
               gather_extensions
-                ((ext.oext_name, ext.oext_args, ext.oext_ret_type) :: acc)
+                (constructor_of_extension_constructor ext :: acc)
                 items
           | _ -> (List.rev acc, items)
       in
       let exts, items =
         gather_extensions
-          [(ext.oext_name, ext.oext_args, ext.oext_ret_type)]
+          [constructor_of_extension_constructor ext]
           items
       in
       let te =
@@ -798,23 +836,26 @@ let rec print_items ppf =
           otyext_constructors = exts;
           otyext_private = ext.oext_private }
       in
-        fprintf ppf "@[%a@]" !out_type_extension te;
+        fprintf ppf "@[%a@]" (Format_doc.compat !out_type_extension) te;
         if items <> [] then fprintf ppf "@ %a" print_items items
   | (tree, valopt) :: items ->
       begin match valopt with
         Some v ->
-          fprintf ppf "@[<2>%a =@ %a@]" !out_sig_item tree
+          fprintf ppf "@[<2>%a =@ %a@]" (Format_doc.compat !out_sig_item) tree
             !out_value v
-      | None -> fprintf ppf "@[%a@]" !out_sig_item tree
+      | None -> fprintf ppf "@[%a@]" (Format_doc.compat !out_sig_item) tree
       end;
       if items <> [] then fprintf ppf "@ %a" print_items items
 
 let print_out_phrase ppf =
   function
     Ophr_eval (outv, ty) ->
-      fprintf ppf "@[- : %a@ =@ %a@]@." !out_type ty !out_value outv
+      fprintf ppf "@[- : %a@ =@ %a@]@." (compat !out_type) ty !out_value outv
   | Ophr_signature [] -> ()
   | Ophr_signature items -> fprintf ppf "@[<v>%a@]@." print_items items
   | Ophr_exception (exn, outv) -> print_out_exception ppf exn outv
 
 let out_phrase = ref print_out_phrase
+
+type 'a printer = 'a Format_doc.printer ref
+type 'a toplevel_printer = (Format.formatter -> 'a -> unit) ref
