@@ -18,7 +18,7 @@
 open Asttypes
 open Typedtree
 open Types
-open Format
+open Format_doc
 
 let is_cons = function
 | {cstr_name = "::"} -> true
@@ -52,21 +52,27 @@ let rec pretty_val : type k . _ -> k general_pattern -> _ = fun ppf v ->
     | [] ->
   match v.pat_desc with
   | Tpat_any -> fprintf ppf "_"
-  | Tpat_var (x,_) -> fprintf ppf "%s" (Ident.name x)
+  | Tpat_var (x,_,_) -> fprintf ppf "%s" (Ident.name x)
   | Tpat_constant c -> fprintf ppf "%s" (pretty_const c)
   | Tpat_tuple vs ->
       fprintf ppf "@[(%a)@]" (pretty_vals ",") vs
-  | Tpat_construct (_, cstr, []) ->
+  | Tpat_construct (_, cstr, [], _) ->
       fprintf ppf "%s" cstr.cstr_name
-  | Tpat_construct (_, cstr, [w]) ->
+  | Tpat_construct (_, cstr, [w], None) ->
       fprintf ppf "@[<2>%s@ %a@]" cstr.cstr_name pretty_arg w
-  | Tpat_construct (_, cstr, vs) ->
+  | Tpat_construct (_, cstr, vs, vto) ->
       let name = cstr.cstr_name in
-      begin match (name, vs) with
-        ("::", [v1;v2]) ->
+      begin match (name, vs, vto) with
+        ("::", [v1;v2], None) ->
           fprintf ppf "@[%a::@,%a@]" pretty_car v1 pretty_cdr v2
-      |  _ ->
+      | (_, _, None) ->
           fprintf ppf "@[<2>%s@ @[(%a)@]@]" name (pretty_vals ",") vs
+      | (_, _, Some ([], _t)) ->
+          fprintf ppf "@[<2>%s@ @[(%a : _)@]@]" name (pretty_vals ",") vs
+      | (_, _, Some (vl, _t)) ->
+          let vars = List.map (fun x -> Ident.name x.txt) vl in
+          fprintf ppf "@[<2>%s@ (type %s)@ @[(%a : _)@]@]"
+            name (String.concat " " vars) (pretty_vals ",") vs
       end
   | Tpat_variant (l, None, _) ->
       fprintf ppf "`%s" l
@@ -78,7 +84,7 @@ let rec pretty_val : type k . _ -> k general_pattern -> _ = fun ppf v ->
             | (_,_,{pat_desc=Tpat_any}) -> false (* do not show lbl=_ *)
             | _ -> true) lvs in
       begin match filtered_lvs with
-      | [] -> fprintf ppf "_"
+      | [] -> fprintf ppf "{ _ }"
       | (_, lbl, _) :: q ->
           let elision_mark ppf =
             (* we assume that there is no label repetitions here *)
@@ -92,8 +98,8 @@ let rec pretty_val : type k . _ -> k general_pattern -> _ = fun ppf v ->
       fprintf ppf "@[[| %a |]@]" (pretty_vals " ;") vs
   | Tpat_lazy v ->
       fprintf ppf "@[<2>lazy@ %a@]" pretty_arg v
-  | Tpat_alias (v, x,_) ->
-      fprintf ppf "@[(%a@ as %a)@]" pretty_val v Ident.print x
+  | Tpat_alias (v, x,_,_) ->
+      fprintf ppf "@[(%a@ as %a)@]" pretty_val v Ident.doc_print x
   | Tpat_value v ->
       fprintf ppf "%a" pretty_val (v :> pattern)
   | Tpat_exception v ->
@@ -102,19 +108,19 @@ let rec pretty_val : type k . _ -> k general_pattern -> _ = fun ppf v ->
       fprintf ppf "@[(%a)@]" pretty_or v
 
 and pretty_car ppf v = match v.pat_desc with
-| Tpat_construct (_,cstr, [_ ; _])
+| Tpat_construct (_,cstr, [_ ; _], None)
     when is_cons cstr ->
       fprintf ppf "(%a)" pretty_val v
 | _ -> pretty_val ppf v
 
 and pretty_cdr ppf v = match v.pat_desc with
-| Tpat_construct (_,cstr, [v1 ; v2])
+| Tpat_construct (_,cstr, [v1 ; v2], None)
     when is_cons cstr ->
       fprintf ppf "%a::@,%a" pretty_car v1 pretty_cdr v2
 | _ -> pretty_val ppf v
 
 and pretty_arg ppf v = match v.pat_desc with
-| Tpat_construct (_,_,_::_)
+| Tpat_construct (_,_,_::_,None)
 | Tpat_variant (_, Some _, _) -> fprintf ppf "(%a)" pretty_val v
 |  _ -> pretty_val ppf v
 
@@ -139,25 +145,29 @@ and pretty_lvals ppf = function
         lbl.lbl_name pretty_val v pretty_lvals rest
 
 let top_pretty ppf v =
-  fprintf ppf "@[%a@]@?" pretty_val v
+  fprintf ppf "@[%a@]" pretty_val v
 
-let pretty_pat p =
-  top_pretty Format.str_formatter p ;
-  prerr_string (Format.flush_str_formatter ())
+let pretty_pat ppf p =
+  top_pretty ppf p ;
+  pp_print_flush ppf ()
 
 type 'k matrix = 'k general_pattern list list
 
-let pretty_line fmt =
+let pretty_line ppf line =
+  fprintf ppf "@[";
   List.iter (fun p ->
-    Format.fprintf fmt " <";
-    top_pretty fmt p;
-    Format.fprintf fmt ">";
-  )
+      fprintf ppf "<%a>@ "
+        pretty_val p
+    ) line;
+  fprintf ppf "@]"
 
-let pretty_matrix fmt (pss : 'k matrix) =
-  Format.fprintf fmt "begin matrix\n" ;
-  List.iter (fun ps ->
-    pretty_line fmt ps ;
-    Format.fprintf fmt "\n"
-  ) pss;
-  Format.fprintf fmt "end matrix\n%!"
+let pretty_matrix ppf (pss : 'k matrix) =
+  fprintf ppf "@[<v 2>  %a@]"
+    (pp_print_list ~pp_sep:pp_print_cut pretty_line)
+    pss
+
+module Compat = struct
+  let pretty_pat ppf x = compat pretty_pat ppf x
+  let pretty_line ppf x = compat pretty_line ppf x
+  let pretty_matrix ppf x = compat pretty_matrix ppf x
+end
